open Types
open Content
open Triplet
open Group
open Boundingbox
open Viewport

class box (col:couleur) (position:point) (wire:draw) (cont: container option) (measureVal:point) =
  object(self)
    inherit content col position wire cont as super
    
    (*Members*)
    val mutable measure = measureVal

    (*Methods*)
    (*-------*)
    method setMeasure newMeasure = measure <- newMeasure
    method getMeasure = measure


    (*Draws this primitive after the content has been drawn by the parent object*)
    method drawSpecific sector drawInt = let x = measure#getX and y = measure#getY and z = measure#getZ in
					   let drawSol drawInt = normeSens (0., -1., 0.) drawInt;
					     GlDraw.vertex3 (  x, -.y,  z);
					     GlDraw.vertex3 (-.x, -.y,  z);
					     GlDraw.vertex3 (-.x, -.y,-.z);
					     GlDraw.vertex3 (  x, -.y,-.z)

					   and drawToit drawInt = normeSens (0., 1., 0.) drawInt;
					                          GlDraw.vertex3 (  x, y,  z);
								  GlDraw.vertex3 (  x, y,-.z);
								  GlDraw.vertex3 (-.x, y,-.z);
								  GlDraw.vertex3 (-.x, y,  z)
								    
					   and drawMurs y1 y2 drawInt = let un = if drawInt
					                                         then -1.
					                                         else  1.
					     in
					       GlDraw.normal3 (  un, 0.,  0.);
					       GlDraw.vertex3 (   x, y2,   z);
					       GlDraw.vertex3 (   x, y1,   z);
					       GlDraw.vertex3 (   x, y1,-. z);
					       GlDraw.vertex3 (   x, y2,-. z);
					       
					       GlDraw.normal3 (-.un, 0.,  0.);
					       GlDraw.vertex3 (-. x, y2,   z);
					       GlDraw.vertex3 (-. x, y2,-. z);
					       GlDraw.vertex3 (-. x, y1,-. z);
					       GlDraw.vertex3 (-. x, y1,   z);
					       
					       GlDraw.normal3 (  0., 0.,  un);
					       GlDraw.vertex3 (   x, y2,   z);
					       GlDraw.vertex3 (-. x, y2,   z);
					       GlDraw.vertex3 (-. x, y1,   z);
					       GlDraw.vertex3 (   x, y1,   z);
					       
					       GlDraw.normal3 (  0., 0.,-.un);
					       GlDraw.vertex3 (   x, y2,-. z);
					       GlDraw.vertex3 (   x, y1,-. z);
					       GlDraw.vertex3 (-. x, y1,-. z);
					       GlDraw.vertex3 (-. x, y2,-. z)
					       in
					         let sbt = match sector with
						   | Bottom -> (fun drawInt -> drawSol drawInt)

						   | Inf    -> (fun drawInt -> drawSol drawInt;
							       drawMurs (-.y) 0. drawInt)

						   | Sup    -> (fun drawInt -> drawToit drawInt;
							       drawMurs 0. y drawInt)

						   | All    -> (fun drawInt -> drawSol drawInt;
							       drawToit drawInt;
							       drawMurs (-.y) y drawInt)
						   in
						     self#setGLColor; 
						     GlDraw.begins `quads;
						     sbt false;
						     GlDraw.ends ();
						     if drawInt
						     then
						       begin
							 GlDraw.front_face `cw;
							 GlDraw.begins `quads;
							 sbt true;
							 GlDraw.ends ();
							 GlDraw.front_face `ccw
						       end

    method getBoundingBox () = let (minX, minY, minZ, maxX, maxY, maxZ), zoom, trans = self#getContainerBounds
                               in
                                 if container = None
				 then
				   begin
				     if measure#isDefault
				     then
				       measure#setAll (1.0, 1.0, 1.0)
				   end
				 else
				   begin
				     if measure#isDefault
				     then
				       measure#setAll ((maxX -. minX) /. 2.0, (maxY -. minY) /. 2.0, (maxZ -. minZ) /. 2.0)
				     else
				       begin
					 let minR = min (min (abs_float ((2.0 *. measure#getX) /. (maxX -. minX)))
							   (abs_float ((2.0 *. measure#getY) /. (maxY -. minY))))
						      (abs_float ((2.0 *. measure#getZ) /. (maxZ -. minZ)))
					 in
					   if minR < 1.0
					   then
					     zoom := minR
				       end
				   end;

                                   (-.measure#getX, -.measure#getY, -.measure#getZ,
				    measure#getX,   measure#getY,   measure#getZ)
				   
				   
  end
