open Types
open Parametres
open Content
open Viewport
open Frustrum
open Disc
open Group

class cone (col:couleur) (position:point) (wire:draw) (cont: container option) (radiusVal:float) (heightVal : float) (slicesVal:int) (solidVal:bool)=
  object (self)
    inherit content col position wire cont as super

    (*Members*)
    val mutable radius = radiusVal
    val mutable height = heightVal
    val mutable slices = slicesVal
    val mutable solid = solidVal

    (*Methods*)
    (*-------*)

    (*Setter & Getter*)
    method setRadius newRadius = radius <- newRadius
    method getRadius = radius

    method setHeight newHeight = height <-newHeight
    method getHeight = height
  
    method setSlices newSlices = slices <- newSlices
    method getSlice = slices

    method setSolid newSolid = solid <- newSolid
    method getSolid = solid

    (***********************************************************)
    (* Dessine un cne en fonction de la composante sur Y de   *)
    (* (down) et du sommet (up) la base                        *)
    (* drawInt indique si l'intrieur est galement dessin.   *)
    (***********************************************************)
    (* Type : float -> float -> bool -> unit                   *)
    (***********************************************************)
    method drawCone down up drawInt = let height = up -. down
                                      in
                                        let cosSlice = Array.make slices 0.0
					and sinSlice = Array.make slices 0.0
					and cst = 2.0 *. (cos (pi /. (float_of_int slices)))
					and yNormale = 1.0 /. (sqrt (1.0 +. 4.0*.height*.height/.(radius*.radius)))
					in
					  let cst2 = 2.0 *. height *. yNormale /. radius
					  in
					    let triangle i1 i2 drawInt = let normale sinT cosT = normeSens (cst2 *. cosT, yNormale, cst2 *. sinT) drawInt
					                                 in
									   normale sinSlice.(i1) cosSlice.(i1);
					                                   GlDraw.vertex3 (radius *. cosSlice.(i1), down, radius *. sinSlice.(i1));
									   normale ((sinSlice.(i1) +. sinSlice.(i2)) /. cst) ((cosSlice.(i1) +. cosSlice.(i2)) /. cst);
									   GlDraw.vertex3 (0.0, up, 0.0);
									   normale sinSlice.(i2) cosSlice.(i2);
									   GlDraw.vertex3 (radius *. cosSlice.(i2), down, radius *. sinSlice.(i2))
					    in
					      for j = 0  to (slices - 1)
					      do
						let phi = (pi2 *. (float_of_int j)) /. (float_of_int slices)
						in
						  cosSlice.(j) <- cos phi;
						  sinSlice.(j) <- sin phi
					      done;

					      let cote drawInt = GlDraw.begins `triangles;
						for j = 0 to (slices-2)
						do
						  triangle j (j+1) drawInt
						done;
						triangle (slices-1) 0 drawInt;
						GlDraw.ends ()

					      and bas drawInt = GlDraw.begins `triangle_fan;
						                normeSens (0.0, -1.0, 0.0) drawInt;
								GlDraw.vertex3 (0.0, down, 0.0);
								for j = 0 to (slices - 1)
								do
								  GlDraw.vertex3 (radius *. cosSlice.(j), down, radius *. sinSlice.(j))
								done;
								GlDraw.vertex3 (radius, down, 0.0);
								GlDraw.ends ()
					      in

					        let sbt = match solid
						          with
							    |  true -> (fun drawInt -> bas drawInt;
								                       cote drawInt)

							    | false -> (fun drawInt -> cote drawInt)
						in
						  sbt false;
						  if drawInt
						  then
						    begin
						      GlDraw.front_face `cw;
						      sbt true;
						      GlDraw.front_face `ccw
						    end

    (***************************************************************)
    (* Dessine un cne en fonction de la demi-hauteur (height)     *)
    (* secteur dtermine quel morceau de sphre il faut dessiner : *)
    (*   - Inf    -> 1/2 sphre infrieur                          *)
    (*   - Sup    -> 1/2 sphre suprieur                          *)
    (*   - All    -> sphre entire                                *)
    (*   - Bottom -> Ple Sud                                      *)
    (* drawInt indique si l'intrieur est galement dessin.       *)
    (***************************************************************)
    (* Type : Types.cont_mode -> bool -> unit                      *)
    (***************************************************************)
    method drawSpecific secteur drawInt = match secteur
                                          with
					    | Bottom -> GlMat.push ();
					                GlMat.translate3 (0., -.height, 0.);
							let d = (new disc col pos self#getWire radius slices) in d#drawSelf ();
							GlMat.pop ()
					    | Inf    -> let f = (new frustrum col pos Close None radius height 1.0 slices solid) in
						        f#drawFrustrum (-.height) 0. 0.5 solid false drawInt
					    | Sup    -> self#drawCone 0. height drawInt
					    | All    -> self#drawCone (-.height) height drawInt

    method getBoundingBox () = let rad = ref radius and hei = ref height 
                               and coeffRayonCont cotes = cos (Parametres.pi /. (float_of_int cotes))
                               and (minX, minY, minZ, maxX, maxY, maxZ), zoom, trans = self#getContainerBounds
                               in
                                 if container = None
				 then
				   begin
				     if !rad == nan
				     then
				       rad := 1.0;
				     if !hei == nan
				     then
				       hei := 1.0
				   end
				 else
				   begin
				     let coeff = coeffRayonCont slices
				     and diag  = 0.5 *. (sqrt (let dX, dZ = maxX -. minX, maxZ -. minZ
							       in
								 dX*.dX +. dZ*.dZ))
				     and haut  = 0.5 *. (maxY -. minY)
				     in
				     let zoomOfR_H r h = min 1.0 (1.0 /. ((haut/. (!hei)) +. (diag/.((!rad)*.coeff))))
				       in
				       if !rad == nan
				       then
					 begin
					   if !hei == nan
					   then
					     begin
					       rad := (diag /. coeff) +. haut;
					       hei := !rad
					     end
					   else
					     begin
					       if !hei > haut
					       then
						 rad := diag /. (coeff *. (1.0 -. (haut /. !hei)))
					       else
						 begin
						   rad    := !hei;
						   zoom := zoomOfR_H !rad !hei
						 end
					     end
					 end
				       else
					 begin
					   if !hei == nan
					   then
					     begin
					       if !rad > (diag /. coeff)
					       then
						 hei := haut /. (1.0 -. (diag /. (!rad *. coeff)))
					       else
						 begin
						   hei    := !rad;
						   zoom := zoomOfR_H !rad !hei
						 end
					     end
					   else
					     zoom := zoomOfR_H !rad !hei
					 end;
				       
				       trans#updateY (-. !hei *. diag /. (!rad*.coeff))
				   end;
      
                                   (-. !rad, -. !hei, -. !rad,
				    !rad,    !hei,    !rad)
  end
