(******************************************************************************)
(*                                                                            *)
(* Le fichier coupe_mediane.ml permet de dimminer le nombre de couleurs d'une *)
(* image  256. L'algorithme des coupes mdianes est appliqu  l'image.      *)
(* Cela sert essentiellement pour sauvegarder des images au format GIF qui ne *)
(* supporte pas plus de 256 couleurs.                                         *)
(*                                                                            *)
(******************************************************************************)



(******************************************************************)
(* Indique sur quelle composante la coupe mdiane sera effectue. *)
(******************************************************************)
type colorCompo =
  | Rouge
  | Vert
  | Bleu



(************************************************************************)
(* Un lment de type 'space' correspond  une portion de l'espace RVB  *)
(* comprise entre les valeurs (minR, minV, minB) et (maxR, maxV, maxB). *)
(* 'avg' reprsente la valeur moyenne de tous les pixels de l'image     *)
(* inclus dans cet cette portion.                                       *)
(* La mthode 'size' renvoie le carr de la distance entre 2 coins de   *)
(* l'espace.                                                            *)
(************************************************************************)
class space ((minR, minV, minB):int * int * int) ((maxR, maxV, maxB):int * int * int) (avg:int * int * int) =
object

  val taille = let r = maxR - minR
	       and v = maxV - minV
	       and b = maxB - minB
	       in
		 r*r + v*v + b*b

  method size    = taille
  method min     = (minR, minV, minB)
  method max     = (maxR, maxV, maxB)
  method moyenne = avg

end





(***********************************************************)
(* Insre les espaces de la liste lstInsert dans lstSpace. *)
(* lstSpace, lstInsert et la liste de retour sont ranges  *)
(* suivant le paramter 'size' dcroissant.                *)
(***********************************************************)
(* Type : space * space -> space list -> space list        *)
(***********************************************************)
let insertOrdonne lstInsert lstSpace = let rec rempile l_origin l_dest = match l_origin
									 with
									   | []   -> l_dest
									   | t::q -> rempile q (t::l_dest)
				       in
				       let rec depile l_origin l_dest l_insert = match l_insert
										 with
										   | []            -> rempile l_dest l_origin
										   | spc::q_insert -> match l_origin
												      with
													| []          -> depile l_origin (spc::l_dest) q_insert
													| t::q_origin -> if t#size > spc#size
															 then
															   depile q_origin (  t::l_dest) l_insert
															 else
															   depile l_origin (spc::l_dest) q_insert
				       in
					 depile lstSpace [] lstInsert



(**************************************************************)
(* Divise l'espace spc de l'image img en deux.                *)
(* La limite entre lse 2 espaces est le plan coupant l'espace *)
(* sur sa plus grande composante et passant par la valeur     *)
(* valeur moyenne suivant cette mme composante.              *)
(**************************************************************)
(* Type : space -> Rbg24.t list -> space * space              *)
(**************************************************************)
let split lstImg spc = let (minR, minV, minB) = spc#min
		       and (maxR, maxV, maxB) = spc#max
		       and (avgR, avgV, avgB) = spc#moyenne
		       in
		       (* On dtermine le plan mdian : on fixe sa normale ... *)
		       let compo = let deltaR = maxR - minR
				   and deltaV = maxV - minV
				   and deltaB = maxB - minB
				   in
				     if (deltaR >= deltaV) && (deltaR >= deltaB)
				     then Rouge
				     else
				       begin
					 if (deltaV >= deltaB)
					 then Vert
					 else Bleu
				       end
		       in
		       (* ... et sa position sur cet axe. *)
		       let separation = match compo
					with
					  | Rouge -> let (moy, _, _) = spc#moyenne
						     in
						       moy
					  | Vert  -> let (_, moy, _) = spc#moyenne
						    in
						      moy
					  | Bleu  -> let (_, _, moy) = spc#moyenne
						    in
						      moy
		       in

		       let ((min1, max1, avg1),
			    (min2, max2, avg2)) = (* Fonction permettant de parcourir tous les pixels de l'image *)
						  let rec forAllPxl img
								    u v
								    n1 sum1 min1 max1
								    n2 sum2 min2 max2 = if u = img.Rgb24.width
											then
											  (* On est arriv en fin de ligne *)
											  begin
											    if v = img.Rgb24.height-1
											    then
											      (* L'image a t entirement parcourue *)
											      begin
												n1, sum1, min1, max1,
												n2, sum2, min2, max2
											      end
											    else
											      (* On entame une nouvelle ligne *)
											      forAllPxl img
													0 (v+1) n1 sum1 min1 max1
														n2 sum2 min2 max2
											  end
											else
											  begin
											    let pxl = Rgb24.get img u v
											    in
											    let (r, g, b) = (pxl.Color.r, pxl.Color.g, pxl.Color.b)
											    in
											      if (r >= minR) &&
												 (r <= maxR) &&
												 (g >= minV) &&
												 (g <= maxV) &&
												 (b >= minB) &&
												 (b <= maxB)
											      then
												(* Le pixel est dans l'espace RVB sur lequel s'effectue la coupe mdiane *)
												begin
												  if (match compo
												      with
													| Rouge -> r
													| Vert  -> g
													| Bleu  -> b
												     ) <= separation
												  then
												    (* Le pixel appartient au premier sous espace *)
												    begin
												      let (sumR1, sumV1, sumB1) = sum1
												      and (minR1, minV1, minB1) = min1
												      and (maxR1, maxV1, maxB1) = max1
												      in
													forAllPxl img
														  (u+1) v
														  (n1+1) (sumR1+r, sumV1+g, sumB1+b) (min minR1 r, min minV1 g, min minB1 b)
																		     (max maxR1 r, max maxV1 g, max maxB1 b)
														  n2 sum2 min2 max2
												    end
												  else
												    (* Le pixel appartient au second sous espace *)
												    begin
												      let (sumR2, sumV2, sumB2) = sum2
												      and (minR2, minV2, minB2) = min2
												      and (maxR2, maxV2, maxB2) = max2
												      in
													forAllPxl img
														  (u+1) v
														  n1 sum1 min1 max1
														  (n2+1) (sumR2+r, sumV2+g, sumB2+b) (min minR2 r, min minV2 g, min minB2 b)
																		     (max maxR2 r, max maxV2 g, max maxB2 b)
												    end
												end
											      else
												(* Le pixel est hors de l'espace RVB sur lequel s'effectue la coupe mdiane *)
												forAllPxl img
													  (u+1) v
													  n1 sum1 min1 max1
													  n2 sum2 min2 max2
											  end
						  in

						  let rec forAllImg lst n1 sum1 min1 max1
									n2 sum2 min2 max2 = match lst
											    with
											      | []   -> let (sumR1, sumV1, sumB1) = sum1
													and (sumR2, sumV2, sumB2) = sum2
													in
													  (min1, max1, (sumR1/n1, sumV1/n1, sumB1/n1)),
													  (min2, max2, (sumR2/n2, sumV2/n2, sumB2/n2))

											      | t::q -> let (n1, sum1, min1, max1,
													     n2, sum2, min2, max2) = forAllPxl t 0 0
																	       n1 sum1 min1 max1
																	       n2 sum2 min2 max2
													in
													  forAllImg q n1 sum1 min1 max1
														      n2 sum2 min2 max2
						  in

						    forAllImg lstImg
							      0 (0, 0, 0) (255, 255, 255) (0, 0, 0)
							      0 (0, 0, 0) (255, 255, 255) (0, 0, 0)
		    in

		      (new space min1 max1 avg1), (new space min2 max2 avg2)



(***********************************************)
(* Calule une palette de 256 couleurs pour la  *)
(* liste d'images lstImg.                      *)
(***********************************************)
(* Type : Rbg24.t list -> Color.rgb Color.map  *)
(***********************************************)
let getPalette lstImg = (* On dtermine l'espace englobant tous les pixels de l'image. *)
			let (mini, maxi, avg) = let rec forAllPxl img u v n sum mini maxi = if u = img.Rgb24.width
											    then
											      begin
												if v = img.Rgb24.height-1
												then
												  begin
												    let (sumR, sumV, sumB) = sum
												    in
												      n, sum, mini, maxi
												  end
												else
												  forAllPxl img 0 (v+1) n sum mini maxi
											      end
											    else
											      begin
												let pxl = Rgb24.get img u v
												in
												let (r, g, b) = (pxl.Color.r, pxl.Color.g, pxl.Color.b)
												in
												let (sumR, sumV, sumB) = sum
												and (minR, minV, minB) = mini
												and (maxR, maxV, maxB) = maxi
												in
												  forAllPxl img
													    (u+1) v
													    (n+1) (sumR+r, sumV+v, sumB+b) (min minR r, min minV g, min minB b)
																	   (max maxR r, max maxV g, max maxB b)
											      end
						in

						let rec forAllImg lst n sum mini maxi = match lst
											with
											  | []   -> let (sumR, sumV, sumB) = sum
												    in
												      (mini, maxi, (sumR/n, sumV/n, sumB/n))

											  | t::q -> let (n, sum, mini, maxi) = forAllPxl t 0 0 n sum mini maxi
												    in
												      forAllImg q n sum mini maxi

						in
						  forAllImg lstImg 0 (0, 0, 0) (255, 255, 255) (0, 0, 0)
			in

			(* Liste des espaces class suivant leur compossante 'size' dcroissante *)
			let lstSpace = ref [new space mini maxi avg]
			(* Nombre d'espaces dtermins *)
			and nbrColor = ref 1
			in
			  (* On coupe le plus grand espace jusqu' qu'ils soient au nombre de 256 ou qu'ils soient tous composs d'une seule couleur [size=0] *)
			  while ((List.hd !lstSpace)#size <> 0 && !nbrColor < 256)
			  do
			    match !lstSpace
			    with
			      | []   -> (* Arrivera jamais :@ *)
					()
			      | t::q -> let (space1, space2) = split lstImg t
					in
					  lstSpace := insertOrdonne (if space1#size >= space2#size
								     then [space1 ; space2]
								     else [space2 ; space1]
								    ) q;
					  incr nbrColor
			  done;

			  let palette = ({ Color.max = !nbrColor ;
					   Color.map = Array.make !nbrColor { Color.r = 0 ; Color.g = 0 ; Color.b = 0 }
					 }:(Color.rgb Color.map)
					)
			  in
			  (* On entre les valeurs moyennes de chaque espace dans la palette *)
			  let rec myIter l i = match l
					       with
						 | []   -> ()
						 | t::q -> let (r, g, b) = t#moyenne
							   in
							     palette.Color.map.(i) <- { Color.r = r ; Color.g = g ; Color.b = b };
							     myIter q (i+1)
			  in
			    myIter !lstSpace 0;
			    palette



(*********************************************)
(* Dtermine un image en 256 couleurs de img *)
(*********************************************)
(* Type : Rbg24.t -> Index8.t                *)
(*********************************************)
let coupeMediane img = let palette = getPalette [img]
		       in
			 (* Dtermine la nouvelle image en utilisant la palette *)
			 Reduce.error_diffuse img palette
