open Types
open Parametres
open Triplet
open Viewport
open Core



let  nomRegexp = Str.regexp "%nom"
let    hRegexp = Str.regexp "%h"
let    wRegexp = Str.regexp "%w"
let  fpsRegexp = Str.regexp "%fps"
let listRegexp = Str.regexp "%list"



let vireTimout () = (match core#getTimer
		     with
		       | None   -> ()
		       | Some t -> GMain.Timeout.remove t;
				   core#setTimer None;
		    );
		    core#setState Stop;
		    core#setDir Avant;
		    !Parametres.ecrireStatut "Enregistrement";
		    core#doStop ()



let tmpFilesName pathTmp = (* Insre l'entier i dans la liste croissante lst *)
			   let rec insertOrdonne i lst = match lst 
							 with
							   | []   -> [i]
							   | t::q -> if t<i
								     then t::(insertOrdonne i q)
								     else if t=i
									  then lst
									  else i::lst
			   in

			   let rec listDir dir = try
						   let file = Unix.readdir dir
						   in
						     try
						       let i = Scanf.sscanf file "imo_%d%s" (fun i s -> i)
						       in
							 insertOrdonne i (listDir dir)
						     with
						       | _ -> listDir dir
						 with
						   | End_of_file -> []
			   in

			   let rec firstOut lst i = match lst
						    with
						      | []   -> i
						      | t::q -> if t=i
								then
								  firstOut q (i+1)
								else
								  i
			   in

			   let dir = Unix.opendir pathTmp
			   in
			   let i = firstOut (listDir dir) 0
			   in

			     Filename.concat pathTmp (Printf.sprintf "imo_%d" i)



let rec move mvt deltaT saveFun = let amplitude vitesse duree limite = min_triplet (mult_triplet (vitesse)#getAll duree) (limite)#getAll
				  in
				    (* Si le mouvement n'est pas fini, on continue *)
				    if ((mvt#transDelta)#getAll <> (0.0, 0.0, 0.0)) ||
				       ((mvt#  rotDelta)#getAll <> (0.0, 0.0, 0.0)) ||
				       ((mvt# spinDelta)#getAll <> (0.0, 0.0, 0.0))
				    then
				      begin
					(* Calcule des amplitudes de mouvement  raliser pour chaque type de mouvement *)
					let trans = amplitude mvt#transSpeed deltaT mvt#transDelta
					and rot   = amplitude mvt#  rotSpeed deltaT mvt#  rotDelta
					and spin  = amplitude mvt# spinSpeed deltaT mvt# spinDelta
					in
					  (* Mise  jour des amplitudes restant  raliser *)
					  (mvt#transDelta)#setAll (diff_triplet (mvt#transDelta)#getAll trans);
					  (mvt#  rotDelta)#setAll (diff_triplet (mvt#  rotDelta)#getAll rot  );
					  (mvt# spinDelta)#setAll (diff_triplet (mvt# spinDelta)#getAll spin );
					  (* Ralisation des mouvements *)
					  view#translation trans;
					  view#rotation    rot;
					  view#spin        spin (0.0, 0.0, 0.0);

					  view#draw ();
					  saveFun ();

					  move mvt deltaT saveFun
				      end



let execCommand cmd deltaT pauseFun saveFun = let masque b lst = List.iter (if b
									    then
									      (fun scn -> scn#hide)
									    else
									      (fun scn -> scn#show)
									   )
									   lst
					      in
						match cmd
						with
						  | Pause                     -> pauseFun ()

						  | Show scnLst               -> masque false scnLst;
										 view#draw ();
										 saveFun ()

						  | Hide scnLst               -> masque true  scnLst;
										 view#draw ();
										 saveFun ()

						  | Replace (hide, show)      -> masque false show;
										 masque true  (unSome hide);
										 view#draw ();
										 saveFun ()

						  | Command _                 -> ()

						  | View (viewr, centr, vert) -> (* On fixe le centre de la vision *)
										 view#getCentre#setAll (match centr
													with
													  | AutoCenter -> view#getMilieuDessin#getAll
													  | Center pt  -> pt#getAll
												       );

										 (* On fixe le vecteur Z du repre de la camera *)
										 (match viewr
										  with
										    | Viewer pt  -> let vecteur = diff_triplet pt#getAll view#getCentre#getAll
												    in
												    let norme = normale_triplet vecteur
												    in
												      view#setZoom norme;
												      view#getVectZ#setAll (div_triplet vecteur norme)
										    | AutoViewer -> view#setZoom view#getRayon;
												    view#getVectZ#setAll (0.0, 0.5, 0.86602540378445)
										 );

										 (* On fixe la verticale du repre de la camera *)
										 let z = view#getVectZ#getAll
										 in
										 let vect = (match vert
											     with
											       | AutoVertical -> if (colineaire (0.0, 1.0, 0.0) z)
														 then (0.0, 0.0, -1.0)
														 else (0.0, 1.0,  0.0)
											       | Vertical pt  -> (if (colineaire pt#getAll z)
														  then if (pt#getAll = (0.0, 1.0, 0.0))
														       then (0.0, 0.0, -1.0)
														       else (0.0, 1.0,  0.0)
														  else pt#getAll
														 )
											    )
										 in
										 let sbt = normalise_triplet (diff_triplet vect (mult_triplet z (scalaire vect z)))
										 in
										   view#getVectY#setAll sbt;

										   (* On fixe le verteur X du repre de la camera *)
										   view#getVectX#setAll (prod_triplet sbt z);

										   view#draw ();
										   saveFun ()

						  | Movement mvt              -> (* On ajoute un timout ralisant le mouvement *)
										 let copie = new Types.mouvement
										 in
										   copie#setrotSpeed   (Oo.copy mvt#rotSpeed);
										   copie#settransSpeed (Oo.copy mvt#transSpeed);
										   copie#setspinSpeed  (Oo.copy mvt#spinSpeed);
										   copie#setrotDelta   (Oo.copy mvt#rotDelta);
										   copie#settransDelta (Oo.copy mvt#transDelta);
										   copie#setspinDelta  (Oo.copy mvt#spinDelta);
										   move copie deltaT saveFun



let rec delete file lst = match lst
			  with
			    | []   -> Sys.remove file
			    | t::q -> if t = file
				      then
					delete file q
				      else
					begin
					  Sys.remove file;
					  delete t q
					end



let createLst name format rate fps =

  (* On supprime un ventuel timout pour viter les interfrences *)
  vireTimout ();

  let deltaT = (1.0 /. (float_of_int fps))
  and index = ref (-1)
  and (nom, ext) = let (nom, ext) = Usefull.splitName name
		   in
		     try
		       if (Image.guess_extension ext) = format
		       then
			 (nom, ext)
		       else
			 (name, Image.extension format)
		     with
		       | Not_found -> (name, Image.extension format)
  in

  let saveImage () = try
		       incr index;
		       let name = Printf.sprintf "%s%d.%s" nom !index ext
		       and img  = Screenshot.saveImage ()
		       in
			 Screenshot.enregImage name
					       format
					       (if format = Image.Jpeg
						then
						  [Image.Save_Quality 100]
						else
						  []
					       )
					       img
					       rate;
			 Rgb24.destroy img
		     with
		       | e -> print_endline (Printexc.to_string e);
			      exit 2
  in

    while not (core#getCommandToDo#isEmpty)
    do
      let cmd = core#getCommandToDo#popBegin ()
      in
	execCommand cmd
		    deltaT
		    ignore
		    saveImage;
	core#getCommandDone#pushBegin cmd
    done;

    !Parametres.ecrireStatut "Stop";
    core#doStop ()



let createVideo pathTmp name rate fps pauseTempo cmdTempo = 

  (* On supprime un ventuel timout pour viter les interfrences *)
  vireTimout ();

  let deltaT = (1.0 /. (float_of_int fps))
  and index = ref (-1)
  and nom = tmpFilesName pathTmp
  in
  let liste = ref ([]:string list)
  in

  let insertImages name nbr = for i = 1 to nbr
			      do
				liste := (name :: !liste)
			      done
  in
  let saveImage () = try
		       incr index;
		       let name = Printf.sprintf "%s_%d.jpg" nom !index
		       and img  = Screenshot.saveImage ()
		       in
			 Screenshot.enregImage name
					       Image.Jpeg
					       [Image.Save_Quality 100]
					       img
					       rate;
			 Rgb24.destroy img;
			 insertImages name 1
		     with
		       | e -> print_endline (Printexc.to_string e);
			      exit 2
  in

    while not (core#getCommandToDo#isEmpty)
    do
      let cmd = core#getCommandToDo#popBegin ()
      in
	execCommand cmd
		    deltaT
		    (fun () -> insertImages (Printf.sprintf "%s_%d.jpg" nom !index)
					    pauseTempo
		    )
		    saveImage;
	core#getCommandDone#pushBegin cmd;
	insertImages (Printf.sprintf "%s_%d.jpg" nom !index) cmdTempo
    done;

    let listeFile = Unix.openfile (nom ^ ".list")
				  [Unix.O_CREAT ; Unix.O_WRONLY]
				  0o600
    in
      
    let rec printFile lst = match lst
			    with
			      | []   -> ()
			      | t::q -> printFile q;
					ignore (Unix.write listeFile (t ^ "\n") 0 (String.length t + 1))
    in
      printFile !liste;
      Unix.close listeFile;

      (let    nomLine = Str.global_replace  nomRegexp name                                          !Parametres.encodeLine
       in
       let      wLine = Str.global_replace    wRegexp (string_of_int (Usefull.round (!Parametres.width  *. rate))) nomLine
       in
       let      hLine = Str.global_replace    hRegexp (string_of_int (Usefull.round (!Parametres.height *. rate)))   wLine
       in
       let    fpsLine = Str.global_replace  fpsRegexp (string_of_int fps)                                            hLine
       in
       let encodeLine = Str.global_replace listRegexp (nom ^ ".list")                                              fpsLine
       in
	 ignore (Unix.system encodeLine)
      );

      delete (nom ^ ".list") !liste;
      !Parametres.ecrireStatut "Stop";
      core#doStop ()



let createGifAnime pathTmp name rate fps pauseTempo cmdTempo = 

  (* On supprime un ventuel timout pour viter les interfrences *)
  vireTimout ();

  let deltaT = (1.0 /. (float_of_int fps))
  and index = ref (-1)
  and nom = tmpFilesName pathTmp
  in
  let listeImg   = ref ([]:string list)
  and listeDuree = ref ([]:float ref list)
  in

  let saveImage () = try
		       incr index;
		       let name = Printf.sprintf "%s_%d.bmp" nom !index
		       and img  = Screenshot.saveImage ()
		       in
			 Screenshot.enregImage name
					       Image.Jpeg
					       [Image.Save_Quality 100]
					       img
					       rate;
			 Rgb24.destroy img;
			 listeImg   := name :: !listeImg;
			 listeDuree := (ref deltaT) :: !listeDuree
		     with
		       | e -> print_endline (Printexc.to_string e);
			      exit 2
  in

    while not (core#getCommandToDo#isEmpty)
    do
      let cmd = core#getCommandToDo#popBegin ()
      in


	execCommand cmd
		    deltaT
		    (fun () -> let d = List.hd !listeDuree
			       in
				 d := !d +. pauseTempo
		    )
		    saveImage;
	core#getCommandDone#pushBegin cmd;
	let d = List.hd !listeDuree
	in
	  d := !d +. cmdTempo
    done;

    let rec commandLine lstImg lstDuree = match lstImg, lstDuree
					  with
					    |        [],            [] -> "convert -loop 0 "

					    | img::qImg, duree::qDuree -> (commandLine qImg qDuree) ^ (Printf.sprintf "-delay %d %s " (Usefull.round (100.0 *. !duree)) img)

					    |         _,             _ -> raise (Erreur "Erreur dans la cration de la ligne de commande :|")
    in

      ignore (Unix.system ((commandLine !listeImg !listeDuree) ^ name));
      delete (List.hd !listeImg) (List.tl !listeImg);
      !Parametres.ecrireStatut "Stop";
      core#doStop ()
