open Types
open Parametres
open Content
open Viewport
open Group

class cylinder (col:couleur) (position:point) (wire:draw) (cont: container option) (radiusVal:float) (heightVal : float) (slicesVal:int) (solidVal:bool)=
  object (self)
    inherit content col position wire cont as super

    (*Members*)
    val mutable radius = radiusVal
    val mutable height = heightVal
    val mutable slices = slicesVal
    val mutable solid = solidVal

    (*Methods*)
    (*-------*)

    (*Setter & Getter*)
    method setRadius newRadius = radius <- newRadius
    method getRadius = radius

    method setHeight newHeight = height <-newHeight
    method getHeight = height
  
    method setSlices newSlices = slices <- newSlices
    method getSlice = slices

    method setSolid newSolid = solid <- newSolid
    method getSolid = solid

    method drawSpecific sector drawInt = let cosSlice = Array.make slices 0.0
					 and sinSlice = Array.make slices 0.0
                                         in
                                           for j =  0  to (slices - 1)
					   do
					     let phi = (pi2 *. (float_of_int j)) /. (float_of_int slices)
					     in
					       cosSlice.(j) <- cos phi;
					       sinSlice.(j) <- sin phi
					   done;

                                         let haut drawInt = GlDraw.begins `triangle_fan;
					   normeSens (0.0, 1.0, 0.0) drawInt;
					   GlDraw.vertex3 (0.0, height, 0.0);
					   GlDraw.vertex3 (radius, height, 0.0);
					   for j = (slices - 1) downto 0
					   do
					     GlDraw.vertex3 (radius *. cosSlice.(j), height, radius *. sinSlice.(j))
					   done;
					   GlDraw.ends ()
					   
					 and bas drawInt = GlDraw.begins `triangle_fan;
					   normeSens (0.0, -1.0, 0.0) drawInt;
					   GlDraw.vertex3 (0.0, -.height, 0.0);
					   for j = 0 to (slices - 1)
					   do
					     GlDraw.vertex3 (radius *. cosSlice.(j), -.height, radius *. sinSlice.(j))
					   done;
					   GlDraw.vertex3 (radius, -.height, 0.0);
					   GlDraw.ends ()
					     
					 and cote y1 y2 drawInt = GlDraw.begins `quad_strip;
					   for j = 0 to (slices - 1)
					   do
					     normeSens (cosSlice.(j), 0.0, sinSlice.(j)) drawInt;
					     GlDraw.vertex3 (radius *. cosSlice.(j), y1, radius *. sinSlice.(j));
					     GlDraw.vertex3 (radius *. cosSlice.(j), y2, radius *. sinSlice.(j))
					   done;
					   normeSens (cosSlice.(0), 0.0, sinSlice.(0)) drawInt;
					   GlDraw.vertex3 (radius, y1, 0.0);
					   GlDraw.vertex3 (radius, y2, 0.0);
					   GlDraw.ends ()
					 in
					   
					   let sbt = match solid, sector
					   with
					     |  true, Bottom -> (fun drawInt -> bas drawInt)
						  
					     |  true, Inf    -> (fun drawInt -> bas drawInt;
								cote (-.height) 0. drawInt)

					     |  true, Sup    -> (fun drawInt -> haut drawInt;
								cote 0. height drawInt)

					     |  true, All    -> (fun drawInt -> bas drawInt;
								haut drawInt;
								cote (-.height) height drawInt)

					     | false, _      -> (fun drawInt -> cote (-.height) height drawInt)
					   in
					     sbt false;
					     if drawInt
					     then
					       begin
						 GlDraw.front_face `cw;
						 sbt true;
						 GlDraw.front_face `ccw
					       end

    method getBoundingBox () = let rad = ref radius and hei = ref height 
                               and coeffRayonCont cotes = cos (Parametres.pi /. (float_of_int cotes))
                               and (minX, minY, minZ, maxX, maxY, maxZ), zoom, trans = self#getContainerBounds
                               in
                                 if container = None
				 then
				   begin
				     if !rad == nan
				     then
				       rad := 1.0;
				     if !hei == nan
				     then
				       hei := 1.0
				   end
				 else
				   begin
				     let coeff = coeffRayonCont slices
				     and diag  = 0.5 *. (sqrt (let dX = maxX -. minX
							       and dZ = maxZ -. minZ
							       in
								 dX*.dX +. dZ*.dZ))
				     in
				     let defRayon = diag /. coeff
				     and defHaut  = 0.5 *. (maxY -. minY)
				     in
				     let rR = (abs_float (!rad /. defRayon))
				     and rH = (abs_float (!hei /. defHaut))
				     in
				       if !rad == nan
				       then
					 begin
					   if !hei == nan
					   then
					     begin
					       rad := defRayon;
					       hei := defHaut
					     end
					   else
					     begin
					       if rH < 1.0
					       then
						 zoom := rH;
					       rad := defRayon *. !zoom
					     end
					 end
				       else
					 begin
					   if !hei == nan
					   then
					     begin
					       if rR < 1.0
					       then
						 zoom := rR;
					       hei := defHaut *. !zoom
					     end
					   else
					     begin
					       let minR = min rR rH
					       in
						 if minR < 1.0
						 then
						   zoom := minR
					     end
					 end
				   end;

                                   (-. !rad, -. !hei, -. !rad,
				    !rad,    !hei,    !rad)
  end
