open Types
open Content
open Viewport
open Group

class empty (col:couleur) (position:point) (wire:draw) (cont: container option) (measureVal:point) (visibleVal:bool) =
  object(self)
    inherit content col position wire cont as super
    
    (*Members*)
    val mutable measure = measureVal
    val mutable visible = visibleVal

    (*Methods*)
    (*-------*)
    method setMeasure newMeasure = measure <- newMeasure
    method getMeasure = measure

    method setVisible newVisible = visible <-newVisible
    method getVisible = visible
 
    (*Draws this primitive after the content has been drawn by the parent object*)
    method drawSpecific sector drawInt = let x=measure#getX and y = measure#getY and z=measure#getZ in
                                           (* Les traits n'ont pas de normale -> on dsactive la lumire *)
                                           Gl.disable `lighting;
                                           if view#getPolygonMode = `point
					   then
					     begin
					       GlDraw.begins `points;
					       let bas  () = GlDraw.vertex3 (-. x, -.y,   z);
					                     GlDraw.vertex3 (-. x, -.y, -.z);
							     GlDraw.vertex3 (   x, -.y,   z);
							     GlDraw.vertex3 (   x, -.y, -.z)

					       and haut () = GlDraw.vertex3 (  x, y, -.z);
					                     GlDraw.vertex3 (  x, y,   z);
							     GlDraw.vertex3 (-.x, y, -.z);
							     GlDraw.vertex3 (-.x, y,   z)
					       in
						 (match sector
						   with
						     | Bottom
						     | Inf -> bas ()
						     | Sup -> haut ()
						     | All -> bas ();
							      haut ()
						 );
						 GlDraw.ends ()
					     end
					   else
					     begin
					       GlDraw.line_stipple ~factor:5 0xAAAA;
					       let bas  () = GlDraw.begins `line_strip;
						             GlDraw.vertex3 (  x, -.y,   z);
							     GlDraw.vertex3 (  x, -.y, -.z);
							     GlDraw.vertex3 (-.x, -.y, -.z);
							     GlDraw.vertex3 (-.x, -.y,   z);
							     GlDraw.vertex3 (  x, -.y,   z);
							     GlDraw.ends ()

					       and haut () = GlDraw.begins `line_strip;
						             GlDraw.vertex3 (  x, y,   z);
							     GlDraw.vertex3 (  x, y, -.z);
							     GlDraw.vertex3 (-.x, y, -.z);
							     GlDraw.vertex3 (-.x, y,   z);
							     GlDraw.vertex3 (  x, y,   z);
							     GlDraw.ends ()

					       and cote y1 y2 = GlDraw.begins `lines;
						                GlDraw.vertex3 (  x, y1,   z);
								GlDraw.vertex3 (  x, y2,   z);
								GlDraw.vertex3 (-.x, y1,   z);
								GlDraw.vertex3 (-.x, y2,   z);
								GlDraw.vertex3 (  x, y1, -.z);
								GlDraw.vertex3 (  x, y2, -.z);
								GlDraw.vertex3 (-.x, y1, -.z);
								GlDraw.vertex3 (-.x, y2, -.z);
								GlDraw.ends ()
					       in
					         let sbt () = match sector
						   with
						     | Bottom -> bas ()
						     | Inf    -> bas ();
								 cote (-.y) 0.
						     | Sup    -> haut ();
								 cote 0. y
						     | All    -> bas  ();
								 haut ();
								 cote (-.y) y
						 in
						   if !Parametres.isVectoring 
						   then
						     begin
						       (* Pointills :) *)
						       Vectoriel.psEnable `line_stipple;
						       sbt ();
						       Vectoriel.psDisable `line_stipple
						     end
						   else
						     begin
						       Gl.enable `line_stipple;
						       sbt ();
						       Gl.disable `line_stipple
						     end;
					     end;
					   if view#getLighting
					   then
					     Gl.enable `lighting

    method getBoundingBox () = let (minX, minY, minZ, maxX, maxY, maxZ), zoom, trans = self#getContainerBounds
                               in
                                 if container = None
				 then
				   begin
				     if measure#isDefault
				     then
				       measure#setAll (1.0, 1.0, 1.0)
				   end
				 else
				   begin
				     if measure#isDefault
				     then
				       measure#setAll ((maxX -. minX) /. 2.0, (maxY -. minY) /. 2.0, (maxZ -. minZ) /. 2.0)
				     else
				       begin
					 let minR = min (min (abs_float ((2.0 *. measure#getX) /. (maxX -. minX)))
							   (abs_float ((2.0 *. measure#getY) /. (maxY -. minY))))
						      (abs_float ((2.0 *. measure#getZ) /. (maxZ -. minZ)))
					 in
					   if minR < 1.0
					   then
					     zoom := minR
				       end
				   end;
      
                                   (-.measure#getX, -.measure#getY, -.measure#getZ,
				    measure#getX,   measure#getY,   measure#getZ)
      
  end
