(*********************************************************)
(*                                                       *)
(* Ce fichier dfinit une file  deux entres : chaque   *)
(* extrmit est entrante et sortante.                    *)
(* Pour crer une nouvelle file, il n'est pas ncessaire *)
(* de connatre le type du contenu.                      *)
(* Cependant, une fois que la file a t remplie, son    *)
(* contenu doit tre de type uniforme.                   *)
(*                                                       *)
(*********************************************************)


exception FileVide



(**********************)
(* Maillon de la file *)
(**********************)
class ['a] item (value:'a) =
object
  val mutable pred  = (None:(('a item) option))
  val mutable suiv  = (None:(('a item) option))

  method valeur     = value
  method getPred () = match pred
		      with
			| Some sbt -> sbt
			| None     -> raise FileVide
  method getSuiv () = match suiv
		      with
			| Some sbt -> sbt
			| None     -> raise FileVide
  method setPred p  = pred <- Some p
  method setSuiv s  = suiv <- Some s
end


(***********************)
(* File  deux entres *)
(***********************)
class ['a] file =
object
  val mutable premier   = (None:(('a item) option))
  val mutable dimension = 0

  (* Mthode pour ajouter un lment en dbut de file *)
  method pushBegin (elt:'a) =
    let sbt = new item elt
    in
      dimension <- dimension + 1;
      match premier
      with
	| None   -> sbt#setPred sbt;
		    sbt#setSuiv sbt;
		    premier <- Some sbt
	| Some p -> let d = p#getPred ()
		    in
		      sbt#setPred d;
		      sbt#setSuiv p;
		      p#setPred sbt;
		      d#setSuiv sbt;
		      premier <- Some sbt

  (* Mthode pour supprimer et obtenir l'lment en dbut de file *)
  (* Mthode pour obtenir l'lment en dbut de file *)
  method popBegin () =
    match premier
    with
      | None   -> raise FileVide
      | Some p -> let s = p#getSuiv ()
		  and d = p#getPred ()
		  in
		    dimension <- dimension - 1;
		    s#setPred d;
		    d#setSuiv s;
		    (if dimension = 0
		     then
		       premier <- None
		     else
		       premier <- Some s
		    );
		    p#valeur

  (* Mthode pour ajouter un lment en fin de file *)
  method pushEnd (elt:'a) =
    let sbt = new item elt
    in
      dimension <- dimension + 1;
      match premier
      with
	| None   -> sbt#setPred sbt;
		    sbt#setSuiv sbt;
		    premier <- Some sbt
	| Some p -> let d = p#getPred ()
		    in
		      sbt#setPred d;
		      sbt#setSuiv p;
		      p#setPred sbt;
		      d#setSuiv sbt

  (* Mthode pour supprimer et obtenir l'lment en fin de file *)
  method popEnd () =
    match premier
    with
      | None   -> raise FileVide
      | Some p -> let d = p#getPred ()
		  in
		  let a = d#getPred ()
		  in
		    dimension <- dimension - 1;
		    p#setPred a;
		    a#setSuiv p;
		    if dimension = 0
		    then
		      premier <- None;
		    d#valeur

  (* Mthode pour vider la file *)
  method clear () = premier   <- None;
		    dimension <- 0

  (* Mthode pour obtenir la dimension de la file *)
  method length  = dimension

  (* Mthode pour savoir si la file est vide ou non *)
  method isEmpty = (premier = None)

end
