open Types
open Parametres
open Content
open Viewport
open Disc
open Group

class frustrum (col:couleur) (position:point) (wire:draw) (cont: container option) (radiusVal:float) (heightVal : float) (rateVal:float) (slicesVal:int) (solidVal:bool)=
  object (self)
    inherit content col position wire cont as super

    (*Members*)
    val mutable radius = radiusVal
    val mutable height = heightVal
    val mutable rate = rateVal
    val mutable slices = slicesVal
    val mutable solid = solidVal

    (*Methods*)
    (*-------*)

    (*Setter & Getter*)
    method setRadius newRadius = radius <- newRadius
    method getRadius = radius

    method setHeight newHeight = height <-newHeight
    method getHeight = height

    method setRate newRate = rate <- newRate
    method getRate = rate
  
    method setSlices newSlices = slices <- newSlices
    method getSlice = slices

    method setSolid newSolid = solid <- newSolid
    method getSolid = solid

    (***********************************************************)
    (* Dessine un tronc de cne en fonction de la composante   *)
    (* sur Y de la base (down) et du sommet (up) et            *)
    (* du rapport entre les rayons suprieurs et infrieur.    *)
    (* solidBas et solidHaut indiquent si la base et le sommet *)
    (* du tronc de cne sont pleins.                           *)
    (* drawInt indique si l'intrieur est galement dessin.   *)
    (***********************************************************)
    (* Type : float -> float -> float -> bool -> bool ->  bool *)
    (*        -> unit                                          *)
    (***********************************************************)
    method drawFrustrum down up rate solidBas solidHaut drawInt = let height = up -. down
                                                                  in
                                                                    let cosSlice  = Array.make slices 0.0
								    and sinSlice  = Array.make slices 0.0
								    and rayonHaut = radius *. rate
								    and cst = 2.0 *. (cos (pi /. (float_of_int slices)))
								    and yNormale = 1.0 /. (sqrt (1.0 +. 4.0*.height*.height*.rate*.rate/.(radius*.radius)))
								    in
								      let cst2 = 2.0 *. height *. rate *. yNormale /. radius
								      in
								        let trait i = normeSens (cst2 *. cosSlice.(i), yNormale, cst2 *. sinSlice.(i)) drawInt;
									              GlDraw.vertex3 (radius    *. cosSlice.(i), down, radius    *. sinSlice.(i));
										      GlDraw.vertex3 (rayonHaut *. cosSlice.(i),   up, rayonHaut *. sinSlice.(i));
									in
									  for j =  0  to (slices - 1)
									  do
									    let phi = (pi2 *. (float_of_int j)) /. (float_of_int slices)
									    in
									      cosSlice.(j) <- cos phi;
									      sinSlice.(j) <- sin phi
									  done;

									  let cote drawInt = GlDraw.begins `quad_strip;
									                     for j = 0 to (slices-1)
											     do
											       trait j
											     done;
											     trait 0;
											     GlDraw.ends ()

									  and haut drawInt = GlDraw.begins `triangle_fan;
									                     normeSens (0.0,  1.0, 0.0) drawInt;
											     GlDraw.vertex3 (0.0, up, 0.0);
											     GlDraw.vertex3 (rayonHaut, up, 0.0);
											     for j = (slices - 1) downto 0
											     do
											       GlDraw.vertex3 (rayonHaut *. cosSlice.(j), up, rayonHaut *. sinSlice.(j))
											     done;
											     GlDraw.ends ()

									  and bas drawInt = GlDraw.begins `triangle_fan;
									                     normeSens (0.0, -1.0, 0.0) drawInt;
											     GlDraw.vertex3 (0.0, down, 0.0);
											     for j = 0 to (slices - 1)
											     do
											       GlDraw.vertex3 (radius *. cosSlice.(j), down, radius *. sinSlice.(j))
											     done;
											     GlDraw.vertex3 (radius, down, 0.0);
											     GlDraw.ends ()
									  in
									    
									    let sbt = match solidHaut, solidBas
									              with
											|  true, true  -> (fun drawInt -> bas drawInt;
											                                  haut drawInt;
															  cote drawInt)

											|  true, false -> (fun drawInt -> haut drawInt;
														          cote drawInt)

											| false,  true -> (fun drawInt -> bas drawInt;
														          cote drawInt)

											| false, false -> (fun drawInt -> cote drawInt)
									    in
									      sbt false;
									      if drawInt
									      then
										begin
										  GlDraw.front_face `cw;
										  sbt true;
										  GlDraw.front_face `ccw
										end

    (***************************************************************)
    (* Dessine un tronc de cne                                    *)
    (* secteur dtermine quel morceau de sphre il faut dessiner : *)
    (*   - Inf    -> 1/2 sphre infrieur                          *)
    (*   - Sup    -> 1/2 sphre suprieur                          *)
    (*   - All    -> sphre entire                                *)
    (*   - Bottom -> Ple Sud                                      *)
    (* drawInt indique si l'intrieur est galement dessin.       *)
    (***************************************************************)
    (* Type : Types.cont_mode -> bool -> unit                      *)
    (***************************************************************)
    method drawSpecific secteur drawInt = match secteur
					  with
					    | Bottom -> GlMat.push ();
					                GlMat.translate3 (0., -.height, 0.);
							let d = (new disc col pos self#getWire radius slices) in d#drawSelf ();
							GlMat.pop ()
					    | Inf    -> self#drawFrustrum (-.height) 0. (0.5 *. (rate+.1.)) solid false drawInt
					    | Sup    -> self#drawFrustrum 0. height (2./.(1.+.1./.rate)) false solid drawInt
					    | All    -> self#drawFrustrum (-.height) height rate solid solid drawInt

    method getBoundingBox () = let rad = ref radius and hei = ref height and tail = ref rate
                               and coeffRayonCont cotes = cos (Parametres.pi /. (float_of_int cotes))
                               and (minX, minY, minZ, maxX, maxY, maxZ), zoom, trans = self#getContainerBounds
                               in
                                 if !tail == nan
				 then
				   tail := 0.25;
                                 if container = None
				 then
				   begin
				     if !rad == nan
				     then
				       rad := 1.0;
				     if !hei == nan
				     then
				       hei := 1.0
				   end
				 else
				   begin
				     let coeff = coeffRayonCont slices
				     and diag  = 0.5 *. (sqrt (let dX, dZ = maxX -. minX, maxZ -. minZ
							       in
								 dX*.dX +. dZ*.dZ))
				     and haut  = 0.5 *. (maxY -. minY)
				     in
				       if !rad == nan
				       then
					 begin
					   if !hei == nan
					   then
					     begin
					       hei := haut;
					       rad := diag /. (coeff *. !tail)
					     end
					   else
					     begin
					       if !hei >= haut
					       then
						 begin
						   rad := 2.*.diag/.(coeff*.(2.+.(!tail-.1.)*.(1. +. haut /. !hei)))
						 end
					       else
						 begin
						   zoom := !hei /. haut;
						   rad    := diag /. (!tail *. coeff)
						 end
					     end
					 end
				       else
					 begin
					   if !hei == nan
					   then
					     begin
					       if !rad > (diag /. coeff)
					       then
						 begin
						   hei := max haut (haut *. (1. -. !tail) /. (1. -. (diag/.(!rad*.coeff))));
						   trans#updateY (-. !hei +. haut)
						 end
					       else
						 begin
						   zoom := !tail *. !rad *. coeff /. diag;
						   hei    := !zoom *. haut
						 end
					     end
					   else
					     begin
					       if haut > !hei
					       then
						 zoom := !hei /. haut;
					       if !zoom*.diag > !rad *. coeff *. (1. -. !zoom *. haut *.(1. -. !tail) /. !hei)
					       then
						 zoom := !hei /. (!hei*.diag/.(!rad*.coeff) +. haut*.(1.-. !tail));
					       trans#updateY (max 0. ((((2.*. !hei /. (!tail-.1.)) *. (!zoom*.diag/.(!rad*.coeff) -. 1.) -. !hei)/. !zoom) -. haut))
					     end
					 end
				   end;
				 
				 (-. !rad, -. !hei, -. !rad,
				  !rad,    !hei,    !rad)
  end
  
