open Types
open PositionedGroup
open Boundingbox
open Triplet

(* TODO*)
(* Find a convenient way to reorganize the grids and hexagrids as well as bounding functions into proper objects *)

class grid0 (col:couleur) (pos:point) (cont:(drawable*point) list ref) (align1:align) (align2:align) (align3:align) =
  object(self)
    inherit positionedGroup col pos cont as super

    initializer
      (* Compute position of each object *)
      let col    = ref 0.0 
      and row    = ref 0.0 
      and stage  = ref 0.0
      in 
        let (x_pred, y_pred, z_pred) = (ref 0.0, ref 0.0, ref 0.0)
	in
	  (* Calcule de la taille du Grid0 et de la 1re partie de la translation pour passer d'un objet au suivant *)
	  List.iter (function (obj, vect) -> let (min1, min2, min3, max1, max2, max3) = obj#getBoundingBox ()
	                                     in
					       max_diff_update_ref col   max1 min1;
	                                       max_diff_update_ref row   max2 min2;
					       max_diff_update_ref stage max3 min3;
					       let (x, y, z) = (sbt align1 min1 max1,
							        sbt align2 min2 max2,
							        sbt align3 min3 max3)
					       in
						 vect#setAll (x -. !x_pred,
							     y -. !y_pred,
							     z -. !z_pred);
						 x_pred := x;
						 y_pred := y;
						 z_pred := z;
		    ) !content;

	  (* Calcule 2nde partie de la translation pour passer d'un objet au suivant *)
	  (match !content
	    with
	      | []           -> ()
	      | (_, vect)::_ -> vect#updateAll (sbt2 align1 0.0   !col,
					       sbt2 align2 0.0   !row,
					       sbt2 align3 0.0 !stage);
	  );
	  bbFunction <- (fun () -> boundingBox_of self#getPos !col !row !stage AxeX AxeY)

  end

class grid1 (col:couleur) (pos:point) (cont:(drawable*point) list ref) (axis1:axe) (align2:align) (align3:align) (space:float) =
 object(self)
    inherit positionedGroup col pos cont as super

    initializer
      let dim   = List.length !content
      and axis2 = axe_suiv axis1
      in
        let col    = Array.make dim 0.0 
	and row    = ref 0.0 
	and stage  = ref 0.0
	and indice = ref 0 
	in 
          let (x_pred, y_pred, z_pred) = (ref 0.0, ref 0.0, ref 0.0)
	  in
	    (* Calcule de la taille de chaque case du  Grid1 et de la 1re partie de la translation pour passer d'un objet au suivant *)
	    List.iter (function (obj, vect) -> let (min1, min2, min3, max1, max2, max3) = desordre (obj#getBoundingBox ()) axis1 axis2
	                                       in
						 max_diff_update_tab col !indice max1 min1;
	                                         max_diff_update_ref row         max2 min2;
						 max_diff_update_ref stage       max3 min3;
						 let (x, y, z) = (sbt Middle min1 max1,
								  sbt align2 min2 max2,
								  sbt align3 min3 max3)
						 in
						   vect#setAll (permut (x -. !x_pred,
								        y -. !y_pred,
								        z -. !z_pred)
								axis1
								axis2);
						   x_pred := x;
						   y_pred := y;
						   z_pred := z;
 						   incr indice
		      ) !content;
	    
	    (* Calcule 2nde partie de la translation pour passer d'un objet au suivant *)
	    let sumCol   = (sumTab col)   +. (float_of_int (dim-1)) *. space
	    in
	      (match !content
	       with
		 | []            -> ()
		 | (_, vect)::tl -> vect#updateAll (permut ((col.(0)   -.   sumCol) /. 2.0,
							     sbt2 align2 0.0   !row,
							     sbt2 align3 0.0 !stage)
						    axis1
						    axis2);
		                    indice := 1;
				    List.iter (function (_, vect) -> let delta = ((col.(!indice-1) +. col.(!indice)) /. 2.0 +. space,
										 0.0,
										 0.0)
				                                     in
								       vect#updateAll (permut delta axis1 axis2);
				                                       incr indice
					      ) tl
	      );
	      bbFunction <- (fun () -> boundingBox_of self#getPos sumCol !row !stage axis1 axis2)

  end

(************************************************************)
(* Cration d'un Grid2                                      *)
(*   pt     : centre                                        *)
(*   axis1  : 1er axe de remplissage                        *)
(*   axis2  : 2me axe de remplissage                       *)
(*   align1 : alignement sur le 1er axe                     *)
(*   align2 : alignement sur le 2me axe                    *)
(*   align3 : alignement sur le 3me axe                    *)
(*   space1 : espace minimum entre 2 objets sur le 1er axe  *)
(*   space2 : espace minimum entre 2 objets sur le 2me axe *)
(*   rate   : triplet entr pour le ratio                   *)
(*   coul   : couleur du Grid2                              *)
(*   lst    : liste des objets du Grid2                     *)
(************************************************************)
(* Type : point -> axe -> axe -> align -> align -> align -> *)
(*        float -> float -> float * float * float ->        *)
(*        couleur -> (objet * point) list -> objet          *)
(************************************************************)
class grid2 (col:couleur) (pos:point) (cont:(drawable*point) list ref) (axis1:axe) (axis2:axe) (align1:align) (align2:align) (align3:align) (space1:float) (space2:float) (rate:float*float*float) =
 object(self)
    inherit positionedGroup col pos cont as super

    initializer
      let (axis1, axis2, ratio1, ratio2) = self#axis_ratio2 axis1 axis2 rate (List.length !content)
      in 
      let  col    = Array.make ratio1 0.0
      and row    = Array.make ratio2 0.0 
      and stage  = ref 0.0
      and indice = ref 0 
      in 
        let (x_pred, y_pred, z_pred) = (ref 0.0, ref 0.0, ref 0.0)
	in
	  (* Calcule de la taille de chaque case du  Grid2 et de la 1re partie de la translation pour passer d'un objet au suivant *)
	  List.iter (function (obj, vect) -> let (min1, min2, min3, max1, max2, max3) = desordre (obj#getBoundingBox ()) axis1 axis2
	                                     and i = !indice mod ratio1
	                                     and j = !indice / ratio1
	                                     in
					       max_diff_update_tab col i max1 min1;
	                                       max_diff_update_tab row j max2 min2;
					       max_diff_update_ref stage max3 min3;
					       let (x, y, z) = (sbt align1 min1 max1,
							        sbt align2 min2 max2,
							        sbt align3 min3 max3)
					       in
						 vect#setAll (permut (x -. !x_pred,
								      y -. !y_pred,
								      z -. !z_pred)
							      axis1
							      axis2);
						 x_pred := x;
						 y_pred := y;
						 z_pred := z;
 						 incr indice
		    ) !content;

	  (* Calcule 2nde partie de la translation pour passer d'un objet au suivant *)
	  let sumCol   = (sumTab col) +. (float_of_int (ratio1-1)) *. space1
	  and sumRow   = (sumTab row) +. (float_of_int (ratio2-1)) *. space2
	  in
	    (match !content
	     with
	       | []            -> ()
	       | (_, vect)::tl -> vect#updateAll (permut ((col.(0) -. sumCol) /. 2.0 +. (sbt2 align1 0.0 col.(0)),
							  (row.(0) -. sumRow) /. 2.0 +. (sbt2 align2 0.0 row.(0)),
							   sbt2 align3 0.0 !stage)
						  axis1
						  axis2);
		                  indice := 1;
				  List.iter (function (_, vect) -> let i     =  !indice     mod ratio1 
				                                   and i_old = (!indice -1) mod ratio1 
				                                   and j     =  !indice      / ratio1
				                                   and j_old = (!indice - 1) / ratio1
				                                   in
				                                     let delta = (if i = 0
								                  then ((col.(ratio1-1) +. col.(0)) /. 2.0 -. sumCol,
										       (row.(j-1) +. row.(j)) /. 2.0 +. space2,
										       0.0)
								                  else ((col.(i-1) +. col.(i)) /. 2.0 +. space1,
										         0.0,
										         0.0)
										 )
								     and delta2 = (sbt2 align1   col.(i_old) col.(i),
										   sbt2 align2   row.(j_old) row.(j),
										   0.0)
								     in
								       vect#updateAll (permut (add_triplet delta delta2)
										       axis1
										       axis2);
								       incr indice
					    ) tl
	    );
	    bbFunction <- (fun () -> boundingBox_of self#getPos sumCol sumRow !stage axis1 axis2)
  end

(************************************************************)
(* Cration d'un Grid3                                      *)
(*   pt     : centre                                        *)
(*   axis1  : 1er axe de remplissage                        *)
(*   axis2  : 2me axe de remplissage                       *)
(*   align1 : alignement sur le 1er axe                     *)
(*   align2 : alignement sur le 2me axe                    *)
(*   align3 : alignement sur le 3me axe                    *)
(*   space1 : espace minimum entre 2 objets sur le 1er axe  *)
(*   space2 : espace minimum entre 2 objets sur le 2me axe *)
(*   space3 : espace minimum entre 2 objets sur le 3me axe *)
(*   r1     : valeur entre pour le ratio sur l'axe axis1   *)
(*   r2     : valeur entre pour le ratio sur l'axe axis2   *)
(*   r3     : valeur entre pour le ratio sur l'axe "axis3" *)
(*   coul   : couleur du Grid2                              *)
(*   lst    : liste des objets du Grid2                     *)
(************************************************************)
(* Type : point -> axe -> axe -> align -> align -> align -> *)
(*        float -> float -> float -> float -> float         *)
(*        float -> couleur -> (objet * point) list -> objet *)
(************************************************************)
class grid3 (col:couleur) (pos:point) (cont:(drawable*point) list ref) (axis1:axe) (axis2:axe) (align1:align) (align2:align) (align3:align) (space1:float) (space2:float) (space3:float) (ratio1:float) (ratio2:float) (ratio3:float)=
 object(self)
    inherit positionedGroup col pos cont as super

    initializer
      (*Compute appropriate axes and axes ratios*)
      let (axis1, axis2, ratio1, ratio2, ratio3) = self#axis_ratio3 axis1 axis2 ratio1 ratio2 ratio3 (List.length !content)
      in
      let col    = Array.make ratio1 0.0 
      and row    = Array.make ratio2 0.0 
      and stage  = Array.make ratio3 0.0
      and indice = ref 0 
      in 
        let (x_pred, y_pred, z_pred) = (ref 0.0, ref 0.0, ref 0.0)
	in
	  (* Calcule de la taille de chaque case du  Grid3 et de la 1re partie de la translation pour passer d'un objet au suivant *)
	  List.iter (function (obj, vect) -> let (min1, min2, min3, max1, max2, max3) = desordre (obj#getBoundingBox ()) axis1 axis2
	                                     and i = !indice mod ratio1
	                                     and j = (!indice / ratio1) mod ratio2
	                                     and k = !indice / (ratio1 * ratio2) 
	                                     in
					       max_diff_update_tab col   i max1 min1;
	                                       max_diff_update_tab row   j max2 min2;
					       max_diff_update_tab stage k max3 min3;
					       let (x, y, z) = (sbt align1 min1 max1,
							        sbt align2 min2 max2,
							        sbt align3 min3 max3)
					       in
						 vect#updateAll (permut (x -. !x_pred,
									 y -. !y_pred,
									 z -. !z_pred)
								 axis1
								 axis2);
						 x_pred := x;
						 y_pred := y;
						 z_pred := z;
 						 incr indice
		    ) !content;

	  (* Calcule 2nde partie de la translation pour passer d'un objet au suivant *)
	  let sumCol   = (sumTab col)   +. (float_of_int (ratio1-1)) *. space1
	  and sumRow   = (sumTab row)   +. (float_of_int (ratio2-1)) *. space2
	  and sumStage = (sumTab stage) +. (float_of_int (ratio3-1)) *. space3
	  in
	    (match !content
	     with
	       | []            -> ()
	       | (_, vect)::tl -> vect#updateAll (permut ((col.(0)   -.   sumCol) /. 2.0 +. (sbt2 align1 0.0   col.(0)),
							  (row.(0)   -.   sumRow) /. 2.0 +. (sbt2 align2 0.0   row.(0)),
							  (stage.(0) -. sumStage) /. 2.0 +. (sbt2 align3 0.0 stage.(0)))
						  axis1
						  axis2);
		                  indice := 1;
				  List.iter (function (_, vect) -> let i     =  !indice                 mod ratio1 
				                                   and i_old = (!indice -1)             mod ratio1 
				                                   and j     =  (!indice      / ratio1) mod ratio2
				                                   and j_old = ((!indice - 1) / ratio1) mod ratio2
				                                   and k     =  !indice     / (ratio1 * ratio2)
				                                   and k_old = (!indice -1) / (ratio1 * ratio2)
				                                   in
				                                     let delta = (match (i, j) 
								     with
								       | (0, 0) -> ((col.(ratio1-1) +. col.(0)) /. 2.0 -. sumCol,
										    (row.(ratio2-1) +. row.(0)) /. 2.0 -. sumRow,
										    (stage.(k-1) +. stage.(k)) /. 2.0 +. space3)
									   
								       | (0, _) -> ((col.(ratio1-1) +. col.(0)) /. 2.0 -. sumCol,
										    (row.(j-1) +. row.(j)) /. 2.0 +. space2,
										     0.0)
									   
								       | (_, _) -> ((col.(i-1) +. col.(i)) /. 2.0 +. space1,
										     0.0,
										     0.0)
										   )
								     and delta2 = (sbt2 align1   col.(i_old) col.(i),
										   sbt2 align2   row.(j_old) row.(j),
										   sbt2 align3 stage.(k_old) stage.(k))
								     in
								       vect#updateAll (permut (add_triplet delta delta2)
										       axis1
										       axis2);
								       incr indice
					    ) tl
	    );
	    bbFunction <- (fun () -> boundingBox_of self#getPos sumCol sumRow sumStage axis1 axis2)
  end
