open Types
open Boundingbox

(* Contains a group of drawables *)
class container (posVal:point) (scaleVal:float) (contVal:drawable list ref) = 
  object(self)
 
    inherit drawable as super

    (*---------*)
    (* Members *)
    (*---------*)
    val mutable pos = posVal
    val mutable scale = scaleVal
    val mutable contenue = contVal

    (*---------*)
    (* Methods *)
    (*---------*)
    method setPos newPos = pos <- newPos
    method getPos = pos
    
    method setScale newScale = scale <-newScale
    method getScale = scale

    method setContenue newContenue = contenue <- ref newContenue
    method getContenue = !contenue

    (*Draw content of the container*)
    method draw () = if (!contenue = []) then ()
		     else
		       begin
			 GlMat.push ();
			 if scale <> 1.0
			 then
			   GlMat.scale3 (scale, scale, scale);
			   GlMat.translate3 pos#getAll;
			   List.iter (fun prim -> prim#draw()) !contenue;
			   GlMat.pop ()
		       end;

    method getBoundingBox () = raise (Erreur("Need to call BoundingBox of surrounding object!"))
  end

class group (col:couleur) (cont:drawable list ref)=
  object(self)
    inherit container (Parametres.ori) 1.0 cont as super

    (*---------*)
    (* Members *)
    (*---------*)
    val mutable color = col

    (*---------*)
    (* Methods *)
    (*---------*)
    method setColor newColor = color <- newColor
    method getColor = color
    
    (***********************************************)
    (* Fixe la couleur de l'objet  partir des     *)
    (* composantes rouge, vert, bleu et alpha      *)
    (***********************************************)
    (* Type : float * float *float * float -> unit *)
    (***********************************************)
    method setGLColor = let alpha = color#getAlpha and rouge = color#getRouge and vert = color#getVert and bleu = color#getBleu in
                          GlDraw.color ~alpha:alpha (rouge, vert, bleu);
                          GlLight.material `both (`diffuse (rouge, vert, bleu, alpha));
			  GlLight.material `both (`ambient (rouge, vert, bleu, alpha))

    method draw () = self#setGLColor;
                     List.iter (fun obj->obj#draw()) !contenue

    method getBoundingBox () = boundingList !contenue

  end
