(*****************************************************)
(*                                                   *)
(*                                                   *)
(*****************************************************)



open Types



class imo_couleur () =

  let tooltips = GData.tooltips ()
  in

  let accel_group = GtkData.AccelGroup.create ()
  in

  let window = GWindow.window ~wm_name:"Couleur du conteneur"
			      ~title:"ImoCamlView - Couleur du conteneur"
			      ~position:`CENTER
			      ~kind:`DIALOG
			      ~modal:false
			      ~allow_shrink:false
			      ~allow_grow:false
			      ~auto_shrink:false
			      ()
  in

  let _ = window#add_accel_group accel_group
  in

  let vbox = GPack.vbox ~spacing:0
			~homogeneous:false
			~packing:window#add
			()
  in

  let color_select = GMisc.color_selection ~opacity:true
					   ~packing:(vbox#pack ~padding:0
							       ~fill:false
							       ~expand:false
						    )
					   ()
  in


  let bottom = GPack.button_box `HORIZONTAL
				~spacing:10
				~child_width:80
				~child_height:24
				~layout:`SPREAD
				~packing:(vbox#pack ~padding:10
						    ~fill:false
						    ~expand:false
					 )
				()
  in


  let ok = GButton.button ~packing:(bottom#pack)
			  ~label:"Valider"
			  ()
  in

  let _ = GtkBase.Widget.set_can_focus   ok#as_widget true
  and _ = GtkBase.Widget.set_can_default ok#as_widget true
  and _ = ok#grab_default ()
  in

  let apply = GButton.button ~packing:(bottom#pack)
			     ~label:"Appliquer"
			     ()
  in

  let _ = GtkBase.Widget.set_can_focus   apply#as_widget true
  and _ = GtkBase.Widget.set_can_default apply#as_widget true
  in

  let cancel = GButton.button ~packing:(bottom#pack)
			      ~label:"Annuler"
			      ()
  in

  let _ = GtkBase.Widget.set_can_focus   cancel#as_widget true
  and _ = GtkBase.Widget.set_can_default cancel#as_widget true
  in

  let statut = GMisc.statusbar ~packing:(vbox#pack ~padding:0
						   ~fill:false
						   ~expand:false
					)
			       ()
  in

  let statut_blabla = statut#new_context ""
  in


object(self)


  method window = window
  method on_quitter () = window#destroy ()
  method on_window_remove (widget:GObj.widget) = self#on_quitter ()

  (* Methode qui crit la chane de caractres str sur la barre de statut *)
  method ecrire_statut str = statut_blabla#pop ();
			     ignore (statut_blabla#push str);
			     ()

  method double_clic () = match !(Parametres.select)
			  with
			    | Container (_, d, c, _, _) -> color_select#set_color ~red:c#getRouge
										  ~green:c#getVert
										  ~blue:c#getBleu
										  ~opacity:c#getAlpha
										  ();
			    | _                         -> self#ecrire_statut "Erreur : pas de container slectionn";

  method applique () = match !(Parametres.select)
		       with
			 | Container (_, d, c, _,  _) -> let col = color_select#get_color
							 in
							   c#setAll (col.Gtk.red,
								     col.Gtk.green,
								     col.Gtk.blue,
								     col.Gtk.opacity
								    );
                         | _                          -> self#ecrire_statut "Erreur : pas de container slectionn";

initializer

  self#double_clic();
  let _ = cancel#connect#clicked ~callback:self#on_quitter
  in
  let _ =  apply#connect#clicked ~callback:self#applique
  in
  let _ =     ok#connect#clicked ~callback:(fun () -> self#applique();self#on_quitter())
  in
  let _ = window#connect#remove  ~callback:self#on_window_remove
  in


  (* ESC -> ferme la fentre *)
  let _ = window#event#connect#key_press ~callback:(fun (event: GdkEvent.Key.t) -> begin
										     if (GdkEvent.Key.keyval event) = GdkKeysyms._Escape
										     then
										       begin
											 self#on_quitter();
											 true
										       end
										     else false
										   end
						   )
  in
    () 


end
