(******************************************)
(*                                        *)
(* Le fichier guiimage.ml permet d'ouvrir *)
(* la fentre de captures d'images.       *)
(*                                        *)
(******************************************)



open Types



(************************************************)
(* Renvoie le format de CamlImages associ  f. *)
(* Lve l'exception Types.APas si ce format     *)
(* n'est pas support.                          *)
(************************************************)
(* Type : format -> Image.format                *)
(************************************************)
let saveFormat f = match f
		   with
		     | Bmp  -> Image.Bmp
		     | Gif  -> Image.Gif
		     | Jpeg -> Image.Jpeg
		     | Ppm  -> Image.Ppm
		     | Png  -> Image.Png
		     | Ps   -> Image.Ps
		     | _    -> raise APas



(************************************)
(* Dtermine si l'extension ext est *)
(* approprie pour le format formt. *)
(************************************)
(* Type : format -> string -> bool  *)
(************************************)
let guessExtension ext formt = let ext = String.lowercase ext
			       in
				match ext, formt
				with
				  | "gif" , Gif
				  | "bmp" , Bmp
				  | "jpeg", Jpeg
				  | "jpg" , Jpeg
				  | "png" , Png
				  | "ppm" , Ppm
				  | "pgm" , Ppm
				  | "pbm" , Ppm
				  | "eps" , Ps
				  | "epsf", Ps
				  | "epsi", Ps
				  | "ps"  , PsVect
				  | "eps" , EpsVect
				  | "pdf" , Pdf  -> true
				  |      _, _    -> false



(***************************)
(* Renvoie l'extension     *)
(* associe au format f.   *)
(***************************)
(* Type : format -> string *)
(***************************)
let getExtension f = match f
		     with
		       | Gif     -> "gif"
		       | Bmp     -> "bmp"
		       | Jpeg    -> "jpg"
		       | Png     -> "png"
		       | Ppm     -> "ppm"
		       | Ps      -> "eps"
		       | EpsVect -> "eps"
		       | PsVect  -> "ps"
		       | Pdf     -> "pdf"



(* Tableau des formats d'images grs pour les captures. *)
(*  -> blabla  afficher dans le menu                    *)
let extension = [|"Bitmap (.bmp)"               ;
		  "GIF (.gif)"                  ;
		  "JPEG (.jpg)"                 ;
		  "Pixmap Portable (.ppm)"      ;
		  "PNG (.png)"                  ;
		  "PostScript Encapsule (.eps)" ;
		  "EPS vectoriel (.eps)" ;
		  "PostScript vectoiel (.ps)"   ;
		  "PDF (.pdf)"
		|]



(* Tableau des formats d'images grs pour les captures.                                *)
(*  -> format associe (de type Image.format) associ au texte contenu dans 'extension' *)
let formatAssoc = [| Bmp     ;
		     Gif     ;
		     Jpeg    ;
		     Ppm     ;
		     Png     ;
		     Ps      ;
		     EpsVect ;
		     PsVect  ;
		     Pdf
		   |]



(*******************************************************)
(* Classe dfinissant la fentre de capture d'images   *)
(* Ne pas oublier d'appeler la mthode #window#show () *)
(* pour afficher la fentre.                           *)
(*******************************************************)
class imo_image () =

  let window = GWindow.window ~wm_name:"Captures d'images"
			      ~title:"Imoview - Captures d'images"
			      ~position:`CENTER
			      ~kind:`DIALOG
			      ~modal:false
			      ~allow_shrink:false
			      ~allow_grow:false
			      ~auto_shrink:false
			      ()
  in

  let table = GPack.table ~rows:2
			  ~columns:2
			  ~row_spacings:0
			  ~col_spacings:0
			  ~homogeneous:false
			  ~packing:window#add
			  ()
  in

  let capture = GBin.frame ~packing:(table#attach ~left:0
						  ~top:0
						  ~right:1
						  ~bottom:1
						  ~xpadding:5
						  ~ypadding:5
						  ~expand:`BOTH
						  ~shrink:`NONE
						  ~fill:`BOTH
				    )
			   ~border_width:5
			   ~label:"Capture actuelle"
			   ~shadow_type:`ETCHED_IN
			   ()
  in

  let vboxCapt = GPack.vbox ~spacing:0
			    ~homogeneous:false
			    ~packing:capture#add
			    ()
  in

  let previewBorder = GBin.frame ~packing:(vboxCapt#pack ~padding:10
							 ~fill:true
							 ~expand:true
					  )
				 ~border_width:10
				 ~shadow_type:`OUT
				 ()
  in

  let preview = GMisc.drawing_area ~packing:previewBorder#add
				   ~height:256
				   ~width:256
				   ()
  in

  (* Zone d'affichage de l'apperu de la capture *)
  let drawing = preview#misc#realize ();
		new GDraw.drawable (preview#misc#window)
  in

  let bottomCapt = GPack.button_box `HORIZONTAL
				    ~spacing:30
				    ~child_width:85
				    ~child_height:24
				    ~layout:`SPREAD
				    ~packing:(vboxCapt#pack ~padding:0
							    ~fill:true
							    ~expand:true
					     )
				    ()
  in

  (* Bouton pour effectuer une nouvelle capture *)
  let newCapt = GButton.button ~packing:(bottomCapt#pack )
				()
  in

  let _ = GtkBase.Widget.set_can_focus newCapt#as_widget true
  and _ = GtkBase.Widget.set_can_default newCapt#as_widget true
  in

  let _ = My_controls.iconLabeled "Nouvelle capture"
				  (GDraw.pixmap_from_xpm_d ~data:Icones.shot
							   ~window
							   ()
				  )
				  ~packing:newCapt#add
				  ()
  in

  let options = GBin.frame ~packing:(table#attach ~left:1
						  ~top:0
						  ~right:2
						  ~bottom:1
						  ~xpadding:5
						  ~ypadding:5
						  ~expand:`X
						  ~shrink:`NONE
						  ~fill:`BOTH
				    )
			   ~border_width:5
			   ~label:"Options"
			   ~shadow_type:`ETCHED_IN
			   ()
  in

  let tableOpt = GPack.table ~rows:6
			     ~columns:3
			     ~row_spacings:10
			     ~col_spacings:10
			     ~border_width:10
			     ~homogeneous:false
			     ~packing:options#add
			     ()
  in

  (* Menu de slection du format d'image *)
  let format = GMenu.option_menu ~packing:(tableOpt#attach ~left:1
							   ~top:2
							   ~right:2
							   ~bottom:3
							   ~xpadding:0
							   ~ypadding:0
							   ~expand:`X
							   ~shrink:`NONE
							   ~fill:`X
					  )
				 ()
  in

  let choix = GMenu.menu ~show:true ()
  in

  (* On insert les items dans le menu option.                      *)
  (* Les items contiennent le texte dcrivant les formats d'images *)
  let _ = (for i = 0 to (Array.length extension) - 1
	   do
	     let item = GMenu.menu_item ~label:extension.(i) ()
	     in
	       choix#append item
	   done
	  )
  in

  let _ = format#set_menu choix
  and _ = format#set_history 0
  and _ = GtkBase.Widget.set_can_focus format#as_widget true
  in

  (* Zone de saisie du chemin du fichier destination *)
  let path = GEdit.entry ~packing:(tableOpt#attach ~left:1
						   ~top:0
						   ~right:2
						   ~bottom:1
						   ~xpadding:0
						   ~ypadding:0
						   ~expand:`X
						   ~shrink:`NONE
						   ~fill:`X
				  )
			 ~max_length:0
			 ~visibility:true
			 ~editable:true
			 ()
  in

  let _ = GtkBase.Widget.set_can_focus path#as_widget true
  in

  (* Zone de saisie du nom du fichier destination *)
  let name = GEdit.entry ~packing:(tableOpt#attach ~left:1
						   ~top:1
						   ~right:2
						   ~bottom:2
						   ~xpadding:0
						   ~ypadding:0
						   ~expand:`X
						   ~shrink:`NONE
						   ~fill:`X
				  )
			 ~max_length:0
			 ~visibility:true
			 ~editable:true
			 ()
  in

  let _ = GtkBase.Widget.set_can_focus name#as_widget true
  in

  let _ = GMisc.separator `HORIZONTAL
			  ~packing:(tableOpt#attach ~left:0
						    ~top:4
						    ~right:3
						    ~bottom:5
						    ~xpadding:0
						    ~ypadding:0
						    ~expand:`Y
						    ~shrink:`NONE
						    ~fill:`BOTH
				   )
			  ()
  in

  let bottomOpt = GPack.button_box `HORIZONTAL
				   ~spacing:30
				   ~child_width:85
				   ~child_height:24
				   ~layout:`SPREAD
				   ~packing:(tableOpt#attach ~left:0
							     ~top:5
							     ~right:3
							     ~bottom:6
							     ~xpadding:0
							     ~ypadding:0
							     ~expand:`Y
							     ~shrink:`NONE
							     ~fill:`Y
					    )
				   ()
  in

  (* Bouton Enregistrer pour sauvegarder l'image *)
  let save = GButton.button ~packing:(bottomOpt#pack)
			    ()
  in

  let _ = GtkBase.Widget.set_can_focus   save#as_widget true
  and _ = GtkBase.Widget.set_can_default save#as_widget true
  and _ = GtkBase.Widget.set_sensitive   save#as_widget false
  in

  let _ = My_controls.iconLabeled "Enregistrer"
				  (GDraw.pixmap_from_xpm_d ~data:Icones.store
							   ~window
							   ()
				  )
				  ~packing:save#add
				  ()
  in

  (* Bouton Quitter pour fermer la fentre de capture *)
  let quit = GButton.button ~packing:(bottomOpt#pack)
			    ()
  in

  let _ = GtkBase.Widget.set_can_focus   quit#as_widget true
  and _ = GtkBase.Widget.set_can_default quit#as_widget true
  and _ = quit#grab_default ()
  in

  let _ = My_controls.iconLabeled "Quitter"
				  (GDraw.pixmap_from_xpm_d ~data:Icones.quit
							   ~window
							   ()
				  )
				  ~packing:quit#add
				  ()
  in

  (* Bouton permettant d'ouvrir un browser pour slectionner le nom et le chemin du fichier *)
  let select = GButton.button ~packing:(tableOpt#attach ~left:2
							~top:1
							~right:3
							~bottom:2
							~xpadding:0
							~ypadding:0
							~expand:`NONE
							~shrink:`NONE
							~fill:`X
				       )
			      ()
  in

  let _ = GtkBase.Widget.set_can_focus select#as_widget true
  in

  let _ = My_controls.iconLabeled "Slectionner ... "
				  (GDraw.pixmap_from_xpm_d ~data:Icones.browse
							   ~window
							   ()
				  )
				  ~packing:select#add
				  ()
  in

  (* Bouton d'options de sauvegarde pour les formats JPEG et GIF *)
  let opts = GButton.button ~packing:(tableOpt#attach ~left:2
						      ~top:2
						      ~right:3
						      ~bottom:3
						      ~xpadding:0
						      ~ypadding:0
						      ~expand:`NONE
						      ~shrink:`NONE
						      ~fill:`X
				     )
			    ()
  in

  let _ = GtkBase.Widget.set_can_focus opts#as_widget true
  and _ = GtkBase.Widget.hide          opts#as_widget
  in

  let _ = My_controls.iconLabeled "Options"
				  (GDraw.pixmap_from_xpm_d ~data:Icones.option
							   ~window
							   ()
				  )
				  ~packing:opts#add
				  ()
  in

  let _ = GMisc.label ~text:"Rpertoire"
		      ~packing:(tableOpt#attach ~left:0
						~top:0
						~right:1
						~bottom:1
						~xpadding:0
						~ypadding:20
						~expand:`NONE
						~shrink:`NONE
						~fill:`X
			       )
		      ~xalign:0.
		      ~yalign:0.5
		      ~xpad:0
		      ~ypad:0
		      ~line_wrap:false
		      ()
  in

  let _ = GMisc.label ~text:"Nom du fichier"
		      ~packing:(tableOpt#attach ~left:0
						~top:1
						~right:1
						~bottom:2
						~xpadding:0
						~ypadding:20
						~expand:`NONE
						~shrink:`NONE
						~fill:`X
			       )
		      ~xalign:0.
		      ~yalign:0.5
		      ~xpad:0
		      ~ypad:0
		      ~line_wrap:false
		      ()
  in

  let _ = GMisc.label ~text:"Format"
		      ~packing:(tableOpt#attach ~left:0
						~top:2
						~right:1
						~bottom:3
						~xpadding:0
						~ypadding:20
						~expand:`NONE
						~shrink:`NONE
						~fill:`X
			       )
		      ~xalign:0.
		      ~yalign:0.5
		      ~xpad:0
		      ~ypad:0
		      ~line_wrap:false
		      ()
  in

  let _ = GMisc.label ~text:"Taille"
		      ~packing:(tableOpt#attach ~left:0
						~top:3
						~right:1
						~bottom:4
						~xpadding:0
						~ypadding:20
						~expand:`NONE
						~shrink:`NONE
						~fill:`X
			       )
		      ~xalign:0.
		      ~yalign:0.5
		      ~xpad:0
		      ~ypad:0
		      ~line_wrap:false
		      ()
  in

  let sizeRate = GRange.scale `HORIZONTAL
			      ~adjustment:(GData.adjustment ~value:100.
							    ~lower:1.
							    ~upper:100.
							    ~step_incr:1.
							    ~page_incr:10.
							    ~page_size:0.
							    ()
					  )
			      ~packing:(tableOpt#attach ~left:1
							~top:3
							~right:2
							~bottom:4
							~xpadding:0
							~ypadding:20
							~expand:`NONE
							~shrink:`NONE
							~fill:`X
				       )
			      ~digits:0
			      ~draw_value:false
			      ()
  in
  let _ = sizeRate#set_update_policy `CONTINUOUS
  and _ = GtkBase.Widget.set_can_focus sizeRate#as_widget true
  in

  let rate = GMisc.label ~text:(Usefull.rateText 100.0)
			 ~packing:(tableOpt#attach ~left:2
						   ~top:3
						   ~right:3
						   ~bottom:4
						   ~xpadding:0
						   ~ypadding:20
						   ~expand:`NONE
						   ~shrink:`NONE
						   ~fill:`X
				  )
			 ~xalign:0.
			 ~yalign:0.5
			 ~xpad:0
			 ~ypad:0
			 ~line_wrap:false
			 ()
  in



object(self)

  method window      = window
  method preview     = preview
  method drawing     = drawing
  method newCapt     = newCapt
  method choix       = choix
  method path        = path
  method name        = name
  method save        = save
  method quit        = quit
  method select      = select
  method opts        = opts

  method getFileName = Filename.concat path#text name#text

  (* index dterminant quel format d'image est activ dans les tableaux 'extension' et 'formatAssoc' *)
  val mutable ext    = 0

  (* image de la capture d'cran *)
  val mutable img    = None
  (* images de l'aperu *)
  val mutable ximage = None

  (* indique si la sauvegarde peut ou non avoir lieu (nom de fichier rempli) *)
  val mutable savable = false

  val mutable gifEntrelasse = false
  method setEntrelasse b = gifEntrelasse <- b;

  val mutable jpgQuality = 100
  method setQuality i = jpgQuality <- i;

  val mutable titre = ""
  val mutable background = false
  method setTitreBackground t b = titre      <- t;
				  background <- b

  method supprimeImg () = (match img
			   with
			     | None        -> ()
			     | Some oldImg -> Rgb24.destroy oldImg
			  );
			  (match ximage
			   with
			     | None        -> ()
			     | Some oldImg -> Ximage.destroy oldImg
			  )

  (* Gnre une nouvelle capture d'cran *)
  method screenshot () = self#supprimeImg ();
			 img <- Some (Screenshot.saveImage ());
			 self#ajoutDessin ()

  (* Redessine l'apperu de la capture d'cran *)
  method dessiner () = drawing#set_foreground (`RGB (26112, 26112, 26112));
		       drawing#rectangle ~x:0
					 ~y:0
					 ~width:256
					 ~height:256
					 ~filled:true
					 ();
		       if ximage = None              ||
			  formatAssoc.(ext) = Pdf    ||
			  formatAssoc.(ext) = PsVect ||
			  formatAssoc.(ext) = EpsVect
		       then
			 begin
			   let font = preview#misc#style#font
			   and s1 = "Pas d'aperu pour les"
			   and s2 = "formats vectoriels"
			   in
			   let (w1  , h1  ) = Gdk.Font.string_width font s1, Gdk.Font.string_height font s1
			   and (w2  , h2  ) = Gdk.Font.string_width font s2, Gdk.Font.string_height font s2
			   in
			   let marge = (256 - (h1 + h2 + max h1 h2)) / 2
			   in
			     drawing#set_foreground (`WHITE);
			     drawing#string s1 ~font:font ~x:((256 - w1) / 2) ~y:(marge + h1);
			     drawing#string s2 ~font:font ~x:((256 - w2) / 2) ~y:(256 - marge)
			 end
		       else
			 begin
			   let x = unSome ximage
			   in
			     drawing#put_image ~x:(128 - (x.Ximage.width/2))
					       ~y:(128 - (x.Ximage.height/2))
					       ~width:x.Ximage.width
					       ~height:x.Ximage.height
					       ~xsrc:0
					       ~ysrc:0
					       x.Ximage.data
			 end

  (* Met  jour l'image de la zone de preview *)
  method ajoutDessin () = let h = int_of_float !Parametres.width
			  and w = int_of_float !Parametres.height
			  in
			  let maxi = max h w
			  in
			  let newimage = Rgb24.resize None (unSome img) ((252*h)/maxi) ((252*w)/maxi)
			  in 
			    ximage <- Some (Ximage.of_image (preview#misc#visual) None (Image.Rgb24 newimage));
			    self#dessiner ()

  (* Enregistre l'image sur le disque *)
  method sauverImage () = try
			    match formatAssoc.(ext)
			    with
			      | Gif     -> Screenshot.enregImage self#getFileName
								 Image.Gif
								 (if gifEntrelasse
								  then
								    [Image.Save_Interlace]
								  else
								    []
								 )
								 (unSome img)
								 (sizeRate#adjustment#value /. 100.0)

			      | Jpeg    -> Screenshot.enregImage self#getFileName
								 Image.Jpeg
								 [Image.Save_Quality jpgQuality]
								 (unSome img)
								 (sizeRate#adjustment#value /. 100.0)

			      | PsVect
			      | Pdf
			      | EpsVect -> Screenshot.saveVect titre formatAssoc.(ext) background self#getFileName

			      | format  -> Screenshot.enregImage self#getFileName
								 (saveFormat format)
								 []
								 (unSome img)
								 (sizeRate#adjustment#value /. 100.0)
			  with
			    | e -> print_endline (Printexc.to_string e);
				   exit 2

  (* Mthode qui vrifie si les champs du Chemin et du Nom sont remplis. *)
  (* Active le bouton d'enregistrement en fonction.                      *)
  method onChanged () = let sbt = (not (path#text="")) && (not (name#text=""))
			in
			  if (sbt && (not savable)) || ((not sbt) && savable)
			  then
			    begin
			      savable <- not savable;
			      GtkBase.Widget.set_sensitive save#as_widget savable
			    end

  method onQuitter () = self#supprimeImg ();
			window#destroy ()



initializer



  let _ = self#screenshot ()
  and _ = path#set_text (Sys.getcwd ())
  in

  (* Fixe l'action  faire lorsqu'un format de fichier est slectionn *)
  let _ = let itemList = choix#children
	  in
	  let rec activator lst i = match lst
				    with
				      | []   -> ()
				      | t::q -> t#connect#activate ~callback:(fun () -> ext <- i;
											let (nom, typ) = Usefull.splitName name#text
											in
											(* Modifie l'extension du fichier en fonction du format *)
											let new_name = try
													 if guessExtension typ formatAssoc.(ext)
													 then name#text
													 else nom ^ "." ^ (getExtension (formatAssoc.(ext)))
												       with
													 | APas -> if nom = ""
														   then
														     ""
														   else
														     name#text ^ "." ^ (getExtension (formatAssoc.(ext)))
											in
											  name#set_text new_name;
											  (* Pour les formats faisant l'objet d'options, active le bouton d'option *)
											  (* Sinon, le dsactive ...                                               *)
											  match formatAssoc.(i)
											  with
											    | Jpeg   -> GtkBase.Widget.show sizeRate#as_widget;
													GtkBase.Widget.show opts#as_widget;
													rate#set_text (Usefull.rateText sizeRate#adjustment#value);
													self#dessiner ()
											    | Gif
											    | Pdf
											    | EpsVect
											    | PsVect -> GtkBase.Widget.hide sizeRate#as_widget;
													rate#set_text (Usefull.rateText 100.0);
													GtkBase.Widget.show opts#as_widget;
													self#dessiner ()
											    | _      -> GtkBase.Widget.show sizeRate#as_widget;
													GtkBase.Widget.hide opts#as_widget;
													rate#set_text (Usefull.rateText sizeRate#adjustment#value);
													self#dessiner ()
									     );
						activator q (i+1)
	  in
	    activator itemList 0
  in

  (* Ouvre une fentre lorsque le bouton Option est cliqu *)
  let _ = opts#connect#clicked ~callback:(fun () -> match formatAssoc.(ext)
						    with
						      | Gif    -> let opt = new Image_opt.gif_opt gifEntrelasse     self#setEntrelasse      ()
								  in
								   opt#window#show ()
						      | Jpeg   -> let opt = new Image_opt.jpg_opt jpgQuality        self#setQuality         ()
								  in
								    opt#window#show ()
						      | Pdf
						      | EpsVect
						      | PsVect -> let opt = new Image_opt.vect_opt titre background self#setTitreBackground ()
								  in
								    opt#window#show ()
						      | _      -> ()
					 )
  in

  (* Gestion du clavier *)
  let _ = window#event#connect#key_press ~callback:(fun (event: GdkEvent.Key.t) -> begin
										     let key = GdkEvent.Key.keyval event
										     in
										       match key
										       with
											 | t when t=GdkKeysyms._Escape -> self#onQuitter ();
															  true
											 | _                           -> false
										   end
						   )
  in

  let _ = window #connect#remove       ~callback:(fun (widget:GObj.widget) -> self#onQuitter ())
  and _ = quit   #connect#clicked      ~callback:self#onQuitter
  and _ = newCapt#connect#clicked      ~callback:self#screenshot
  and _ = save   #connect#clicked      ~callback:self#sauverImage
  and _ = preview#event#connect#expose ~callback:(fun (ev:GdkEvent.Expose.t) -> self#dessiner ();
										true
						 )
  and _ = path   #connect#changed      ~callback:self#onChanged
  and _ = name   #connect#changed      ~callback:self#onChanged
  and _ = select #connect#clicked      ~callback:(fun () -> let w = new Guifichier.imo_fichier Guifichier.Load 
											       self#path#text 
											       (fun str -> self#path#set_text (Filename.dirname  str);
													   self#name#set_text (Filename.basename str)
											       )
							    in
							      w#window#show ()
						 )
  and _ = sizeRate#adjustment#connect#value_changed ~callback:(fun () -> rate#set_text (Usefull.rateText sizeRate#adjustment#value))
  in
    ()

end



(*************************************************************)
(* Fonction crant une nouvelle fentre de capture d'images. *)
(*************************************************************)
(* Type : unit -> imo_image                                  *)
(*************************************************************)
let capture () = let w = new imo_image ()
		 in
		   w#window#show ();
		   w#dessiner ()
