open Types
open GMain
open Viewport
open Core
    
class imo_window () =

  let tooltips    = GData.tooltips ()
  and accel_group = GtkData.AccelGroup.create ()
  in

  let window = GWindow.window ~wm_name:"Imoview"
			      ~title:"Imoview"
			      ~position:`MOUSE
			      ~kind:`TOPLEVEL
			      ~modal:false
			      ~allow_shrink:true
			      ~allow_grow:true
			      ~auto_shrink:true
			      ()
  in

  let _ =  window#add_accel_group accel_group
  in

  let vbox = GPack.vbox ~spacing:0
			~homogeneous:false
			~packing:window#add
			()
  in


(******************************************************************************)
(*                                    MENU                                    *)
(******************************************************************************)

  let menubar = GMenu.menu_bar ~packing:(vbox#pack ~padding:0
						   ~fill:false
						   ~expand:false
					)
			       ()
  in


  (* Menu Fichier *)
  let fichier = GMenu.menu_item ~label:"Fichier"
				~packing:menubar#add
				()
  in
  let fichier_menu = GMenu.menu ~packing:fichier#set_submenu
				()
  in

  (* Item Ouvrir *)
  let ouvrir = GMenu.menu_item ~label:"Ouvrir"
			       ~packing:fichier_menu#add
			       ()
  in
  let _ = tooltips#set_tip ~text:"Ouvre un nouveau fichier"
			   ouvrir#coerce
  in

  (* Item Quitter *)
  let quitter = GMenu.menu_item ~label:"Quitter"
				~packing:fichier_menu#add
				()
  in
(*  let _ = My_controls.iconLabeled "Quitter"
				  (GDraw.pixmap_from_xpm_d ~data:Icones.quit
							   ~window
							   ()
				  )
				  ~packing:quitter#add
				  ()
  in*)
  let _ = tooltips#set_tip ~text:"Quitter l'application :'("
			   quitter#coerce
  in


  (* Menu Options *)
  let option = GMenu.menu_item ~label:"Options"
			       ~packing:menubar#add
			       ()
  in


  (* Menu Capture *)
  let capture = GMenu.menu_item ~label:"Capture"
				~packing:menubar#add
				()
  in
  let capture_menu = GMenu.menu ~packing:capture#set_submenu
				()
  in


  (* Item Film *)
  let film = GMenu.menu_item ~packing:capture_menu#add
			     ()
  in
  let _ = My_controls.iconLabeled "Film"
				  (GDraw.pixmap_from_xpm_d ~data:Icones.film
							   ~window
							   ()
				  )
				  ~packing:film#add
				  ()
  in
  let _ = tooltips#set_tip ~text:"Enregistrer une animation"
			   film#coerce
  in

  (* Item Image *)
  let image = GMenu.menu_item ~packing:capture_menu#add
			      ()
  in
  let _ = My_controls.iconLabeled "Image"
				  (GDraw.pixmap_from_xpm_d ~data:Icones.shot16
							   ~window
							   ()
				  )
				  ~packing:image#add
				  ()
  in
  let _ = tooltips#set_tip ~text:"Sauvegarder une image"
			   image#coerce
  in


  (* Menu A propos *)
  let apropos = GMenu.menu_item ~label:"A propos"
				~packing:menubar#add
				()
  in
  let apropos_menu = GMenu.menu ~packing:apropos#set_submenu
				()
  in

  (* Item A propos *)
  let apropos_item = GMenu.menu_item ~label:"A propos ..."
				     ~packing:apropos_menu#add
				     ()
  in

  let _ = quitter#misc#add_accelerator ~group:accel_group
				       GdkKeysyms._Escape
				       ~sgn:{
					      GtkSignal.name       = "activate";
					      GtkSignal.marshaller = GtkSignal.marshal_unit ;
					      GtkSignal.classe     = `base
					    }
				       ~modi:[]
  in


(******************************************************************************)
(*                                   TOOLBAR                                  *)
(******************************************************************************)

  let toolbar = GButton.toolbar ~packing:(vbox#pack ~padding:0
						    ~fill:false
						    ~expand:false
					 )
				~orientation:`HORIZONTAL
				~button_relief:`NORMAL
				~tooltips:true
				~style:`ICONS
				~space_size:8
				~space_style:`EMPTY
				()
  in

  (* Cration de vidos *)
  let button_film = toolbar#insert_button ~text:"Film"
					  ~tooltip:"Cration d'une vido"
					  ~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.film
										       ~window
										       ()
							      )
							      ()
						)#coerce
					  ()

  (* Capture d'cran *)
  and button_image = toolbar#insert_button ~text:"Image"
					   ~tooltip:"Sauver une image"
					   ~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.shot16
											~window
											()
							       )
							       ()
						 )#coerce
					   ()

  and _ = toolbar#insert_widget (GMisc.separator `VERTICAL ())#coerce

  and _ = toolbar#insert_space ()

  (* Retour au dbut de l'animation *)
  and button_arriere = toolbar#insert_button ~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.debut
											  ~window
											  ()
								 )
								 ()
						   )#coerce
					     ()

  (* Recule d'une commmande *)
  and button_step_arr = toolbar#insert_button ~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.back_step
											   ~window
											   ()
								  )
								  ()
						    )#coerce
					      ()

  (* Lecture arrire de l'animation *)
  and button_lecture_arr = toolbar#insert_button ~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.lecture_arr
											      ~window
											      ()
								     )
								     ()
						       )#coerce
						 ()

  (* Arrt de l'animation *)
  and button_stop = toolbar#insert_button ~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.arret
										       ~window
										       ()
							      )
							      ()
						)#coerce
					  ()

  (* Met en pause l'animation *)
  and button_pause = toolbar#insert_button ~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.pause
											~window
											()
							       )
							       ()
						 )#coerce
					   ()

  (* Lecture de l'animation *)
  and button_lecture = toolbar#insert_button ~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.lecture
											  ~window
											  ()
								 )
								 ()
						   )#coerce
					     ()

  (* Avance d'une commmande *)
  and button_step = toolbar#insert_button ~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.step
										       ~window
										       ()
							      )
							      ()
						)#coerce
					  ()

  (* Va  la fin de l'animation *)
  and button_avance = toolbar#insert_button ~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.fin
											 ~window
											 ()
								)
								()
						  )#coerce
					    ()

  and _ = toolbar#insert_space ()

  and axeText () = if view#getAfficheAxe
		   then
		     "Masquer les axes"
		   else
		     "Afficher les axes"

  and fogText () = if view#getBrouillard
		   then
		     "Dsactiver le brouillard"
		   else
		     "Activer le brouillard"

  and lumText () = if view#getLighting
		   then
		     "Dsactiver la lumire"
		   else
		     "Activer la lumire"

  in

  let button_axe = toolbar#insert_toggle_button ~tooltip:(axeText ())
						~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.axe
											     ~window
											      ()
								    )
								    ()
						      )#coerce
						()

  and button_fog = toolbar#insert_toggle_button ~tooltip:(fogText ())
						~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.fog
											     ~window
											      ()
								    )
								    ()
						      )#coerce
						()

  and button_lum = toolbar#insert_toggle_button ~tooltip:(lumText ())
						~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.light
											     ~window
											      ()
								    )
								    ()
						      )#coerce
						()

  and button_point = toolbar#insert_toggle_button ~tooltip:"Mode point"
						  ~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.point
											       ~window
											        ()
								      )
								      ()
							)#coerce
						  ()

  and button_wire = toolbar#insert_toggle_button ~tooltip:"Mode filaire"
						 ~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.wire
											      ~window
											       ()
								     )
								     ()
						       )#coerce
						 ()

  and button_face = toolbar#insert_toggle_button ~tooltip:"Mode plein"
						 ~icon:(GMisc.pixmap (GDraw.pixmap_from_xpm_d ~data:Icones.face
											      ~window
											       ()
								     )
								     ()
						       )#coerce
						 ()

  in


(******************************************************************************)
(*                             CORP DE LA FENETRE                             *)
(******************************************************************************)

  (* Aire OpenGL *)
  let drawingarea = GlGtk.area (if view#getStereo
				then [ `USE_GL ; `RGBA ; `DEPTH_SIZE 1 ; `DOUBLEBUFFER ; `STEREO ]
				else [ `USE_GL ; `RGBA ; `DEPTH_SIZE 1 ; `DOUBLEBUFFER ]
			       )
			       ~width:(int_of_float !(Parametres.width))
			       ~height:(int_of_float !(Parametres.height))
			       ~packing:(vbox#pack ~padding:0
						   ~fill:true
						   ~expand:true
					)
			       ()
  in
  let _ = drawingarea#event#add [`EXPOSURE ; `BUTTON_MOTION ; `POINTER_MOTION_HINT ; `BUTTON_PRESS ; `BUTTON_RELEASE]
  in

  (* Barre d'tat pour l'volution de l'animation *)
  let statusbar = GMisc.statusbar ~packing:(vbox#pack ~padding:0
						      ~fill:false
						      ~expand:false
					   )
				  ()
  in
  let statusContext = statusbar#new_context "Statut"
  in
  let _ = statusContext#push "Stop"
  in

(*********************************************************************************************)

                                  
object(self)

  method tooltips      = tooltips
  method window        = window
  method accel_group   = accel_group
  method vbox          = vbox
  method menubar       = menubar
  method toolbar       = toolbar
  method button_film   = button_film
  method button_image  = button_image
  method drawingarea   = drawingarea
  method fichier       = fichier
  method quitter       = quitter
  method on_quitter () = exit 0

  (* Mthode qui crit une chaine de caractres [chaine] sur la barre de statut *)
  method ecrireStatut str = statusContext#pop ();
			    ignore (statusContext#push str);
                                                  
  method animate etat sens () = match core#getState
				with
				  | Mvt -> core#setNextState (Some (etat, sens))

				  | _   -> core#animToString;
					   core#setState      etat;
					   core#setDir        sens;
					   core#setNextState  None;
					   core#setTimer     (Some (GMain.Timeout.add ~ms:10
										      ~callback:(core#checkNewState)
									))

  initializer

    Parametres.ecrireStatut := self#ecrireStatut;
    Parametres.swapbuffers  := drawingarea#swap_buffers;
    Parametres.area         := Some drawingarea;

    button_axe#set_active view#getAfficheAxe;
    button_fog#set_active view#getBrouillard;
    button_lum#set_active view#getLighting;

    (match view#getPolygonMode
     with
       | `point -> button_point#set_active true;
       | `line  -> button_wire #set_active true;
       | `fill  -> button_face #set_active true;
    );


(******************************************************************************)
(*                                  CALLBACK                                  *)
(******************************************************************************)

    let _ = quitter#connect#activate ~callback:self#on_quitter

    and _ = film#connect#activate ~callback:Guivideo.video

    and _ = image#connect#activate ~callback:Guiimage.capture

    and _ = option#connect#activate ~callback:(fun () -> let w = new Guioptions.imo_options ()
							 in
							 let _ = w#window#show ()
							 in
							   ()
					      )

    and _ = apropos_item#connect#activate ~callback:(fun () -> let w = new Guiapropos.imo_apropos ()
							       in
							       let _ = w#window#show ()
							       in
								 ()
						    )

    and _ = ouvrir#connect#activate ~callback:(fun () -> let w = new Guifichier.imo_fichier Guifichier.Load 
											    ""
											    ( fun str -> let ancien = !Parametres.file_name
													 in
													   Parametres.file_name := str;
													   try
													     try
													       Load.load_file ();
													       view#draw      ()
													     with
													       | _         -> Parametres.file_name := ancien;
															      Load.load_file ()
													   with
													     | _ -> ()
											    )
							 in
							   w#window#show ()
					      )

    and _ = window#connect#remove ~callback:(fun (widget:GObj.widget) -> self#on_quitter ())

    and _ = button_film#connect#clicked ~callback:Guivideo.video

    and _ = button_image#connect#clicked ~callback:Guiimage.capture

    and _ = button_lecture    #connect#clicked ~callback:(self#animate Lct   Avant           )
    and _ = button_lecture_arr#connect#clicked ~callback:(self#animate Lct   Arriere         )
    and _ = button_step_arr   #connect#clicked ~callback:(self#animate Step  Arriere         )
    and _ = button_step       #connect#clicked ~callback:(self#animate Step  Avant           )
    and _ = button_stop       #connect#clicked ~callback:(self#animate Stop  Avant           )
    and _ = button_pause      #connect#clicked ~callback:(self#animate Tempo core#getDir     )
    and _ = button_arriere    #connect#clicked ~callback:(self#animate Rapide Arriere        )
    and _ = button_avance     #connect#clicked ~callback:(self#animate Rapide Avant          )

    and _ = button_axe#connect#toggled ~callback:(fun () -> view#setAfficheAxe (not view#getAfficheAxe);
							    tooltips#set_tip ~text:(axeText ())
									     button_axe#coerce;
							    if core#getState <> Mvt
							    then
							      view#draw ()
						 )
    and _ = button_fog#connect#toggled ~callback:(fun () -> view#setBrouillard (not view#getBrouillard);
							    tooltips#set_tip ~text:(fogText ())
									     button_fog#coerce;
							    if core#getState <> Mvt
							    then
							      view#draw ()
						 )
    and _ = button_lum#connect#toggled ~callback:(fun () -> view#setLighting (not view#getLighting);
							    tooltips#set_tip ~text:(lumText ())
									     button_lum#coerce;
							    if core#getState <> Mvt
							    then
							      view#draw ()
						 )
    and _ = button_point#connect#toggled ~callback:(fun () -> if view#getPolygonMode = `point
							      then
								begin
								  if not button_point#active
								  then
								    button_point#set_active true
								end
							      else
								begin
								  if button_point#active
								  then
								    begin
								      view#setPolygonMode `point;
								      button_wire#set_active false;
								      button_face#set_active false;
								      GlDraw.cull_face `back;
								      Gl.disable `cull_face;
								      GlDraw.polygon_mode ~face:`both `point;
								      if core#getState <> Mvt
								      then
									view#draw ()
								    end
								end
						   )
    and _ = button_wire#connect#toggled ~callback:(fun () -> if view#getPolygonMode = `line
							     then
							       begin
								 if not button_wire#active
								 then
								   button_wire#set_active true
							       end
							     else
							       begin
								 if button_wire#active
								 then
								   begin
								     view#setPolygonMode `line;
								     button_point#set_active false;
								     button_face#set_active false;
								     GlDraw.cull_face `back;
								     Gl.disable `cull_face;
								     GlDraw.polygon_mode ~face:`both `line;
								     if core#getState <> Mvt
								     then
								       view#draw ()
								   end
							       end
						  )
    and _ = button_face#connect#toggled ~callback:(fun () -> if view#getPolygonMode = `fill
							     then
							       begin
								 if not button_face#active
								 then
								   button_face#set_active true
							       end
							     else
							       begin
								 if button_face#active
								 then
								   begin
								     view#setPolygonMode `fill;
								     button_point#set_active false;
								     button_wire#set_active false;
								     GlDraw.cull_face `back;
								     Gl.enable `cull_face;
								     GlDraw.polygon_mode ~face:`both `fill;
								     if core#getState <> Mvt
								     then
								       view#draw ()
								   end
							       end
						  )

    and _ = drawingarea#connect#realize ~callback:Opengl.init_gl
    and _ = drawingarea#connect#reshape ~callback:Opengl.reshape
    and _ = drawingarea#connect#display ~callback:view#draw
    and _ = drawingarea#event#connect#button_press   ~callback:(Mouse.mouseClic DOWN)
    and _ = drawingarea#event#connect#button_release ~callback:(Mouse.mouseClic UP)
    and _ = drawingarea#event#connect#motion_notify  ~callback:Mouse.mouseMove
    in

    (* Gestion du clavier *)
    let _ =window#event#connect#key_press ~callback:(fun (event:GdkEvent.Key.t) ->
						       begin
							 let key = GdkEvent.Key.keyval event
							 in
							   match key
							   with
							     | t when t=GdkKeysyms._F1  -> button_face#clicked ();
											   true

							     | t when t=GdkKeysyms._F2  -> button_wire#clicked ();
											   true

							     | t when t=GdkKeysyms._F3  -> button_point#clicked ();
											   true

							     | t when t=GdkKeysyms._F4  -> button_axe#clicked ();
											   true

							     | t when t=GdkKeysyms._F5  -> Load.load_file ();
											   view#draw();
											   true

							     | t when t=GdkKeysyms._Escape -> exit 0;
											      true

							     | t when t=GdkKeysyms._Page_Down -> (* Diminue d'cartement des yeux *)
												 view#setInterYeux (view#getInterYeux *. 0.9);
												 view#draw();
												 true

							     | t when t=GdkKeysyms._Page_Up -> (* Augmente d'cartement des yeux *)
											       view#setInterYeux (view#getInterYeux *. 1.1);
											       view#draw();
											       true

							     | t when t=GdkKeysyms._L || t=GdkKeysyms._l -> button_lum#clicked ();
													    true

							     | t when t=GdkKeysyms._F || t=GdkKeysyms._f -> button_fog#clicked ();
													    true

							     | t when t=GdkKeysyms._7 || t=GdkKeysyms._egrave -> view#setGrille1 AxeY;
														 view#setGrille2 AxeZ;
														 view#draw();
														 true

							     | t when t=GdkKeysyms._8 || t=GdkKeysyms._underscore -> view#setGrille1 AxeZ;
														     view#setGrille2 AxeX;
														     view#draw();
														     true

							     | t when t=GdkKeysyms._9 || t=GdkKeysyms._ccedilla -> view#setGrille1 AxeX;
														   view#setGrille2 AxeY;
														   view#draw();
														   true

							     | t when t=GdkKeysyms._0 || t=GdkKeysyms._agrave -> view#setGrille1 AxeNone;
														 view#setGrille2 AxeNone;
														 view#draw();
														 true

							     | t when t=GdkKeysyms._1 || t=GdkKeysyms._ampersand -> view#setGrille1 AxeNone;
														    view#setSpin [| [| 1.0 ; 0.0 ; 0.0 ; 0.0 |];
																    [| 0.0 ; 1.0 ; 0.0 ; 0.0 |];
																    [| 0.0 ; 0.0 ; 1.0 ; 0.0 |];
																    [| 0.0 ; 0.0 ; 0.0 ; 1.0 |]
																 |];
														    view#getVectX#setAll (0.0, 1.0, 0.0);
														    view#getVectY#setAll (0.0, 0.0, 1.0);
														    view#getVectZ#setAll (1.0, 0.0, 0.0);
														    view#getCentre#setAll (0.0, 0.0, 0.0);
														    view#setZoom view#getRayon;
														    view#draw();
														    true

                                                             | t when t=GdkKeysyms._2 || t=GdkKeysyms._eacute -> view#setGrille1 AxeNone;
														 view#setSpin ([| [| 1.0 ; 0.0 ; 0.0 ; 0.0 |];
																  [| 0.0 ; 1.0 ; 0.0 ; 0.0 |];
																  [| 0.0 ; 0.0 ; 1.0 ; 0.0 |];
																  [| 0.0 ; 0.0 ; 0.0 ; 1.0 |]
															       |]);
														 view#getVectX#setAll (0.0, 0.0, 1.0);
														 view#getVectY#setAll (1.0, 0.0, 0.0);
														 view#getVectZ#setAll (0.0, 1.0, 0.0);
														 view#getCentre#setAll (0.0, 0.0, 0.0);
														 view#setZoom view#getRayon;
														 view#draw();
														 true

                                                             | t when t=GdkKeysyms._3 || t=GdkKeysyms._quotedbl -> view#setGrille1 AxeNone;
														   view#setSpin ([| [| 1.0 ; 0.0 ; 0.0 ; 0.0 |];
																    [| 0.0 ; 1.0 ; 0.0 ; 0.0 |];
																    [| 0.0 ; 0.0 ; 1.0 ; 0.0 |];
																    [| 0.0 ; 0.0 ; 0.0 ; 1.0 |]
																 |]);
														   view#getVectX#setAll (1.0, 0.0, 0.0);
														   view#getVectY#setAll (0.0, 1.0, 0.0);
														   view#getVectZ#setAll (0.0, 0.0, 1.0);
														   view#getCentre#setAll (0.0, 0.0, 0.0);
														   view#setZoom view#getRayon;
														   view#draw();
														   true

                                                             | t when t=GdkKeysyms._space -> self#animate Lct core#getDir ();
											     true

                                                             | t when t=GdkKeysyms._e || t=GdkKeysyms._E -> window#set_default_size ~width:(Gdk.Screen.width ())
																    ~height:(Gdk.Screen.height ());
													    window#set_position `CENTER ;
													    window#set_position `CENTER_ALWAYS ;
													    view#draw();
													    true
							     | _ -> false
						       end
						    )
    in
      ()

end
