(******************************************)
(*                                        *)
(* Le fichier guivideo.ml permet d'ouvrir *)
(* la fentre de cration des vidos.     *)
(*                                        *)
(******************************************)



open Types
open Image



type format =
  | Avi
  | GifAnim
  | LstBmp
  | LstJpg
  | LstPng



(************************************************)
(* Renvoie le format de CamlImages associ  f. *)
(* Lve l'exception Types.APas si ce format     *)
(* n'est pas support.                          *)
(************************************************)
(* Type : format -> Image.format                *)
(************************************************)
let saveFormat f = match f
		   with
		     | LstBmp -> Image.Bmp
		     | LstJpg -> Image.Jpeg
		     | LstPng -> Image.Png
		     | _      -> raise APas



(************************************)
(* Dtermine si l'extension ext est *)
(* approprie pour le format formt. *)
(************************************)
(* Type : format -> string -> bool  *)
(************************************)
let guessExtension ext formt = let ext = String.lowercase ext
			       in
				match ext, formt
				with
				  |  "gif", GifAnim
				  |  "avi", Avi
				  |  "bmp", LstBmp
				  |  "png", LstPng
				  |  "jpg", LstJpg
				  | "jpeg", LstJpg  -> true
				  |      _, _       -> false



(***************************)
(* Renvoie l'extension     *)
(* associe au format f.   *)
(***************************)
(* Type : format -> string *)
(***************************)
let getExtension f = match f
		     with
		       | Avi     -> "avi"
		       | GifAnim -> "gif"
		       | LstBmp  -> "bmp"
		       | LstJpg  -> "jpg"
		       | LstPng  -> "png"



(* Tableau des formats d'images grs pour les captures. *)
(*  -> blabla  afficher dans le menu                    *)
let extension = [|"Vido (.avi)"          ;
		  "GIF anim (.gif)"      ;
		  "Liste d'images Bitmap" ;
		  "Liste d'images Jpeg"   ;
		  "Liste d'images Png"
		|]



(* Tableau des formats d'images grs pour les captures.                                *)
(*  -> format associe (de type Image.format) associ au texte contenu dans 'extension' *)
let formatAssoc = [| Avi     ;
		     GifAnim ;
		     LstBmp  ;
		     LstJpg  ;
		     LstPng
		   |]



(*******************************************************)
(* Classe dfinissant la fentre de cration de vidos *)
(* Ne pas oublier d'appeler la mthode #window#show () *)
(* pour afficher la fentre.                           *)
(*******************************************************)
class imo_video () =

  let window = GWindow.window ~wm_name:"Cration de vidos"
			      ~title:"Imoview - Cration de vidos"
			      ~position:`CENTER
			      ~kind:`DIALOG
			      ~modal:false
			      ~allow_shrink:false
			      ~allow_grow:false
			      ~auto_shrink:false
			      ()
  in

  let icoBrowse = GDraw.pixmap_from_xpm_d ~data:Icones.browse
					  ~window
					  ()
  in

  let vbox = GPack.vbox ~spacing:0
			~homogeneous:false
			~packing:window#add
			~border_width:10
			()
  in

  let dest = GBin.frame ~packing:(vbox#pack ~padding:10
					    ~expand:true
					    ~fill:true
				 )
			~border_width:5
			~label:"Destination"
			~shadow_type:`ETCHED_IN
			()
  in

  let table_dest = GPack.table ~rows:2
			       ~columns:3
			       ~row_spacings:10
			       ~col_spacings:10
			       ~border_width:10
			       ~homogeneous:false
			       ~packing:dest#add
			       ()
  in

  (* Zone de saisie du chemin du fichier destination *)
  let path = GEdit.entry ~packing:(table_dest#attach ~left:1
						     ~top:0
						     ~right:2
						     ~bottom:1
						     ~xpadding:0
						     ~ypadding:0
						     ~expand:`X
						     ~shrink:`NONE
						     ~fill:`X
				  )
			 ~max_length:0
			 ~visibility:true
			 ~editable:true
			 ()
  in

  let _ = GtkBase.Widget.set_can_focus path#as_widget true
  in

  (* Zone de saisie du nom du fichier destination *)
  let name = GEdit.entry ~packing:(table_dest#attach ~left:1
						     ~top:1
						     ~right:2
						     ~bottom:2
						     ~xpadding:0
						     ~ypadding:0
						     ~expand:`X
						     ~shrink:`NONE
						     ~fill:`X
				  )
			 ~max_length:0
			 ~visibility:true
			 ~editable:true
			 ()
  in

  let _ = GtkBase.Widget.set_can_focus name#as_widget true
  in

  (* Bouton permettant d'ouvrir un browser pour slectionner le nom et le chemin du fichier *)
  let select = GButton.button ~packing:(table_dest#attach ~left:2
							  ~top:1
							  ~right:3
							  ~bottom:2
							  ~xpadding:0
							  ~ypadding:0
							  ~expand:`NONE
							  ~shrink:`NONE
							  ~fill:`X
				       )
			      ()
  in

  let _ = GtkBase.Widget.set_can_focus select#as_widget true
  in

  let _ = My_controls.iconLabeled "Slectionner ... "
				  icoBrowse
				  ~packing:select#add
				  ()
  in

  let _ = GMisc.label ~text:"Rpertoire"
		      ~packing:(table_dest#attach ~left:0
						  ~top:0
						  ~right:1
						  ~bottom:1
						  ~xpadding:0
						  ~ypadding:10
						  ~expand:`NONE
						  ~shrink:`NONE
						  ~fill:`X
			       )
		      ~xalign:0.
		      ~yalign:0.5
		      ~xpad:0
		      ~ypad:0
		      ~line_wrap:false
		      ()
  in

  let _ = GMisc.label ~text:"Nom du fichier"
		      ~packing:(table_dest#attach ~left:0
						  ~top:1
						  ~right:1
						  ~bottom:2
						  ~xpadding:0
						  ~ypadding:10
						  ~expand:`NONE
						  ~shrink:`NONE
						  ~fill:`X
			       )
		      ~xalign:0.
		      ~yalign:0.5
		      ~xpad:0
		      ~ypad:0
		      ~line_wrap:false
		      ()
  in

  let format = GBin.frame ~packing:(vbox#pack ~padding:10
					      ~expand:true
					      ~fill:true
				   )
			  ~border_width:5
			  ~label:"Format"
			  ~shadow_type:`ETCHED_IN
			  ()
  in

  let hboxFormat = GPack.hbox ~spacing:10
			      ~homogeneous:false
			      ~border_width:10
			      ~packing:format#add
			      ()
  in

  let _ = GMisc.label ~text:"Format de l'animation :"
		      ~packing:(hboxFormat#pack ~padding:0
						~expand:false
						~fill:true
			       )
		      ~xalign:0.
		      ~yalign:0.5
		      ~xpad:0
		      ~ypad:0
		      ~line_wrap:false
		      ()
  in

  (* Menu de slection du format d'image *)
  let formatMenu = GMenu.option_menu ~packing:(hboxFormat#pack ~padding:0
							       ~expand:false
							       ~fill:false
					      )
				     ()
  in

  let choix = GMenu.menu ~show:true ()
  in

  (* On insert les items dans le menu option.                      *)
  (* Les items contiennent le texte dcrivant les formats d'images *)
  let _ = (for i = 0 to (Array.length formatAssoc) - 1
	   do
	     let item = GMenu.menu_item ~label:extension.(i) ()
	     in
	       choix#append item
	   done
	  )
  and _ = formatMenu#set_menu choix
  and _ = formatMenu#set_history 0
  and _ = GtkBase.Widget.set_can_focus formatMenu#as_widget true
  in

  let options = GBin.frame ~packing:(vbox#pack ~padding:10
					       ~expand:true
					       ~fill:true
				    )
			   ~border_width:5
			   ~label:"Options"
			   ~shadow_type:`ETCHED_IN
			   ()
  in

  let tableOpt = GPack.table ~rows:6
			     ~columns:3
			     ~row_spacings:10
			     ~col_spacings:10
			     ~border_width:10
			     ~homogeneous:false
			     ~packing:options#add
			     ()
  in

  let _ = GMisc.label ~text:"Taille"
		      ~packing:(tableOpt#attach ~left:0
						~top:0
						~right:1
						~bottom:1
						~xpadding:0
						~ypadding:10
						~expand:`NONE
						~shrink:`NONE
						~fill:`X
			       )
		      ~xalign:0.
		      ~yalign:0.5
		      ~xpad:0
		      ~ypad:0
		      ~line_wrap:false
		      ()
  in

  let sizeRate = GRange.scale `HORIZONTAL
			      ~adjustment:(GData.adjustment ~value:100.
							    ~lower:1.
							    ~upper:100.
							    ~step_incr:1.
							    ~page_incr:10.
							    ~page_size:0.
							    ()
					  )
			      ~packing:(tableOpt#attach ~left:1
							~top:0
							~right:2
							~bottom:1
							~xpadding:0
							~ypadding:0
							~expand:`NONE
							~shrink:`NONE
							~fill:`X
				       )
			      ~digits:0
			      ~draw_value:false
			      ()
  in
  let _ = sizeRate#set_update_policy `CONTINUOUS
  and _ = GtkBase.Widget.set_can_focus sizeRate#as_widget true
  in

  let rate = GMisc.label ~text:(Usefull.rateText 100.0)
			 ~packing:(tableOpt#attach ~left:2
						   ~top:0
						   ~right:3
						   ~bottom:1
						   ~xpadding:0
						   ~ypadding:0
						   ~expand:`NONE
						   ~shrink:`NONE
						   ~fill:`X
				  )
			 ~xalign:0.
			 ~yalign:0.5
			 ~xpad:0
			 ~ypad:0
			 ~line_wrap:false
			 ()
  in

  let _ = GMisc.label ~text:"Nombre d'images par secondes (FPS)"
		      ~packing:(tableOpt#attach ~left:0
						~top:1
						~right:1
						~bottom:2
						~xpadding:0
						~ypadding:5
						~expand:`NONE
						~shrink:`NONE
						~fill:`X
			       )
		      ~xalign:0.
		      ~yalign:0.5
		      ~xpad:0
		      ~ypad:0
		      ~line_wrap:false
		      ()
  in

  let spinFPS = GEdit.spin_button ~adjustment:(GData.adjustment ~value:25.
								~lower:1.
								~upper:1000.
								~step_incr:1.
								~page_incr:10.
								~page_size:10.
								()
					      )
				  ~packing:(tableOpt#attach ~left:1
							    ~top:1
							    ~right:2
							    ~bottom:2
							    ~xpadding:0
							    ~ypadding:0
							    ~expand:`NONE
							    ~shrink:`NONE
							    ~fill:`X
					   )
				  ~rate:0.5
				  ~digits:0
				  ~update_policy:`IF_VALID
				  ~numeric:true
				  ~wrap:false
				  ~snap_to_ticks:false
				  ()
  in

  let bla2Wait = GMisc.label ~text:"Intervale de temps entre 2 commandes"
			     ~packing:(tableOpt#attach ~left:0
						       ~top:2
						       ~right:1
						       ~bottom:3
						       ~xpadding:0
						       ~ypadding:5
						       ~expand:`NONE
						       ~shrink:`NONE
						       ~fill:`X
				      )
			     ~xalign:0.
			     ~yalign:0.5
			     ~xpad:0
			     ~ypad:0
			     ~line_wrap:false
			     ()
  in

  let spinWait = GEdit.spin_button ~adjustment:(GData.adjustment ~value:0.
								 ~lower:0.
								 ~upper:3600.
								 ~step_incr:0.01
								 ~page_incr:1.
								 ~page_size:10.
								 ()
					       )
				   ~packing:(tableOpt#attach ~left:1
							     ~top:2
							     ~right:2
							     ~bottom:3
							     ~xpadding:0
							     ~ypadding:0
							     ~expand:`NONE
							     ~shrink:`NONE
							     ~fill:`X
					    )
				   ~rate:0.5
				   ~digits:2
				   ~update_policy:`IF_VALID
				   ~numeric:true
				   ~wrap:false
				   ~snap_to_ticks:false
				   ()
  in

  let bla2Wait2 = GMisc.label ~text:"seconde(s)"
			      ~packing:(tableOpt#attach ~left:2
							~top:2
							~right:3
							~bottom:3
							~xpadding:0
							~ypadding:0
							~expand:`NONE
							~shrink:`NONE
							~fill:`X
				       )
			      ~xalign:0.
			      ~yalign:0.5
			      ~xpad:0
			      ~ypad:0
			      ~line_wrap:false
			      ()
  in

  let noPause = GButton.radio_button ~label:"Supprimer les pauses"
				     ~active:true
				     ~draw_indicator:true
				     ~packing:(tableOpt#attach ~left:0
							       ~top:3
							       ~right:3
							       ~bottom:4
							       ~xpadding:0
							       ~ypadding:5
							       ~expand:`NONE
							       ~shrink:`NONE
							       ~fill:`X
					      )
				     ()
  in

  let setPause = GButton.radio_button ~group:noPause#group
				      ~active:false
				      ~draw_indicator:true
				      ~packing:(tableOpt#attach ~left:0
								~top:4
								~right:3
								~bottom:5
								~xpadding:0
								~ypadding:5
								~expand:`NONE
								~shrink:`NONE
								~fill:`X
					      )
				     ()
  in

  let hboxPause = GPack.hbox ~spacing:5
			     ~homogeneous:false
			     ~packing:setPause#add
			     ()
  in

  let _ = GMisc.label ~text:"Remplacer les pauses par des temps ports de"
		      ~packing:(hboxPause#pack ~padding:0
					       ~expand:true
					       ~fill:true
			       )
		      ~xalign:0.
		      ~yalign:0.5
		      ~xpad:0
		      ~ypad:0
		      ~line_wrap:false
		      ()
  in

  let spinPause = GEdit.spin_button ~adjustment:(GData.adjustment ~value:0.
								  ~lower:0.
								  ~upper:3600.
								  ~step_incr:0.01
								  ~page_incr:1.
								  ~page_size:10.
								  ()
						)
				    ~packing:(hboxPause#pack ~padding:0
							     ~expand:true
							     ~fill:true
					     )
				    ~rate:0.5
				    ~digits:2
				    ~update_policy:`IF_VALID
				    ~numeric:true
				    ~wrap:false
				    ~snap_to_ticks:false
				    ()
  in

  let _ = GMisc.label ~text:"seconde(s)"
		      ~packing:(hboxPause#pack ~padding:0
					       ~expand:true
					       ~fill:true
			       )
		      ~xalign:0.
		      ~yalign:0.5
		      ~xpad:0
		      ~ypad:0
		      ~line_wrap:false
		      ()
  in

  let bla2Tmp = GMisc.label ~text:"Emplacement des fichiers temporaires"
			    ~packing:(tableOpt#attach ~left:0
						      ~top:5
						      ~right:1
						      ~bottom:6
						      ~xpadding:0
						      ~ypadding:10
						      ~expand:`NONE
						      ~shrink:`NONE
						      ~fill:`X
				     )
			    ~xalign:0.
			    ~yalign:0.5
			    ~xpad:0
			    ~ypad:0
			    ~line_wrap:false
			    ()
  in

  (* Zone de saisie du nom du fichier destination *)
  let pathTmp = GEdit.entry ~packing:(tableOpt#attach ~left:1
						      ~top:5
						      ~right:2
						      ~bottom:6
						      ~xpadding:0
						      ~ypadding:0
						      ~expand:`X
						      ~shrink:`NONE
						      ~fill:`X
				     )
			    ~max_length:0
			    ~visibility:true
			    ~editable:true
			    ()
  in

  let _ = GtkBase.Widget.set_can_focus name#as_widget true
  in

  (* Bouton permettant d'ouvrir un browser pour slectionner le rpertoire des fichier temporaires *)
  let selectTmp = GButton.button ~packing:(tableOpt#attach ~left:2
							   ~top:5
							   ~right:3
							   ~bottom:6
							   ~xpadding:0
							   ~ypadding:0
							   ~expand:`NONE
							   ~shrink:`NONE
							   ~fill:`X
					  )
				 ()
  in

  let _ = GtkBase.Widget.set_can_focus selectTmp#as_widget true
  in

  let _ = My_controls.iconLabeled "Slectionner ... "
				  icoBrowse
				  ~packing:selectTmp#add
				  ()
  in

  let _ = GMisc.separator `HORIZONTAL
			  ~packing:(vbox#pack ~padding:10
					      ~expand:true
					      ~fill:true
				   )
			  ()
  in

  let bottom = GPack.button_box `HORIZONTAL
				~spacing:30
				~child_width:85
				~child_height:24
				~layout:`SPREAD
				~packing:(vbox#pack ~padding:10
						    ~expand:true
						    ~fill:true
					 )
				()
  in

  (* Bouton Enregistrer pour sauvegarder l'image *)
  let save = GButton.button ~packing:(bottom#pack)
			    ()
  in

  let _ = GtkBase.Widget.set_can_focus   save#as_widget true
  and _ = GtkBase.Widget.set_can_default save#as_widget true
  and _ = GtkBase.Widget.set_sensitive   save#as_widget false
  in

  let _ = My_controls.iconLabeled "Enregistrer"
				  (GDraw.pixmap_from_xpm_d ~data:Icones.store
							   ~window
							   ()
				  )
				  ~packing:save#add
				  ()
  in

  (* Bouton Quitter pour fermer la fentre de capture *)
  let quit = GButton.button ~packing:(bottom#pack)
			    ()
  in

  let _ = GtkBase.Widget.set_can_focus quit#as_widget true
  and _ = GtkBase.Widget.set_can_default quit#as_widget true
  and _ = quit#grab_default ()
  in

  let _ = My_controls.iconLabeled "Quitter"
				  (GDraw.pixmap_from_xpm_d ~data:Icones.quit
							   ~window
							   ()
				  )
				  ~packing:quit#add
				  ()
  in



object(self)

  method window      = window
  method path        = path
  method pathTmp     = pathTmp
  method name        = name
  method save        = save
  method quit        = quit
  method select      = select

  val mutable ext = 0

  method getFileName = Filename.concat path#text name#text

  (* indique si la sauvegarde peut ou non avoir lieu (nom de fichier rempli) *)
  val mutable savable = false

  (* Mthode qui vrifie si les champs du Chemin, Nom et Tmp sont remplis. *)
  (* Active le bouton d'enregistrement en fonction.                        *)
  method onChanged () = let sbt = (not (path#text="")) && (not (name#text="")) && (not (pathTmp#text=""))
			 in
			   if (sbt && (not savable)) || ((not sbt) && savable)
			   then
			     begin
			       savable <- not savable;
			       GtkBase.Widget.set_sensitive save#as_widget savable
			     end

  method onQuitter () = window#destroy ()


initializer



  let _ = path#set_text (Sys.getcwd ())
  in

  let _ = pathTmp#set_text (match Sys.os_type
			    with
			      | "Unix"   -> (try Sys.getenv "TMPDIR"
					     with
					       | Not_found -> "/tmp"
					    )

			      | _        -> (try Sys.getenv "TEMP"
					     with
					       | Not_found -> "C:\temp"
					    )
			   )
  in

  (* Gestion du clavier *)
  let _ = window#event#connect#key_press ~callback:(fun (event: GdkEvent.Key.t) -> begin
										     let key = GdkEvent.Key.keyval event
										     in
										       match key
										       with
											 | t when t=GdkKeysyms._Escape -> self#onQuitter ();
															  true
											 | _                           -> false
										   end
						   )
  in

  let _ = window   #connect#remove       ~callback:(fun (widget:GObj.widget) -> self#onQuitter ())
  and _ = quit     #connect#clicked      ~callback:self#onQuitter
  and _ = path     #connect#changed      ~callback:self#onChanged
  and _ = name     #connect#changed      ~callback:self#onChanged
  and _ = pathTmp  #connect#changed      ~callback:self#onChanged
  and _ = save     #connect#clicked      ~callback:(fun () -> match formatAssoc.(ext)
							      with
								| Avi     -> self#onQuitter ();
									     CreationVideo.createVideo pathTmp#text
												       (Filename.concat path#text name#text)
												       (sizeRate#adjustment#value /. 100.0)
												       spinFPS#value_as_int
												       (Usefull.round (if noPause#active
														       then
															 0.0
														       else
															 (spinFPS#value *. spinPause#value)
														      )
												       )
												       (Usefull.round (spinFPS#value *. spinWait#value))

								| GifAnim -> self#onQuitter ();
									     CreationVideo.createGifAnime pathTmp#text
													  (Filename.concat path#text name#text)
													  (sizeRate#adjustment#value /. 100.0)
													  spinFPS#value_as_int
													  (if noPause#active
													   then
													     0.0
													   else
													     spinPause#value
													  )
													  spinWait#value

								| LstBmp
								| LstJpg
								| LstPng  -> self#onQuitter ();
									     CreationVideo.createLst (Filename.concat path#text name#text)
												     (saveFormat formatAssoc.(ext))
												     (sizeRate#adjustment#value /. 100.0)
												     spinFPS#value_as_int
						   )
  and _ = select   #connect#clicked      ~callback:(fun () -> let w = new Guifichier.imo_fichier Guifichier.Load 
												 self#path#text 
												 (fun str -> self#path#set_text (Filename.dirname  str);
													     self#name#set_text (Filename.basename str)
												 )
							      in
								w#window#show ()
						   )

  and _ = selectTmp#connect#clicked      ~callback:(fun () -> let w = new Guifichier.imo_fichier Guifichier.Load 
												 self#pathTmp#text 
												 (fun str -> self#pathTmp#set_text (Filename.dirname  str))
							      in
								w#window#show ()
						   )

  (* Fixe l'action  faire lorsqu'un format de fichier est slectionn *)
  and _ = let itemList = choix#children
	  in
	  let rec activator lst i = match lst
				    with
				      | []   -> ()
				      | t::q -> t#connect#activate ~callback:(fun () -> ext <- i;
											let (nom, typ) = Usefull.splitName name#text
											in
											(* Modifie l'extension du fichier en fonction du format *)
											let new_name = try
													 if guessExtension typ formatAssoc.(ext)
													 then name#text
													 else nom ^ "." ^ (getExtension (formatAssoc.(ext)))
												       with
													 | APas -> if nom = ""
														   then
														     ""
														   else
														     name#text ^ "." ^ (getExtension (formatAssoc.(ext)))
											in
											  name#set_text new_name;
											  (* Pour la liste de Jpg, dsactive les options inutiles *)
											  (* Sinon, les ractive ...                              *)
											  match formatAssoc.(i)
											  with
											    | LstBmp
											    | LstJpg
											    | LstPng -> GtkBase.Widget.hide  bla2Wait#as_widget;
													GtkBase.Widget.hide  spinWait#as_widget;
													GtkBase.Widget.hide bla2Wait2#as_widget;
													GtkBase.Widget.hide   noPause#as_widget;
													GtkBase.Widget.hide  setPause#as_widget;
													GtkBase.Widget.hide   bla2Tmp#as_widget;
													GtkBase.Widget.hide   pathTmp#as_widget;
													GtkBase.Widget.hide selectTmp#as_widget
											    | _      -> GtkBase.Widget.show  bla2Wait#as_widget;
													GtkBase.Widget.show  spinWait#as_widget;
													GtkBase.Widget.show bla2Wait2#as_widget;
													GtkBase.Widget.show   noPause#as_widget;
													GtkBase.Widget.show  setPause#as_widget;
													GtkBase.Widget.show   bla2Tmp#as_widget;
													GtkBase.Widget.show   pathTmp#as_widget;
													GtkBase.Widget.show selectTmp#as_widget
									     );
						activator q (i+1)
	  in
	    activator itemList 0

  and _ = sizeRate#adjustment#connect#value_changed ~callback:(fun () -> rate#set_text (Usefull.rateText sizeRate#adjustment#value))
  in
    ()

end



(*************************************************************)
(* Fonction crant une nouvelle fentre de capture d'images. *)
(*************************************************************)
(* Type : unit -> imo_image                                  *)
(*************************************************************)
let video () = let w = new imo_video ()
	       in
		 w#window#show ()
