open Types
open PositionedGroup
open Boundingbox
open Parametres

(************************************************************)
(* Cration d'un HexaGrid2                                  *)
(*   pt     : centre                                        *)
(*   axis1  : 1er axe de remplissage                        *)
(*   axis2  : 2me axe de remplissage                       *)
(*   align1 : alignement sur le 1er axe                     *)
(*   align2 : alignement sur le 2me axe                    *)
(*   align3 : alignement sur le 3me axe                    *)
(*   space  : espace entre 2 objets                         *)
(*   rate   : triplet entr pour le ratio                   *)
(*   coul   : couleur de l'HexaGrid2                        *)
(*   lst    : liste des objets de l'HexaGrid2               *)
(************************************************************)
(* Type : point -> axe -> axe -> align -> align -> align -> *)
(*        float -> float * float * float -> couleur ->      *)
(*        (objet * point) list -> objet                     *)
(************************************************************)
class hexagrid2 (col:couleur) (pos:point) (cont:(drawable*point) list ref) (axis1:axe) (axis2:axe) (align3:align) (space1:float) (rate:float*float*float)=
 object(self)
    inherit positionedGroup col pos cont as super

    initializer
      let (axis1, axis2, ratio1, ratio2) = self#axis_ratio2 axis1 axis2 rate (List.length !content)  
      and cote   = ref 0.0 
      and indice = ref 0
      in
        let (x_pred, y_pred, z_pred) = (ref 0.0, ref 0.0, ref 0.0)
	in
	  (* Calcule de la taille de chaque case de l'HexaGri2 et de la 1re partie de la translation pour passer d'un objet au suivant *)
	  List.iter (function (obj, vect) -> let (min1, min2, min3, max1, max2, max3) = desordre (obj#getBoundingBox ()) axis1 axis2
					     and i = !indice mod ratio1 
					     and j = !indice / ratio1 
		                             in
		                               max_diff_update_ref cote max1 min1;
		                               max_diff_update_ref cote max2 min2;
					       max_diff_update_ref cote max3 min3;
					       let (x, y, z) = (sbt Middle min1 max1,
								sbt Middle min2 max2,
								sbt align3 min3 max3)
					       in
						 vect#setAll (permut (x -. !x_pred,
								      y -. !y_pred,
								      z -. !z_pred)
						              axis1
							      axis2);
						 x_pred := x;
						 y_pred := y;
						 z_pred := z;
 						 incr indice
		    ) !content;
	  
	  (* Calcule 2nde partie de la translation pour passer d'un objet au suivant *)
  	  let rayon  = (!cote /. sqrt3) +. (space1 /. 2.0)
	  in
	    let sumCol = (2.0 *. rayon *. (float_of_int (ratio1-1))) +. !cote +. (if ((ratio2 > 1)
										      && ((List.length !content) >= (2 * ratio1))
										     )
										  then rayon
										  else 0.0)
	    and sumRow = ((float_of_int (ratio2-1)) *. rayon *. sqrt3) +. !cote
	    in
	      (match !content
	       with
		 | []            -> ()
	 	 | (_, vect)::tl -> vect#updateAll (permut ((!cote -. sumCol) /. 2.0, 
							    (!cote -. sumRow) /. 2.0,
							    sbt2 align3 0.0 !cote)
						    axis1
						    axis2);
		                    indice := 1;
				    List.iter (function (_, vect) -> let i   = (!indice)     mod ratio1
								     and j   = (!indice)     / ratio1
					                             in
					                               let sup = if (j mod 2)=0
								                 then (-. rayon)
								                 else (   rayon)
								       in
								         let (delta1, delta2) = (if (i <> 0) 
												 then (2.0 *. rayon,0.0)
												 else ((2.0 *. rayon *. (float_of_int (1-ratio1))) +. sup,
												       rayon *. sqrt3)
												)
									 in
									   vect#updateAll (permut (delta1, delta2, 0.0)
											   axis1
											   axis2);
 									   incr indice
					      ) tl
	      );
	      bbFunction <- (fun () ->  boundingBox_of self#getPos sumCol sumRow !cote axis1 axis2)
  end

(************************************************************)
(* Cration d'un HexaGrid3                                  *)
(*   pt     : centre                                        *)
(*   axis1  : 1er axe de remplissage                        *)
(*   axis2  : 2me axe de remplissage                       *)
(*   space  : espace entre 2 objets                         *)
(*   r1     : valeur entre pour le ratio sur l'axe axis1   *)
(*   r2     : valeur entre pour le ratio sur l'axe axis2   *)
(*   r3     : valeur entre pour le ratio sur l'axe "axis3" *)
(*   coul   : couleur du Grid2                              *)
(*   lst    : liste des objets du Grid2                     *)
(************************************************************)
(* Type : point -> axe -> axe -> align -> align -> align -> *)
(*        float -> float -> float -> float -> couleur ->    *)
(*        (objet * point) list -> objet                     *)
(************************************************************)
class hexagrid3 (col:couleur) (pos:point) (cont:(drawable*point) list ref) (axis1:axe) (axis2:axe) (space:float) (ratio1:float) (ratio2:float) (ratio3:float)=
 object(self)
    inherit positionedGroup col pos cont as super

    initializer
      let (axis1, axis2, ratio1, ratio2, ratio3) = self#axis_ratio3 axis1 axis2 ratio1 ratio2 ratio3 (List.length !content)
      and cote   = ref 0.0
      and indice = ref 0 
      in 
        let (x_pred, y_pred, z_pred) = (ref 0.0, ref 0.0, ref 0.0)
	in
	  (* Calcule de la taille de chaque case de l'HexaGrid3 et de la 1re partie de la translation pour passer d'un objet au suivant *)
	  List.iter (function (obj, vect) -> let (min1, min2, min3, max1, max2, max3) = desordre (obj#getBoundingBox ()) axis1 axis2
					     and i = !indice mod ratio1
					     and j = (!indice / ratio1) mod ratio2
					     and k = !indice / (ratio1 * ratio2) 
		                             in
		                               max_diff_update_ref cote max1 min1;
		                               max_diff_update_ref cote max2 min2;
					       max_diff_update_ref cote max3 min3;
					       let (x, y, z) = (sbt Middle min1 max1,
								sbt Middle min2 max2,
								sbt Middle min3 max3)
					       in
						 vect#updateAll (permut (x -. !x_pred,
									 y -. !y_pred,
									 z -. !z_pred)
								 axis1
								 axis2);
						 x_pred := x;
						 y_pred := y;
						 z_pred := z;
 						 incr indice
		    ) !content;
	  
	  (* Calcule 2nde partie de la translation pour passer d'un objet au suivant *)
	  let n      = List.length !content
  	  and rayon  = (!cote *. sqrt6 /. 4.0) +. (space /. 2.0)
	  in
	    let sumCol = (2.0 *. rayon *. (float_of_int (ratio1-1))) +. !cote +. (if ((ratio2 > 1)
										      && (n >= (2 * ratio1))
										     )
										  then rayon
										  else 0.0)
	    and sumRow = ((float_of_int (ratio2-1)) *. rayon *. sqrt3) +. !cote +. (if ((ratio2 > 1)
											&& (n > ratio1 * (2 * ratio1 - 1))
										       )
										    then 2.0 *. rayon /. sqrt3
										    else 0.0)
	    and sumStage = (((float_of_int (ratio3-1)) *. 2.0 *. rayon *. sqrt6) /. 3.0) +. !cote
	    in
	      (match !content
	       with
		 | []            -> ()
		 | (_, vect)::tl -> vect#updateAll (permut ((!cote -.   sumCol) /. 2.0,
							    (!cote -.   sumRow) /. 2.0,
							    (!cote -. sumStage) /. 2.0)
						    axis1
						    axis2);
		                    indice := 1;
				    List.iter (function (_, vect) -> let i     =  !indice                 mod ratio1 
								     and j     =  (!indice      / ratio1) mod ratio2
								     and k     =  !indice     / (ratio1 * ratio2)
					                             in
					                               let sup11 = if (ratio2 mod 2)=0
								                   then (-. rayon)
								                   else 0.0
								       and sup12 = if (j mod 2)=0
								                   then (-. rayon)
								                   else (   rayon)
								       and sup2  = if (k mod 2)=0
								                   then (-. rayon *. 2.0 /. sqrt3)
								                   else (   rayon *. 2.0 /. sqrt3)
								       in
								         let delta = (match (i, j) 
										      with
											| (0, 0) -> ((2.0 *. rayon *. (float_of_int (1-ratio1))) +. sup11,
												    (sqrt3 *. rayon *. (float_of_int (1-ratio2))) +. sup2,
												     2.0 *. sqrt6 *. rayon /. 3.0)

											| (0, _) -> ((2.0 *. rayon *. (float_of_int (1-ratio1))) +. sup12,
												     rayon *. sqrt3,
												     0.0)

											| (_, _) -> (2.0 *. rayon,
												     0.0,
												     0.0)
										     )
									 in
									   vect#updateAll (permut delta axis1 axis2);
									   incr indice
					      ) tl
	      );
	      bbFunction <- (fun () -> boundingBox_of self#getPos sumCol sumRow sumStage axis1 axis2)
  end
