open Types
open GMain
open Parametres
open Viewport
open Unix


(*************************************)
(* Gestion des option du programme  *)
(* partir du tableau d'arguments arg *)
(*************************************)
(* Type : string array -> unit       *)
(*************************************)
let options arg =
  Arg.parse [ "-axis", Arg.Unit (function () -> view#setAfficheAxe true), "\tdisplay axes" ;

	      "-file", Arg.String (function str -> if (!entree = File) && (!file_name = "")
						   then file_name := str
						   else
						     begin
						       prerr_string ("Option \"-file " ^ str ^ "\" invalide\n");
						       exit 1
						     end
				  ), "name\tinput file" ;

	      "-gridXY", Arg.Unit (function () -> view#setGrille1 AxeX;
						  view#setGrille2 AxeY
				  ), "\tdisplay XY-grid" ;

	      "-gridYZ", Arg.Unit (function () -> view#setGrille1 AxeY;
						  view#setGrille2 AxeZ
				  ), "\tdisplay YZ-grid" ;

	      "-gridZX", Arg.Unit (function () -> view#setGrille1 AxeZ;
						  view#setGrille2 AxeX
				  ), "\tdisplay ZX-grid" ;

	      "-instant", Arg.Set instantLoad, "\texecute commands instantly after they have been read by the parser" ;

	      "-nofog", Arg.Unit (function () -> view#setBrouillard false), "\tdisable fog" ;

	      "-nolight", Arg.Unit (function () -> view#setLighting false), "\tdisable lighting effects" ;

	      "-size", Arg.String (function str ->
					   let dims = Str.split (Str.regexp ":") (String.sub str 1 ((String.length str) - 2)) in
					     try
					       width  := float_of_string (List.hd dims);
					       height := float_of_string (List.hd (List.tl dims));
					     with _ -> prerr_endline "Option -size: invalide format"; exit 1
				  ), "[x:y]\t3D area geometry";

	      "-stereo", Arg.Unit (function () -> view#setStereo true), "\tenable stereo mode" ;

	      "-stdin", Arg.Unit (function () -> if (!entree = File) && (!file_name <> "")
						 then
						   begin
						     prerr_string ("Incompatible use of -file and -stdin options\n");
						     piped := true;
						     exit 1
						   end
						 else
						   begin
						     entree    := Stdin;
						     piped     := true;
						     file_name := "imoview"
						   end
				 ), "\tread in standard input"

	    ]
	    (function str -> if (!entree = File) && (!file_name = "")
			     then
			       begin
				 entree    := File;
				 file_name := str
			       end
			     else
			       begin
				 prerr_string ("Invalide argument " ^ str ^ "\n");
				 exit 1
			       end
	    )
	    ("Usage : " ^ arg.(0) ^ " [options] [file]\n")


(*Strange solution to enable support of system threads in imoview (never ask why!)*)
let lThread3 () = let t = (GMain.Timeout.add ~ms:1
                                	     ~callback:(fun () -> true)) in ();;

(***********************)
(* Fonction principale *)
(***********************)
let main () =
  (* Dealing with options *)
  options Sys.argv;

  (* Initializing system threads *)
  lThread3 ();

  (* loading first file *)
  Load.load_file ();

  (* Initilizaing lablgtk *)
  GtkMain.Main.init () ;

  let window = new Guimain.imo_window () in
  let _ = window#window#show () in
    GMain.Main.main ()

let _ = Printexc.print main ()

