#include <stdio.h>
#include <caml/mlvalues.h>
#include <caml/fail.h>
#include "gl2ps.h"
#include "gl_tags.h"



FILE *file = NULL;



GLint capToInt (value cap)
{
	switch (cap)
	{
		case MLTAG_blend :
			return GL2PS_BLEND;
		case MLTAG_line_stipple :
			return GL2PS_LINE_STIPPLE;
	}

	failwith ("Cap invalide");
	return 0;
}



void verifResult (GLint res)
{
  switch (res)
	{
		case GL2PS_UNINITIALIZED : 
			failwith ("Uninitialized");
			break;

		case GL2PS_ERROR :
			failwith ("Error");
			break;

		case GL2PS_OVERFLOW :
			failwith ("Overflow");
			break;

		case GL2PS_NO_FEEDBACK :
			failwith ("No feeback");

		default :
			break;
	}
	return ;
}



value psPointSize (value size)
{
	verifResult (gl2psPointSize (Double_val (size)));
	return Val_unit;
}



value psLineWidth (value width)
{
	verifResult (gl2psLineWidth (Double_val (width)));
	return Val_unit;
}



value psEnable (value cap)
{
	verifResult (gl2psEnable (capToInt (cap)));
	return Val_unit;
}



value psDisable (value cap)
{
	verifResult (gl2psDisable (capToInt (cap)));
	return Val_unit;
}



value psBlendFunc (value s, value d)
{
	GLenum paramS, paramD;

	switch (s)
	{
		case MLTAG_dst_alpha :
			paramS = GL_DST_ALPHA;
			break;

		case MLTAG_dst_color :
			paramS = GL_DST_COLOR;
			break;

		case MLTAG_one :
			paramS = GL_ONE;
			break;

		case MLTAG_one_minus_dst_alpha :
			paramS = GL_ONE_MINUS_DST_ALPHA;
			break;

		case MLTAG_one_minus_dst_color :
			paramS = GL_ONE_MINUS_DST_COLOR;
			break;

		case MLTAG_one_minus_src_alpha :
			paramS = GL_ONE_MINUS_SRC_ALPHA;
			break;

		case MLTAG_src_alpha :
			paramS = GL_SRC_ALPHA;
			break;

		case MLTAG_src_alpha_saturate :
			paramS = GL_SRC_ALPHA_SATURATE;
			break;

		case MLTAG_zero :
			paramS = GL_ZERO;
	}

	switch (d)
	{
		case MLTAG_one :
			paramD = GL_ONE;
			break;

		case MLTAG_one_minus_dst_alpha :
			paramD = GL_ONE_MINUS_DST_ALPHA;
			break;

		case MLTAG_one_minus_dst_color :
			paramD = GL_ONE_MINUS_DST_COLOR;
			break;

		case MLTAG_one_minus_src_alpha :
			paramD = GL_ONE_MINUS_SRC_ALPHA;
			break;

		case MLTAG_src_alpha :
			paramD = GL_SRC_ALPHA;
			break;

		case MLTAG_src_color :
			paramD = GL_SRC_COLOR;
			break;

		case MLTAG_zero :
			paramD = GL_ZERO;
	}

	verifResult (gl2psBlendFunc (paramS, paramD));
	return Val_unit;
}



value psBeginPage (value mlTitre, value mlFormat, value mlBackground, value mlBufferSize, value mlFileName)
{
	char *titre = String_val (mlTitre);

	GLint format;

	GLint options = GL2PS_USE_CURRENT_VIEWPORT
		      | GL2PS_SIMPLE_LINE_OFFSET
		      | GL2PS_OCCLUSION_CULL
		      | GL2PS_BEST_ROOT;
	GLint bufferSize = Int_val (mlBufferSize);
	char *fileName = String_val (mlFileName);

	file = fopen (fileName, "wb");
	if (Int_val (mlBackground))
		options |= GL2PS_DRAW_BACKGROUND;

	switch (Long_val (mlFormat))
	{
		case 0 :
			format = GL2PS_PDF;
			break;
		case 1 :
			format = GL2PS_PS;
			break;
		case 2 :
			format = GL2PS_EPS;
	}

	verifResult (gl2psBeginPage (titre,
				     "Imoview",
				     NULL,
				     format,
				     GL2PS_BSP_SORT,
				     options,
				     GL_RGBA,
				     0,
				     NULL,
				     0,
				     0,
				     0,
				     bufferSize,
				     file,
				     fileName));

	return Val_unit;
}



value psEndPage (value unit)
{
	verifResult (gl2psEndPage ());
	fclose (file);
	file = NULL;
	return Val_unit;
}
