(********************************************************************)
(*                                                                  *)
(* Le fichier load.ml contient la fonction pour charger un fichier. *)
(*                                                                  *)
(********************************************************************)



open Types
open Parametres
open Core
open Viewport



(******************************)
(* Mise  zro des paramtres *)
(* du parser et d'affichage.  *)
(******************************)
(* Type : unit -> unit        *)
(******************************)
let init_parser () =
  core#prepareNewLoad;
  select         := None;
  line           := 1

(*
                     dico           := [];
		     dessin         := [];
		     sceneLst       := [];
                     core#prepareNewLoad;
		     select         := Vide;
		     line           := 1;
		     command_todo#clear ();
		     command_done#clear ();

		     vectX #setAll (1., 0.              , 0.              );
		     vectY #setAll (0., 0.86602540378445,-0.5             );
		     vectZ #setAll (0., 0.5             , 0.86602540378445);
		     centre#setAll (0., 0.              , 0.              )*)

(*Execute commands that have not been displayed yet*)
let flushCommands () = core#animToString;
                       core#setState       Lct;
		       core#setDir       Avant;
		       core#setNextState  None;
		       core#setTimer     (Some (GMain.Timeout.add ~ms:10
				  	                          ~callback:( core#checkNewState )
					       ))


(****************************************)
(* Analyse l' inchannel (~flux) chan    *)
(****************************************)
(* Type : inchannel -> unit             *)
(****************************************)
let analyse win chan = Mutex.lock loadMutex; isLoading := true;
                       try
			 let lexBuffer = Lexing.from_channel chan
			 in
			   (* Mise  "zro" des paramtres de l'analyse syntaxique *)
			   init_parser ();
			   (* Analyse syntaxique *)
			   if !Parametres.entree = Stdin
			   then
			     begin
			       while (Parser.std  Scanner.token lexBuffer)
			       do
				 ()
			       done;
			       if (!instantLoad) then flushCommands ()
			       else view#draw ();
			     end
			   else 
			     begin
			       Parser.file Scanner.token lexBuffer;
			       if (!instantLoad) then flushCommands ()
			     end

		       (* On rattrape les erreurs syntaxiques pour indiquer le type d'erreur et la localisation *)
		       with
			 | ParseError s -> close_in chan;
					   My_controls.messageBox "Erreur dans le chargement" (" ligne " ^ (string_of_int !line) ^ " : " ^ s);
                       Mutex.unlock loadMutex; isLoading := false

let load channel = (try
		      analyse 0 channel;
		      (* On ferme le fichier *)
		      close_in channel;
                      print_string "\nLoaded input file!\n";
		      flush stdout
                        
                      (* On rattrape les erreurs systmes (impossible d'ouvrir le fichier ...) *)
	  	    with
		      | Sys_error s -> My_controls.messageBox "Erreur dans le chargement" s )(* ) ()) in  ()*)


let loadViaThread channel = let t = (Thread.create (fun () -> (load channel )) ()) in  ()

(*********************************************)
(* Analyse et lancer l'affichage  partir de *)
(* l'entre dfinie dans Parametres.entree   *)
(*********************************************)
(* Type : unit -> unit                       *)
(*********************************************)
let load_file () =
  match (!entree, !file_name)
  with
      (* On analyse le contenu de l'entre standard *)
    | (Stdin, _) ->
	if (!Parametres.instantLoad)
	then loadViaThread stdin
	else load stdin

    (* Nom de fichier vide -> on met tout  vide *)
    | (File, "") -> init_parser ()

    (* On analyse le fichier name *)
    | (File, name) ->
	if (!Parametres.instantLoad)
	then loadViaThread (open_in name)
	else load (open_in name)


