(**************************************************************)
(*                                                            *)
(* Ce fichier contient les fonctions de gestion de la souris. *)
(*                                                            *)
(**************************************************************)



open Types
open Triplet
open Viewport



(*************************************************************)
(* Change le curseur en fonction du clic.                    *)
(*  - curs est le curseur  utiliser                         *)
(*  - state  est l'tat du clic (DOWN/UP)                    *)
(*************************************************************)
(* Type : Gdk.Cursor.cursor_type -> Types.mouseState -> unit *)
(*************************************************************)
let setCurs curs state = Gdk.Window.set_cursor (unSome !Parametres.area)#misc#window
					       (Gdk.Cursor.create (match state
								   with
								     | DOWN -> (* Clic -> on met le cursor adapt  la fonction souhaite *)
									       curs
								     | UP   -> (* Dclic -> on met le cursor normal *)
									       `X_CURSOR
								  )
					       )



(***************************************************************************)
(* Met  jour de l'instance button avec les valeurs state, x et y.         *)
(***************************************************************************)
(* Type : Types.mouseButton -> GdkEvent.Button.types -> int -> int -> unit *)
(***************************************************************************)
let majButton button state x y = button#setState state ;
				 button#setX x ;
				 button#setY y



(********************************************************)
(* Fonction qui gre les clics de la souris.            *)
(*  - state : tat du bouton (UP/DOWN)                  *)
(*  - ev    : vnement reu                            *)
(********************************************************)
(* Type : Types.mouseState -> GdkEvent.Button.t -> unit *)
(********************************************************)
let mouseClic state ev = let x = GdkEvent.Button.x ev
			 and y = GdkEvent.Button.y ev
			 in
			   match (GdkEvent.Button.button ev)
			   with
			     | 1 -> (* On enregistre l'tat et la position du bouton *)
				    majButton Parametres.leftButton state x y;
				    setCurs `FLEUR state;
				    (match GdkEvent.get_type ev
				     with
				       |`TWO_BUTTON_PRESS -> ()(*Picking.selectionne x y*)
				       | _                -> ()
				    );
				    true

			     | 2 -> (* On enregistre l'tat et la position du bouton *)
				    majButton Parametres.middleButton state x y;
				    setCurs `EXCHANGE state;
				    (* On met  jour l'tat du bouton selon si les touches CTRL et SHIFT sont enfonces *)
				    Parametres.middleButton#setShift ((GdkEvent.Button.state ev land 0x1) <>0 );
				    Parametres.middleButton#setCtrl  ((GdkEvent.Button.state ev land 0x4) <>0 );
				    true

			     | 3 -> (* On enregistre l'tat et la position du bouton *)
				    majButton Parametres.rightButton state x y;
				    setCurs `DOUBLE_ARROW state;
				    true

			     | 4 -> (* On ralise une rotation ou un spin selon si les touches CTRL et SHIFT sont enfonces *)
				    (match GdkEvent.get_type ev
				     with
				       | `BUTTON_PRESS -> (match ((GdkEvent.Button.state ev land 0x1) <>0,(GdkEvent.Button.state ev land 0x4) <>0)
							   with
							     | ( true,  true) -> view#spin (0.0, 0.0,  0.0) (0.0, 0.0, -2.0)
							     | ( true, false) -> view#spin (0.0, 0.0, -2.0) (0.0, 0.0,  0.0)
							     | (false, false) -> view#rotation (0.0, 0.0, -2.0)
							     | (false,  true) -> () (* TODO *)
							  );
							  view#draw ()
				       | _             -> ()
				    );
				    true

			     | 5 -> (* On ralise une rotation ou un spin selon si les touches CTRL et SHIFT sont enfonces *)
				    (match GdkEvent.get_type ev
				     with
				       | `BUTTON_PRESS -> (match ((GdkEvent.Button.state ev land 0x1) <>0,(GdkEvent.Button.state ev land 0x4) <>0)
							   with
							     | ( true,  true) -> view#spin (0.0, 0.0,  0.0) (0.0, 0.0,  2.0)
							     | ( true, false) -> view#spin (0.0, 0.0,  2.0) (0.0, 0.0,  0.0)
							     | (false, false) -> view#rotation (0.0, 0.0,  2.0)
							     | (false,  true) -> () (* TODO *)
							  );
							  view#draw ()
				       | _             -> ()
				    );
				    true

			     | _ -> false



(**********************************************)
(* Fonction appele lors des mouvements de la *)
(* souris quand un bouton est enfonc.        *)
(* ev est l'venement GDK reu.               *)
(**********************************************)
(* Type : GdkEvent.Motion.t -> unit           *)
(**********************************************)
let mouseMove ev = let x = GdkEvent.Motion.x ev
		   and y = GdkEvent.Motion.y ev
		   in
		     (* Translation (bouton gauche) *)
		     if (Parametres.leftButton#getState = DOWN)
		     then
		       begin
			 let coeff = view#getZoom /. 1024.0
			 in
			 (* Calcule du vecteur de la translation du dessin *)
			 let trans = (((Parametres.leftButton#getX -. x)) *. coeff,
				      ((y -. Parametres.leftButton#getY)) *. coeff,
				      0.0
				     )
			 in
			   majButton Parametres.leftButton DOWN x y;
			   view#translation trans;
		       end;

		     (* Rotation ou spin (bouton centrale) *)
		     if Parametres.middleButton#getState = DOWN
		     then
		       begin
			 let w = int_of_float (!Parametres.width)
			 and h = int_of_float (!Parametres.height)
			 in
			 let taux = 180.0 /. (float_of_int (min w h))
			 in
			 (* Calcule des angles de rotation *)
			 let theta = (Parametres.middleButton#getY -. y) *. taux
			 and phi   = (Parametres.middleButton#getX -. x) *. taux
			 in
			   majButton Parametres.middleButton DOWN x y;
			   (match (Parametres.middleButton#getShift, Parametres.middleButton#getCtrl)
			    with
			      | ( true,  true) -> view#spin (   0.,  0., 0.) (theta, phi, 0.)
			      | ( true, false) -> view#spin (theta, phi, 0.) (   0.,  0., 0.)
			      | (false, false) -> view#rotation (theta, phi, 0.)
			      | (false,  true) -> () (* TODO *)
			   )
		       end;

		     (* Zoom (bouton droit) *)
		     if Parametres.rightButton#getState = DOWN
		     then
		       begin
			 let rate = 1.01 ** (y -. Parametres.rightButton#getY)
			 in
			   view#setZoom (view#getZoom *. rate);
			   majButton Parametres.rightButton DOWN x y;
			   (* Mise  jour des paramtres de la perspective *)
			   view#set_perspective ();
			   view#draw ();
		       end;

		     view#draw ();
		     ignore (Gdk.Window.get_pointer_location (unSome !Parametres.area)#misc#window);
		     true
