/* TODO : commenter le fichier */
/* TODO : v?rifier la coh?rence des valeurs (ex. : hauteur>=0.) */


/*
 * Exceptions ?  rattraper :  - ParseError 'string' : 'string' repr?sente le messge d'erreur ? afficher
 */


%{

open Types
open Triplet
open Parametres
open Usefull

open Content
open Spline
open Disc
open Sphere
open Cone
open Box
open Empty
open Cylinder
open Frustrum
open Polygon
open Polyline
open Graph
open ComplexGraph

open Core
open Viewport
open Group
open Transformation
open Grids
open Hexagrids
open GraphPlacer
open Kkplacer
open Font

exception PolygonePlat


(* Just to define placer without knowing the graph *)
type placer = Placer of string*bool*bool*int*string option (* (name of placer) (show placing steps) (place all nodes) (dimension of placing) (path to save result)*)

let parse_error s = raise (ParseError s)

(*First only a simple copy function*)
let copyObject name = Oo.copy (core#getObject name)


(*If a new command has been loaded and instantLoad checked, starts the execution of commands if a certain number of them is available*)
let tryDisplay () = if ((!Parametres.instantLoad) && ( (core#getCommandToDo#length) > (!Parametres.minBufSize)) && ((core#getState)=Fin or (core#getState)=Stop)) 
                    then 
                      begin
                        print_string "Started display!";
			flush stdout;
			core#setState     Lct;
			core#setDir  Avant;
			core#animToString;
			core#setNextState None;
 	                core#setTimer ( Some (GMain.Timeout.add ~ms:1
						                ~callback:(core#checkNewState)
					     ))
		      end

(** Places a freshly defined graph
@param graph Graph to place
@param placer Placer to utilize (for a list see the doc)

*)
let placeGraph graph placer =match placer with
                               | Placer(name,showCours,placeAll,dim,path ) -> ((* Apply matching placer *)
		                                                              match name with
										| "simple" -> let p = new simplePlacer graph placeAll dim
						                                              in
												if (showCours) then
												  p#showPlacing
												else
												  p#place
										| "kk"     -> let p = new kkPlacer graph placeAll dim
						                                              in
												if (showCours) then
												  p#showPlacing
												else
												  p#place
												    (*g#writeToFile "graph.imo";*)
										| "neato"  -> let p = new neato graph placeAll
						                                              in
												p#place
										| "dot"    -> let p = new dot graph placeAll
						                                              in
												p#place
										| "twopi"  -> let p = new twopi graph placeAll
						                                              in
												p#place
										| "circo"  -> let p = new circo graph placeAll
						                                              in
												p#place
										| "fdp"    -> let p = new fdp graph placeAll
						                                              in
												p#place
										| x        -> raise (ParseError ("Unknown placer: " ^ x) )
		                                                              );
				                                              if (path <> None) then
										graph#writeToFile (unSome path)


(** Places a freshly defined simplicial complex
@param graph Graph to place
@param placer Placer to utilize (for a list see the doc)

*)
let placeComplex graph placer = match placer with
                                 | Placer(name,showCours,placeAll,dim,path) -> ((* Apply matching placer *)
		                                                               match name with
										 | "simple"      -> let p = new simplePlacer (graph :>Graph.graph) placeAll dim
						                                                    in
												      if (showCours) then
													p#showPlacing
												      else
													p#place
									         | "kk"    -> let p = new kkComplexPlacer graph placeAll dim
						                                              in
												if (showCours) then
												  p#showPlacing
												else
												  p#place
										 | x        -> raise (ParseError ("Placer " ^ x ^ " unknown for type ComplexGraph.") )
		                                                               );
				                                               if (path <> None) then
										 graph#writeToFile (unSome path)
		

%}


  /***************/
 /* Token typ?s */
/***************/


%token <string>     TokNOM ID
%token <float>      TokNOMBRE
%token <bool>       TokBOOL
%token <string>     TokSTRING




  /*****************/
 /* Token simples */
/*****************/


%token              TokVirgule
%token              TokDebAccol
%token              TokFinAccol
%token              TokDebCrochet
%token              TokFinCrochet
%token              TokInf
%token              TokSup
%token              Tok2points

%token              TokBox
%token              TokCone
%token              TokCylinder
%token              TokDisc
%token              TokFrustum
%token              TokSphere
%token              TokPolygone
%token              TokPolyline
%token              TokSpline
%token              TokEmpty
%token              TokLine
%token              TokLabel

%token              TokGraph 
%token              TokEdge 
%token              TokEdgeList
%token              TokVertex 
%token              TokVertexList 
%token              TokV1 
%token              TokV2 
%token              TokPlacer
%token              TokDirected
%token              TokPlaceAll
%token              TokCours
%token              TokDim
%token              TokSave

%token              TokComplexGraph
%token              TokSimplex
%token              TokSimplexList

%token              TokTranslated
%token              TokScaled
%token              TokAxisRotated

%token              TokGroup
%token              TokGrid0
%token              TokGrid1
%token              TokGrid2
%token              TokGrid3
%token              TokHexa2
%token              TokHexa3

%token              TokColor

%token              TokTextured

%token              TokPosition
%token              TokSize
%token              TokRadius
%token              TokHeight
%token              TokSolid
%token              TokSlices
%token              TokApproxPoints
%token              TokTaper
%token              TokStacks
%token              TokText
%token              TokPointList
%token              TokColorList

%token              TokTranslation
%token              TokScale
%token              TokAxis
%token              TokAngle
%token              TokGeometry

%token              TokVisible

%token              TokStartPoint
%token              TokSpace
%token              TokSpace1
%token              TokSpace2
%token              TokSpace3
%token              TokLineUp1
%token              TokLineUp2
%token              TokLineUp3
%token              TokRatio

%token              TokGridList

%token              TokLeft
%token              TokMiddle
%token              TokRight

%token              TokType
%token              TokOpacity
%token              TokContain
%token              TokDraw

%token              TokBlend
%token              TokClose
%token              TokHalf
%token              TokMixed
%token              TokNone
%token              TokOpen
%token              TokWire
%token              TokUp
%token              TokDown

%token              TokFin

%token              TokAxisX
%token              TokAxisY
%token              TokAxisZ

%token              TokAxis1
%token              TokAxis2

%token              TokDef
%token              TokScene
%token              TokLast
%token              TokShow
%token              TokHide
%token              TokReplace
%token              TokPlay
%token              TokExec
%token              TokPause

%token              TokView
%token              TokAuto
%token              TokVertical
%token              TokCenter
%token              TokViewer

%token              TokMove
%token              TokSpin
%token              TokRotation
%token              TokDeltaX
%token              TokDeltaY
%token              TokDeltaZ
%token              TokSpeedX
%token              TokSpeedY
%token              TokSpeedZ




  /*******************/
 /* R?gle de d?part */
/*******************/


%start file
%start std
%type  <unit> file
%type  <bool> std



%%



file:
    definitions scenes TokFin
      {
	clear_parser ()
      }
  | scenes TokFin
      {
	clear_parser ()
      }



std:
    definitions
      {
	clear_parser ();
	true
      }
  | scen
      {
	clear_parser ();
	true
      }
  | command
      {
	clear_parser ();
	true
      }
  | TokFin
      {
	clear_parser ();
	false
      }




  /**********/
 /* Scenes */
/**********/


scenes: 
    TokFin
      { () }
  | scen scenes
      { () }
  | command scenes
      { () }



scen:
    TokScene TokDebAccol scen_body TokFinAccol 
      {
	let tempScene = $3 in
	  tempScene#setAnonym true;
	  core#addScene tempScene
	  (*Scene.add_scene tempScene*)
      }
  | TokScene TokNOM TokDebAccol scen_body TokFinAccol
      {
	$4#setName $2;
	core#addScene $4
	(*Hashtbl.add !Parametres.sceneLst $2 $4*)
	(*Parametres.sceneLst := insertAlpha $2 $4 !Parametres.sceneLst;*)
	(*Scene.add_scene $4*)
      }



scen_body:
  /* Rien */
      {             
	new scene
      }
  
  | objt scen_body
      {
	(*trans_couleur (new couleur 0.2 0.2 0.2 1.) $1;*)
	let newContenu = $1::($2#getContenu)
	in
	  $2#setContenu newContenu;
	  $2
      }




  /*************/
 /* Commandes */
/*************/


command:
    TokShow scene_list
      {
	(*Scene.add_command (Show (List.map (string2scene !Parametres.sceneLst) $2));*)
	
	core#addCommand (Show (List.map (core#getScene) $2));	
	(*If instant loading is set to true force display of newly loaded scenes*)
        tryDisplay ();
      }
  | TokShow TokLast
      {
	(*let lastScene = List.hd !Parametres.dessin in*)
	  core#addCommand (Show ((core#getLastScene)::[]));
	  (*Remove last scene from list, if it has just been anonym*)
	  (*if (lastScene#getAnonym) then
	    Parametres.dessin := List.tl !Parametres.dessin;*)
	
        (*If instant loading is set to true force display of newly loaded scenes*)
        tryDisplay ();
      }
  | TokHide scene_list
      {
	(*Scene.add_command (Hide (List.map (string2scene !Parametres.sceneLst) $2));*)
	core#addCommand (Hide (List.map (core#getScene) $2));	
        (*If instant loading is set to true force display of newly loaded scenes*)
        tryDisplay ();
      }
  | TokView TokDebAccol cmd_view TokFinAccol
      {
	core#addCommand $3;
        (*If instant loading is set to true force display of newly loaded scenes*)
        tryDisplay ();
      }
  | TokReplace TokDebAccol replace TokFinAccol
      {
	core#addCommand $3;
        (*print_string "Added a replace";
	flush stdout;*)
        (*If instant loading is set to true force display of newly loaded scenes*)
        tryDisplay ();
       }
  | TokPause
      {
	core#addCommand Pause;
        (*If instant loading is set to true force display of newly loaded scenes*)
        tryDisplay ();
      }
  | TokExec TokSTRING
      {
	core#addCommand (Command $2)
      }
  | TokMove TokDebAccol mvt TokFinAccol
      {
	let ((tv, td), (rv, rd), (sv, sd)) = $3
	and verifie vitesse delta = let verif d v = if (d <> 0.) && (v = 0.)
						    then raise (ParseError ("la vitesse et l'amplitude du mouvement ne concordent pas"))
				    in
				      verif (abscisse delta) (abscisse vitesse);
				      verif (ordonnee delta) (ordonnee vitesse);
				      verif (cote     delta) (cote     vitesse)
	in
	  verifie tv td;
	  verifie rv rd;
	  verifie sv sd;
	  let move = new mouvement
	  in
	    (move#transSpeed)#setAll tv;
	    (move#transDelta)#setAll td;
	    (move#  rotSpeed)#setAll rv;
	    (move#  rotDelta)#setAll rd;
	    (move# spinSpeed)#setAll sv;
	    (move# spinDelta)#setAll sd;
	    core#addCommand (Movement move)
      }



cmd_view:
    TokAuto
      {
	View (AutoViewer, AutoCenter, AutoVertical)
      }
  | view_body
      {
	let (x, y, z) = $1
	in
	  View (x, y, z)
      }



replace:
    replace_hide TokShow scene_list replace_hide
      {
	Replace ((match ($1, $4)
		  with
		    | None, None       -> None
		    | None, Some l     -> Some l
		    | Some l, None     -> Some l
		    | Some l1, Some l2 -> Some (l1 @ l2))
		 , (List.map (core#getScene) $3))
      }
  | replace_hide TokShow TokLast replace_hide
      {
	Replace ((match ($1, $4)
		  with
		    | None, None       -> None
		    | None, Some l     -> Some l
		    | Some l, None     -> Some l
		    | Some l1, Some l2 -> Some (l1 @ l2))
		 , (core#getLastScene)::[])
      }


replace_hide:
  /* rien */
      {
	None
      }
  | TokHide scene_list
      {
	Some (List.map (core#getScene) $2)
      }



mvt:
  /* rien */
      {
	(((1., 1., 1.), (0., 0., 0.)), ((1., 1., 1.), (0., 0., 0.)), ((1., 1., 1.), (0., 0., 0.)))
      }
  | TokTranslation mvt_body mvt
      {
	let (_, r, s) = $3
	in
	  ($2, r, s)
      }
  | TokRotation    mvt_body mvt
      {
	let (t, _, s) = $3
	in
	  (t, $2, s)
      }
  | TokSpin        mvt_body mvt
      {
	let (t, r, _) = $3
	in
	  (t, r, $2)
      }




  /***************/
 /* D?finitions */
/***************/


definitions:
  TokDef TokDebAccol definitions_body TokFinAccol
      { () }



definitions_body:
  /* rien */
      { () }
  | definition definitions_body
      { () }



definition:
  definition_body
      {
	let (nom, obj) = $1
	in
	  obj#setName nom;
	  core#addObject (obj :> Types.drawable);
	  obj
      }



definition_body:
    def_primitiv        { $1 }
  | def_transform       { $1 }
  | def_group           { $1 }
  | def_grid            { $1 }



def_primitiv:
    def_box             { $1 }
  | def_cone            { $1 }
  | def_cylindre        { $1 }
  | def_disque          { $1 }
  | def_frustum         { $1 }
  | def_sphere          { $1 }
  | def_polygone        { $1 }
  | def_polyline        { $1 }
  | def_spline          { $1 }
  | def_empty           { $1 }
  | def_graph           { $1 } 
  | def_complexGraph    { $1 }

def_transform:
    def_translation     { $1 }
  | def_scale           { $1 }
  | def_rotation        { $1 }


def_grid:
    TokGrid0 def_grid0  { $2 }
  | TokGrid1 def_grid1  { $2 }
  | TokGrid2 def_grid2  { $2 }
  | TokGrid3 def_grid3  { $2 }
  | TokHexa2 def_hexa2  { $2 }
  | TokHexa3 def_hexa3  { $2 }



def_box:
/* D?finition d'une nouvelle Box */
  TokBox TokNOM TokDebAccol box_body
      {
	let (size, color, wire, cont) = $4
	in
	   (*Coerce to Types.primitive to allow polymorphic usage*)
	  let node = ((new box color Parametres.ori wire cont size) :> Types.drawable)
		     (*Primitive (Box size, color, ref wire, cont)*)
	  in
	    ($2, node)
      }



def_cone:
/* D?finition d'un nouveau Cone */
  TokCone TokNOM TokDebAccol cone_body
      {
	let (rayon, hauteur, solide, slice, color, wire, cont) = $4
	in
	  (*Coerce to Types.primitive to allow polymorphic usage*)
	  let node = ((new cone color Parametres.ori wire cont rayon hauteur slice solide) :> Types.drawable)
	            (*Primitive (Cone (ref rayon, ref hauteur, ref solide, ref slice), color, ref wire, cont)*)
	  in
	    ($2, node)
      }



def_cylindre:
/* D?finition d'un nouveau Cylinder */
  TokCylinder TokNOM TokDebAccol cylindre_body
      {
	let (rayon, hauteur, solide, slice, color, wire, cont) = $4
        in
	  (*Coerce to Types.primitive to allow polymorphic usage*)
	  let node = ((new cylinder color Parametres.ori wire cont rayon hauteur slice solide) :> Types.drawable)
	             (*Primitive (Cylinder (ref rayon, ref hauteur, ref solide, ref slice), color, ref wire, cont)*)
	  in
	    ($2, node)
      }



def_disque:
/* D?finition d'un nouveau Disc */
  TokDisc TokNOM TokDebAccol disque_body
      {
	let (rayon, slice, color, wire) = $4
	in
          (*Coerce to Types.primitive to allow polymorphic usage*)
    	  let node = ((new disc color Parametres.ori wire rayon slice) :> Types.drawable)
                     (*Primitive (Disc (ref rayon, ref slice), color, ref wire, None)*)
	  in
	    ($2, node)
      }



def_frustum:
/* D?finition d'un nouveau Frustum */
  TokFrustum TokNOM TokDebAccol frustum_body
      {
	let (rayon, hauteur, rate, solide, slice, color, wire, cont) = $4
	in
          (*Coerce to Types.primitive to allow polymorphic usage*)
	  let node = ((new frustrum color Parametres.ori wire cont rayon hauteur rate slice solide) :> Types.drawable)
	             (*Primitive (Frustum (ref rayon, ref hauteur, ref rate, ref solide, ref slice), color, ref wire, cont)*)
	  in
	    ($2, node)
      }



def_sphere:
/* D?finition d'une nouvelle Sphere */
  TokSphere TokNOM TokDebAccol sphere_body
      {
	let (rayon, slice, stack, color, wire, cont) = $4
	in
	  (*Coerce to Types.primitive to allow polymorphic usage*)
	  let node = ((new sphere color Parametres.ori wire cont rayon slice stack) :> Types.drawable)
	             (*Primitive (Sphere (ref rayon, ref slice, ref stack), color, ref wire, cont)*)
	  in
	    ($2, node)
      }



def_polyline:
/* D?finition d'un nouveau Polyline */
  TokPolyline TokNOM TokDebAccol polyline_body
      {
	let (liste, color, wire) = $4
	in
	  (*Coerce to Types.primitive to allow polymorphic usage*)
	  let node = ((new polyline color wire (ref liste)) :> Types.drawable)
	             (*Primitive (Polyline (ref liste), color, ref wire, None)*)
	  in
	    ($2, node)
      }

def_spline:
/* D?finition d'un nouveau Spline */
  TokSpline TokNOM TokDebAccol spline_body
      {
	let (liste, approxPoints, radius, slices, color, wire) = $4
	in
	  (*Coerce to Types.primitive to allow polymorphic usage*)
	  let node = if (radius = 0.0) then   ((new spline color wire (ref liste) approxPoints) :> Types.drawable)
		     else                     ((new solidSpline color wire (ref liste) approxPoints slices radius) :> Types.drawable)
		       (*let spline = (computeSpline liste approxNum) in
			 if (radius=0.0) then
			 Primitive (Spline (ref spline,ref [[]],ref []), color, ref wire, None)
			 else
			 let polygons=(createSolidSplinePolygons spline slices radius) in
			 Primitive (Spline (ref spline,ref polygons,ref (List.map (fun poly -> (getPolygoneNormale poly)) polygons)), color, ref wire, None)*)	  
	  in
	    ($2, node)
      }

def_polygone:
/* D?finition d'un nouveau Polygone */
  TokPolygone TokNOM TokDebAccol polygone_body
      {
	let (liste, colList, color, wire) = $4
	in
          (*Coerce to Types.primitive to allow polymorphic usage*)
	  let node = ((new polygone color wire (ref liste) (ref colList) true) :> Types.drawable)
		     (*Primitive (Polygone (ref lst, norme), color, ref wire, None)*)
	  in
	    ($2, node)
      }



def_empty:
/* D?finition d'un nouveau Empty */
  TokEmpty TokNOM TokDebAccol empty_body
      {
	let (size, visible, color, wire, cont) = $4
	in
          (*Coerce to Types.primitive to allow polymorphic usage*)
	  let node = ((new empty color Parametres.ori wire cont size visible) :> Types.drawable)
	             (*Primitive (Empty (size, ref visible), color, ref wire, cont)*)
	  in
	    ($2, node)
      }

def_vertex:
/* D?finition d'un nouveau vertex, no primitive, mais utiliser par graph*/
  TokVertex TokNOM TokDebAccol vertex_body
      {
	let (position, color, prim, text) = $4
	in
          (*Coerce to Types.primitive to allow polymorphic usage*)
	  let node = new vertex position color prim $2 text
	             (*Primitive (Empty (size, ref visible), color, ref wire, cont)*)
	  in 
	    Hashtbl.add !Graph.vertexList $2 node;
	    node
	    
      }

def_graph:
/* D?finition d'un nouveau graph */
  TokGraph TokNOM TokDebAccol graph_body
      {
	let (color, wire, vertices, edges, placer, directed ) = $4
	in
          (*Coerce to Types.primitive to allow polymorphic usage*)
	  let graph = new graph color wire (ref vertices) (ref edges) directed
	             (*Primitive (Empty (size, ref visible), color, ref wire, cont)*)
	  in
	    if (placer <> None) then
	      placeGraph graph (unSome placer);
	    ($2, (graph :> Types.drawable))
      }

def_complexGraph:
/* Dfinition d'un complex simplicieau */
  TokComplexGraph TokNOM TokDebAccol complex_graph_body
      {
	let (color, wire, vertices, simplex, placer) = $4
	in
          (*Coerce to Types.primitive to allow polymorphic usage*)
	  let g = (new complexGraph color wire (ref vertices) (ref simplex))
          in 
	    if (placer<>None) then
	      placeComplex g (unSome placer);
	    ($2, (g :> Types.drawable))
      }



def_translation:
/* D?finition d'une nouvelle Translation */
  TokTranslated TokNOM TokDebAccol translation_body
      {
	($2, $4)
      }



def_scale:
/* D?finition d'une nouvelle homoth?tie */
  TokScaled TokNOM TokDebAccol scale_body
      {
	($2, $4)
      }



def_rotation:
/* D?finition d'une nouvelle Rotation */
  TokAxisRotated TokNOM TokDebAccol rotation_body
      {
	let (axis, x, angle, y, obj, z) = $4
	in
	let node = (if x && y && z
		    then ((new transformer (Rotation (axis, angle)) obj) :> Types.drawable)
		    else raise (ParseError "la rotation est incompl?te")
		   )
	in
	  ($2, node)
      }



def_group:
/* D?finition d'un nouveau Group */
  TokGroup TokNOM TokDebAccol group_body
      {
	let node = let (obj, color) = $4
		   in
		     ((new group color (ref obj)):> Types.drawable)
	in
	  ($2, node)
      }



def_grid0:
/* D?finition d'un nouveau Grid0 */
  TokNOM TokDebAccol grid0_body
      {
	let node = let (pt, alignX, alignY, alignZ, coul, lst) = $3
		   in
		     ((new grid0 coul pt (ref lst) alignX alignY alignZ):> Types.drawable)
	in
	  ($1, node)
      }



def_grid1:
/* D?finition d'un nouveau Grid1 */
  TokNOM TokDebAccol grid1_body
      {
	let (pt, axis, align1, align2, space1, coul, lst) = $3
	in
	  let node = ((new grid1 coul pt (ref lst) axis align1 align2 space1):> Types.drawable)
	  in
	    ($1, node)
      }



def_grid2:
/* D?finition d'un nouveau Grid2 */
  TokNOM TokDebAccol grid2_body
      {
	let (pt, axis1, axis2, align1, align2, align3, space1, space2, rate, coul, lst) = $3
        in
	  let node = ((new grid2 coul pt (ref lst) axis1 axis2 align1 align2 align3 space1 space2 rate):> Types.drawable)
          in
	    ($1, node)
	(*($1, Create_grid.create_grid2 $3)*)
      }



def_grid3:
/* D?finition d'un nouveau Grid3 */
  TokNOM TokDebAccol grid3_body
      {
	let (pt, axis1, axis2, align1, align2, align3, space1, space2, space3, r1, r2, r3, coul, lst) = $3
        in 
	  let node = ((new grid3 coul pt (ref lst) axis1 axis2 align1 align2 align3 space1 space2 space3 r1 r2 r3):> Types.drawable)
          in
	    ($1, node)         
(*($1, Create_grid.create_grid3 $3)*)
      }



def_hexa2:
/* D?finition d'un Grid h?xagonal 2D */
  TokNOM TokDebAccol hexa2_body
      {
	let (pt, axis1, axis2, align3, space, rate, coul, lst) = $3
	in
	  let node = ((new hexagrid2 coul pt (ref lst) axis1 axis2 align3 space rate):> Types.drawable)
          in 
	    ($1, node)         	    
	(*($1, Create_grid.create_hexa2 $3)*)
      }



def_hexa3:
/* D?finition d'un Grid h?xagonal 3D */
  TokNOM TokDebAccol hexa3_body
      {
	let (pt, axis1, axis2, space, r1, r2, r3, coul, lst) = $3
	in
	  let node = ((new hexagrid3 coul pt (ref lst) axis1 axis2 space r1 r2 r3):> Types.drawable)
          in 
	    ($1, node)         	    
	(*($1, Create_grid.create_hexa3 $3)*)
      }




/**********/
/* Objets */
/**********/


objt:
    primitiv        { ($1 :>  Types.drawable)}
  | transform       { $1 }
  | group           { $1 }
  | grid            { $1 }
  | definition      { $1 }
  | TokNOM          { copyObject $1 }



primitiv:
    box             { $1 }
  | cone            { $1 }
  | cylindre        { $1 }
  | disque          { $1 }
  | frustum         { $1 }
  | sphere          { $1 }
  | polygone        { $1 }
  | polyline        { $1 }
  | spline          { $1 }
  | empty           { $1 }
  | graph           { $1 }
  | complexGraph    { $1 }
  | label           { $1 }



transform:
    translation     { $1 }
  | scale           { $1 }
  | rotation        { $1 }



grid:
    TokGrid0 grid0  { $2 }
  | TokGrid1 grid1  { $2 }
  | TokGrid2 grid2  { $2 }
  | TokGrid3 grid3  { $2 }
  | TokHexa2 hexa2  { $2 }
  | TokHexa3 hexa3  { $2 }




  /********************/
 /* Objets Primitifs */
/********************/


box:
  TokBox TokDebAccol box_body
      {
	let (size, color, wire, cont) = $3
	in
	  (*Coerce to Types.primitive to allow polymorphic usage*)
	  (new box color Parametres.ori wire cont size) :> Primitive.primitive
          (*Primitive (Box size, color, ref wire, cont)*)
      }



cone:
  TokCone TokDebAccol cone_body
      {
	let (rayon, hauteur, solide, slice, color, wire, cont) = $3
	in
	  (*Coerce to Types.primitive to allow polymorphic usage*)
	  (new cone color Parametres.ori wire cont rayon hauteur slice solide) :> Primitive.primitive
	  (*Primitive (Cone (ref rayon, ref hauteur, ref solide, ref slice), color, ref wire, cont)*)
      }



cylindre:
  TokCylinder TokDebAccol cylindre_body
      {
	let (rayon, hauteur, solide, slice, color, wire, cont) = $3
	in
	  (*Coerce to Types.primitive to allow polymorphic usage*)
	  (new cylinder color Parametres.ori wire cont rayon hauteur slice solide) :> Primitive.primitive
	  (*Primitive (Cylinder (ref rayon, ref hauteur, ref solide, ref slice), color, ref wire, cont)*)
      }



disque:
  TokDisc TokDebAccol disque_body
      { 
	let (rayon, slice, color, wire) = $3
	in
	  (*Coerce to Types.primitive to allow polymorphic usage*)
	  (new disc color Parametres.ori wire rayon slice) :> Primitive.primitive
          (*Primitive (Disc (ref rayon, ref slice), color, ref wire, None)*)
      }



frustum:
  TokFrustum TokDebAccol frustum_body
      {
	let (rayon, hauteur, rate, solide, slice, color, wire, cont) = $3
	in
	  (*Coerce to Types.primitive to allow polymorphic usage*)
	  (new frustrum color Parametres.ori wire cont rayon hauteur rate slice solide) :> Primitive.primitive
	  (*Primitive (Frustum (ref rayon, ref hauteur, ref rate, ref solide, ref slice), color, ref wire, cont)*)
      }



sphere:
  TokSphere TokDebAccol sphere_body
      {
	let (rayon, slice, stack, color, wire, cont) = $3
	in
  	  (*Coerce to Types.primitive to allow polymorphic usage*)
	  (new sphere color Parametres.ori wire cont rayon slice stack) :> Primitive.primitive
	  (*Primitive (Sphere (ref rayon, ref slice, ref stack), color, ref wire, cont)*)
      }



polygone:
  TokPolygone TokDebAccol polygone_body
      {
	let (liste, colList, color, wire) = $3
	in
	  (*Coerce to Types.primitive to allow polymorphic usage*)
	  (new polygone color wire (ref liste) (ref colList) true) :> Primitive.primitive
          (*Primitive (Polygone (ref lst, norme), color, ref wire, None)*)
      }



polyline:
  TokPolyline TokDebAccol polyline_body
      {
	let (liste, color, wire) = $3
	in
	  (*Coerce to Types.primitive to allow polymorphic usage*)
	  (new polyline color wire (ref liste)) :> Primitive.primitive
	  (*Primitive (Polyline (ref liste), color, ref wire, None)*)
      }

spline:
  TokSpline TokDebAccol spline_body
      {
	let (liste, approxNum, radius,slices, color, wire) = $3
        in
	  (*Coerce to Types.primitive to allow polymorphic usage*)
	  if (radius = 0.0) then   ((new spline color wire (ref liste) approxNum) :> Primitive.primitive)
          else                     ((new solidSpline color wire (ref liste) approxNum slices radius) :> Primitive.primitive)
             (*let spline = (computeSpline liste approxNum) in
	       if (radius=0.0) then
	         Primitive (Spline (ref spline,ref [[]],ref []), color, ref wire, None)
	       else
		 let polygons=(createSolidSplinePolygons spline slices radius) in
		   Primitive (Spline (ref spline,ref polygons,ref (List.map (fun poly -> (getPolygoneNormale poly)) polygons)), color, ref wire, None)*)	  
      }

empty:
  TokEmpty TokDebAccol empty_body
      {
	let (size, visible, color, wire, cont) = $3
	in
	  (*Coerce to Types.primitive to allow polymorphic usage*)
	  (new empty color Parametres.ori wire cont size visible) :> Primitive.primitive
	  (*Primitive (Empty (size, ref visible), color, ref wire, cont)*)
      }

graph:
  /* Simple graph */
  TokGraph TokDebAccol graph_body
      {
	let (color, wire, vertices, edges, placer, directed ) = $3
	in
          (*Coerce to Types.primitive to allow polymorphic usage*)
	  let g = (new graph color wire (ref vertices) (ref edges) directed)
          in 
	    if (placer <> None) then
	      placeGraph g (unSome placer);
	    (g :> Primitive.primitive)
      }

complexGraph:
  /* Complex graph */
  TokComplexGraph TokDebAccol complex_graph_body
      {
	let (color, wire, vertices, simplex, placer) = $3
	in
          (*Coerce to Types.primitive to allow polymorphic usage*)
	  let g = (new complexGraph color wire (ref vertices) (ref simplex))
          in 
	    if (placer<>None) then
	      placeComplex g (unSome placer);
	    (g :> Primitive.primitive)
      }

edge:
/* no primitive but needed by graph */
  TokEdge TokDebAccol edge_body
      {
	let (color, wire, v1, v2) = $3
	in
	  new edge color wire v1 v2
      }

simplex:
/* no primitive but needed by complexGraph */
  TokSimplex TokDebAccol simplex_body
      {
	let (color, wire, vertices, label) = $3
	in
	  new simplex color wire (ref vertices) label
      }

label:
  /* Text */
  TokLabel TokDebAccol label_body
      {
	let (color, wire, pos, text) = $3
	in
	  (new label color wire pos text) :> Primitive.primitive
      }

  /******************/
 /* Tansformations */
/******************/


translation:
  TokTranslated TokDebAccol translation_body
      { $3 }



scale:
  TokScaled TokDebAccol scale_body
      { $3 }



rotation:
  TokAxisRotated TokDebAccol rotation_body
      {
	let (axis, x, angle, y, obj, z) = $3
	in
	  if x && y && z then ((new transformer (Rotation (axis, angle)) obj) :> Types.drawable)
	  else raise (ParseError "la rotation est incompl?te")
      }




  /***********/
 /* Groupes */
/***********/


group:
  TokGroup TokDebAccol group_body
      {
	let (obj, color) = $3
	in
	  ((new group color (ref obj)) :> Types.drawable)
      }




  /*********/
 /* Grids */
/*********/


grid0:
  TokDebAccol grid0_body
      {
	let (pt, alignX, alignY, alignZ, coul, lst) = $2
	in
	  ((new grid0 coul pt (ref lst) alignX alignY alignZ) :> Types.drawable)
      }



grid1:
  TokDebAccol grid1_body
      {
	let (pt, axis, align1, align2, space1, coul, lst) = $2
	in
	  ((new grid1 coul pt (ref lst) axis align1 align2 space1) :> Types.drawable)
      }



grid2:
  TokDebAccol grid2_body
      {
	let (pt, axis1, axis2, align1, align2, align3, space1, space2, rate, coul, lst) = $2
        in
	  ((new grid2 coul pt (ref lst) axis1 axis2 align1 align2 align3 space1 space2 rate) :> Types.drawable)
	 (*create_grid.create_grid2 $2*)
      }



grid3:
  TokDebAccol grid3_body
      {
	let (pt, axis1, axis2, align1, align2, align3, space1, space2, space3, r1, r2, r3, coul, lst) = $2
        in 
	  ((new grid3 coul pt (ref lst) axis1 axis2 align1 align2 align3 space1 space2 space3 r1 r2 r3) :> Types.drawable)
	(*Create_grid.create_grid3 $2*)
      }



hexa2:
  TokDebAccol hexa2_body
      {
	let (pt, axis1, axis2, align3, space, rate, coul, lst) = $2
	in
	  ((new hexagrid2 coul pt (ref lst) axis1 axis2 align3 space rate) :> Types.drawable)
        (*Create_grid.create_hexa2 $2*)
      }



hexa3:
  TokDebAccol hexa3_body
      {
	let (pt, axis1, axis2, space, r1, r2, r3, coul, lst) = $2
	in
	  ((new hexagrid3 coul pt (ref lst) axis1 axis2 space r1 r2 r3) :> Types.drawable)
          (*Create_grid.create_hexa3 $2*)
      }




  /************/
 /* Les body */
/************/


box_body:
    TokFinAccol
      {
	(new point 1.0 1.0 1.0), (new couleur 1.0 1.0 1.0 1.0), Close, None
      }
  | TokSize vect box_body
      {
	let (_, x, y, z) = $3
	in
	  ($2, x, y, z)
      }
  | TokColor color box_body
      {
	let (w, _, y, z) = $3
	in
	  (w, $2, y, z)
      }
  | TokDraw draw box_body
      {
	let (w, x, _, z) = $3
	in
	  (w, x, $2, z)
      }
  | TokContain TokDebCrochet objt_list box_body
      {
	let (w, x, y, _) = $4
	in
	  (w, x, y, Some(new container (new point nan nan nan)  1.  (ref $3) ))
      }



cone_body:
    TokFinAccol
      {
	1.0, 1.0, true, 16, (new couleur 1.0 1.0 1.0 1.0), Close, None
      }
  | TokRadius TokNOMBRE cone_body
      {
	let (_, u, v, w, x, y, z) = $3
	in
	  ($2, u, v, w, x, y, z)
      }
  | TokHeight TokNOMBRE cone_body
      {
	let (t, _, v, w, x, y, z) = $3
	in
	  (t, $2 /. 2., v, w, x, y, z)
      }
  | TokSolid TokBOOL cone_body
      {
	let (t, u, _, w, x, y, z) = $3
	in
	  (t, u, $2, w, x, y, z)
      }
  | TokSlices TokNOMBRE cone_body
      {
	let (t, u, v, _, x, y, z) = $3
	in
	  (t, u, v, int_of_float $2, x, y, z)
      }
  | TokColor color cone_body
      {
	let (t, u, v, w, _, y, z) = $3
	in
	  (t, u, v, w, $2, y, z)
      }
  | TokDraw draw cone_body
      {
	let (t, u, v, w, x, _, z) = $3
	in
	  (t, u, v, w, x, $2, z)
      }
  | TokContain TokDebCrochet objt_list cone_body
      {
	let (t, u, v, w, x, y, _) = $4
	in
	  (t, u, v, w, x, y, Some(new container (new point nan nan nan)  1.  (ref $3) ))
      }



cylindre_body:
    TokFinAccol
      {
	1.0, 1.0, true, 16, (new couleur 1.0 1.0 1.0 1.0), Close, None
      }
  | TokRadius TokNOMBRE cylindre_body
      {
	let (_, u, v, w, x, y, z) = $3
	in
	  ($2, u, v, w, x, y, z)
      }
  | TokHeight TokNOMBRE cylindre_body
      {
	let (t, _, v, w, x, y, z) = $3
	in
	  (t, $2 /. 2., v, w, x, y, z)
      }
  | TokSolid TokBOOL cylindre_body
      {
	let (t, u, _, w, x, y, z) = $3
	in
	  (t, u, $2, w, x, y, z)
      }
  | TokSlices TokNOMBRE cylindre_body
      {
	let (t, u, v, _, x, y, z) = $3
	in
	  (t, u, v, int_of_float $2, x, y, z)
      }
  | TokColor color cylindre_body
      {
	let (t, u, v, w, _, y, z) = $3
	in
	  (t, u, v, w, $2, y, z)
      }
  | TokDraw draw cylindre_body
      {
	let (t, u, v, w, x, _, z) = $3
	in
	  (t, u, v, w, x, $2, z)
      }
  | TokContain TokDebCrochet objt_list cylindre_body
      {
	let (t, u, v, w, x, y, _) = $4
	in
	  (t, u, v, w, x, y, Some(new container (new point nan nan nan)  1.  (ref $3) ))
      }



disque_body:
    TokFinAccol
      {
	1., 16, (new couleur 1.0 1.0 1.0 1.0), Close
      }
  | TokRadius TokNOMBRE disque_body
      {
	let (_, x, y, z) = $3
	in
	  ($2, x, y, z)
      }
  | TokSlices TokNOMBRE disque_body
      {
	let (w, _, y, z) = $3
	in
	  (w, int_of_float $2, y, z)
      }
  | TokColor color disque_body
      {
	let (w, x, _, z) = $3
	in
	  (w, x, $2, z)
      }
  | TokDraw draw disque_body
      {
	let (w, x, y, _) = $3
	in
	  (w, x, y, $2)
      }



frustum_body:
    TokFinAccol
      {
	1.0, 1.0, 0.5, true, 16, (new couleur 1.0 1.0 1.0 1.0), Close, None
      }
  | TokRadius TokNOMBRE frustum_body
      {
	let (_, t, u, v, w, x, y, z) = $3
	in
	  ($2, t, u, v, w, x, y, z)
      }
  | TokHeight TokNOMBRE frustum_body
      {
	let (s, _, u, v, w, x, y, z) = $3
	in
	  (s, $2 /. 2., u, v, w, x, y, z)
      }
  | TokTaper TokNOMBRE frustum_body
      {
	let (s, t, _, v, w, x, y, z) = $3
	in
	  (s, t, $2, v, w, x, y, z)
      }
  | TokSolid TokBOOL frustum_body
      {
	let (s, t, u,  _, w, x, y, z) = $3
	in
	  (s, t, u, $2, w, x, y, z)
      }
  | TokSlices TokNOMBRE frustum_body
      {
	let (s, t, u, v, _, x, y, z) = $3
	in
	  (s, t, u, v, int_of_float $2, x, y, z)
      }
  | TokColor color frustum_body
      {
	let (s, t, u, v, w, _, y, z) = $3
	in
	  (s, t, u, v, w, $2, y, z)
      }
  | TokDraw draw frustum_body
      {
	let (s, t, u, v, w, x, _, z) = $3
	in
	  (s, t, u, v, w, x, $2, z)
      }
  | TokContain TokDebCrochet objt_list frustum_body
      {
	let (s, t, u, v, w, x, y, _) = $4
	in
	  (s, t, u, v, w, x, y, Some(new container (new point nan nan nan)  1. (ref $3) ))
      }



sphere_body:
    TokFinAccol
      {
	1.0, 16, 16, (new couleur 1.0 1.0 1.0 1.0), Close, None
      }
  | TokRadius TokNOMBRE sphere_body
      {
	let (_, v, w, x, y, z) = $3
	in
	  ($2, v, w, x, y, z)
      }
  | TokSlices TokNOMBRE sphere_body
      {
	let (u, _, w, x, y, z) = $3
	in
	  (u, int_of_float $2, w, x, y, z)
      }
  | TokStacks TokNOMBRE sphere_body
      {
	let (u, v, _, x, y, z) = $3
	in
	  (u, v, int_of_float $2, x, y, z)
      }
  | TokColor color sphere_body
      {
	let (u, v, w, _, y, z) = $3
	in
	  (u, v, w, $2, y, z)
      }
  | TokDraw draw sphere_body
      {
	let (u, v, w, x, _, z) = $3
	in
	  (u, v, w, x, $2, z)
      }
  | TokContain TokDebCrochet objt_list sphere_body
      {
	let (u, v, w, x, y, _) = $4
	in
	  (u, v, w, x, y, Some(new container (new point nan nan nan)  1.  (ref $3) ))
      }



empty_body:
    TokFinAccol
      {
	(new point 1.0 1.0 1.0), false, (new couleur 1.0 1.0 1.0 1.0), Close, None
      }
  | TokSize vect empty_body
      {
	let (_, w, x, y, z) = $3
	in
	  ($2, w, x, y, z)
      }
  | TokVisible TokBOOL empty_body
      {
	let (v, _, x, y, z) = $3
	in
	  (v, $2, x, y, z)
      }
  | TokColor color empty_body
      {
	let (v, w, _, y, z) = $3
	in
	  (v, w, $2, y, z)
      }
  | TokDraw draw empty_body
      {
	let (v, w, x, _, z) = $3
	in
	  (v, w, x, $2, z)
      }
  | TokContain TokDebCrochet objt_list empty_body
      {
	let (v, w, x, y, _) = $4
	in
	  (v, w, x, y, Some(new container (new point nan nan nan)  1.  (ref $3) ))
      }



polyline_body:
    TokFinAccol
      {
	[], (new couleur 1.0 1.0 1.0 1.0), Close
      }
  | TokPointList TokDebCrochet polylinelist polyline_body
      {
	let (_, y, z) = $4
	in
	  ($3, y, z)
      }
  | TokColor color polyline_body
      {
	let (x, _, z) = $3
	in
	  (x, $2, z)
     }
  | TokDraw draw polyline_body
      {
	let (x, y, _) = $3
	in
	  (x, y, $2)
      }

polygone_body:
    TokFinAccol
      {
	[],[], (new couleur 1.0 1.0 1.0 1.0), Close
      }
  | TokPointList TokDebCrochet polylinelist polygone_body
      {
	let (_, y, z, a) = $4
	in
	  ($3, y, z, a)
      }
  | TokColorList TokDebCrochet colorlist polygone_body
      {
	let (x, _, z, a) = $4
	in
	  (x, $3, z, a)
     }
  | TokColor color polygone_body
      {
	let (x, y, _, a) = $3
	in
	  (x, y, $2, a)
     }
  | TokDraw draw polygone_body
      {
	let (x, y, z, _) = $3
	in
	  (x, y, z, $2)
      }

spline_body:
    TokFinAccol
      {
	[], 5, 0.0, 7, (new couleur 1.0 1.0 1.0 1.0), Close
      }
  | TokPointList TokDebCrochet polylinelist spline_body
      {
	let (_, _, _, _, y, z) = $4
	in
	  ($3, 5, 0.0, 7, y, z)
      }
  | TokRadius TokNOMBRE spline_body
      {
	let (x, y, _, s, c, z) = $3
	in
	  (x, y, $2, s, c, z)
      }
  | TokSlices TokNOMBRE spline_body
      {
	let (x, y, r, _, c, z) = $3
	in
	  (x, y, r, int_of_float $2, c, z)
      }
  | TokApproxPoints TokNOMBRE spline_body
      {
	let (x, _, a, s, y, z) = $3
	in
	  (x, int_of_float $2, a, s, y, z)
      }
  | TokColor color spline_body
      {
	let (x, approxPoints, a, s, _, z) = $3
	in
	  (x, approxPoints, a, s, $2, z)
     }
  | TokDraw draw spline_body
      {
	let (x, _, a, s, y, _) = $3
	in
	  (x, 5, a, s, y, $2)
      }

vertex_body:
    TokFinAccol
      {
	(new point nan nan nan), (new couleur nan nan nan nan), None, None
      }
  | TokPosition vect vertex_body
      {
	let (_ , col , prim, text ) = $3
	in
	  ($2, col, prim, text)
      }  
  | TokColor color vertex_body
      {
	let (pos , _ , prim, text ) = $3
	in
	  (pos, $2, prim, text)
      }
  | TokContain primitiv vertex_body
      {
	let (pos , col, _ , text) = $3
	in
	  (pos, col, Some( $2 ), text)
      }
  | TokLabel TokSTRING vertex_body
      {
	let (pos , col, prim , _) = $3
	in
	  (pos, col, prim, Some($2))
      }

graph_body:
    TokFinAccol
      {
	(new couleur 1.0 1.0 1.0 1.0), Close, [], [],None, false
      }
  | TokColor color graph_body
      {
	let (_ , wire, vertices, edges, placer, directed) = $3
	in
	  ($2, wire, vertices, edges, placer, directed)
      }
  | TokDraw draw graph_body
      {
	let (col , _, vertices, edges, placer, directed) = $3
	in
	  (col, $2, vertices, edges, placer, directed)
      }
  | TokVertexList TokDebCrochet vertexList graph_body
      {
	let (col , wire, _, edges, placer, directed) = $4
	in
	  (col, wire, $3, edges, placer, directed)
      }
  | TokEdgeList TokDebCrochet edgeList graph_body
      {
	let (col , wire, vertices, _, placer, directed) = $4
	in
	  (col, wire, vertices, $3, placer, directed)
      }
  | TokDirected TokBOOL graph_body
      {
	let (col , wire, vertices, edges, placer, _) = $3
	in
	  (col, wire, vertices, edges, placer, $2)
      }
  | TokPlacer placer_body graph_body
      {
	let (col , wire, vertices, edges, _, directed) = $3
	and (name, showCours, placeAll,dim,path) = $2
	in 
	  (col, wire, vertices, edges, Some(Placer(name,showCours,placeAll,dim,path)) , directed)
      }

complex_graph_body:
    TokFinAccol
      {
	(new couleur 1.0 1.0 1.0 1.0), Close, [], [], None
      }
  | TokColor color complex_graph_body
      {
	let (_ , wire, vertices, simplex, placer) = $3
	in
	  ($2, wire, vertices, simplex, placer)
      }
  | TokDraw draw complex_graph_body
      {
	let (col , _, vertices, simplex, placer) = $3
	in
	  (col, $2, vertices, simplex, placer)
      }
  | TokVertexList TokDebCrochet vertexList complex_graph_body
      {
	let (col , wire, _, simplex, placer)  = $4
	in
	  (col, wire, $3, simplex, placer)
      }
  | TokSimplexList TokDebCrochet simplexList complex_graph_body
      {
	let (col , wire, vertices, _, placer) = $4
	in
	  (col, wire, vertices, $3, placer)
      }
  | TokPlacer placer_body complex_graph_body
      {
	let (col , wire, vertices, simplex, _) = $3
	and (name, showCours, placeAll, dim, path) = $2
	in 
	  (col, wire, vertices, simplex, Some(Placer(name,showCours,placeAll,dim,path)))
      }

edge_body:
    TokFinAccol
      {
	(new couleur nan nan nan nan), Close, Graph.dummyVertex, Graph.dummyVertex
      }
  | TokColor color edge_body
      {
	let (_ , wire, v1, v2 ) = $3
	in
	  ($2, wire, v1, v2)
      }
  | TokWire draw edge_body
      {
	let (col , _, v1, v2 ) = $3
	in
	  (col, $2, v1, v2)
      }
  | TokV1 TokNOM edge_body
      {
	let (col , wire, _, v2 ) = $3
	in
	  (* Find appropriate vertex *)
	  let v1 = Hashtbl.find !Graph.vertexList $2
	  in
	    (col, wire, v1, v2)
      }
  | TokV2 TokNOM edge_body
      {
	let (col , wire, v1, _ ) = $3
	in
	  (* Find appropriate vertex *)
	  let v2 = Hashtbl.find !Graph.vertexList $2
	  in
	    (col, wire, v1, v2)
      }

placer_body:
  TokNOM TokDebAccol placer_body 
      {
	let (_, showCours, placeAll, dim, path)= $3
	in
	  ($1, showCours, placeAll, dim, path)
      }
  | TokFinAccol
      {
        "simple", false, false, 2, None
      }
  | TokCours TokBOOL placer_body
      {
	let (name, _, placeAll, dim, path)= $3
	in
	  (name, $2, placeAll, dim, path)
      }
  | TokPlaceAll TokBOOL placer_body
      {
	let (name, showCours, _, dim, path)= $3
	in
	  (name, showCours, $2, dim, path)
      }
  | TokDim TokNOMBRE placer_body
      {
	let (name, showCours, placeAll, _, path)= $3
	in
	  (name, showCours, placeAll, (int_of_float $2), path)
      }
  | TokSave TokSTRING placer_body
      {
	let (name, showCours, placeAll, dim, _)= $3
	in
	  (name, showCours, placeAll, dim, Some($2))
      }

simplex_body:
    TokFinAccol
      {
	(new couleur nan nan nan nan), Close, [], None
      }
  | TokColor color simplex_body
      {
	let (_ , wire, vertices, text ) = $3
	in
	  ($2, wire, vertices, text)
      }
  | TokWire draw simplex_body
      {
	let (col , _, vertices, text ) = $3
	in
	  (col, $2, vertices, text)
      }
  | TokVertexList TokDebCrochet stringList simplex_body
      {
	let (col , wire, _,text ) = $4
	in
	  (* Find appropriate vertices *)
	  let vertices = List.map (fun vertName -> Hashtbl.find !Graph.vertexList vertName) $3
	  in
	    (col, wire, vertices, text)
      }
  | TokLabel TokSTRING simplex_body
      {
	let (col , wire, vertices, _ ) = $3
	in
	  (col, wire, vertices, Some($2))
      }

label_body:
    TokFinAccol
      {
	(new couleur 0.0 0.0 0.0 0.0), Close, (Parametres.getOri ()), "Le label, c est moi"
      }
  | TokColor color label_body
      {
	let (_ , wire, pos, text ) = $3
	in
	  ($2, wire, pos, text)
      }
  | TokWire draw label_body
      {
	let (col , _ , pos, text ) = $3
	in
	  (col, $2, pos, text)
      }
  | TokPosition vect label_body
      {
	let (col , wire , _, text ) = $3
	in
	  (col, wire, $2, text)
      }
  | TokText TokSTRING label_body
      {
	let (col , wire , pos, _ ) = $3
	in
	  (col, wire, pos, $2)
      }


translation_body:
  TokTranslation vect TokGeometry objt TokFinAccol
      {
	((new transformer (Translation $2) (Some($4))) :> Types.drawable)
      }
  | TokGeometry objt TokTranslation vect TokFinAccol
      {
	((new transformer (Translation $4) (Some($2))) :> Types.drawable)
      }



scale_body:
  TokScale vect TokGeometry objt TokFinAccol
      {
	((new transformer (Scale $2) (Some($4))) :> Types.drawable)
      }
  | TokGeometry objt TokScale vect TokFinAccol
      {
	((new transformer (Scale $4) (Some($2))) :> Types.drawable)
      }



rotation_body:
    TokFinAccol
      {
	AxeX, false, 0., false, None, false
      }
  | TokAxis axis rotation_body
      {
	let (_, _, w, x, y, z) = $3
	in
	  ($2, true, w, x, y, z)
      }
  | TokAngle TokNOMBRE rotation_body
      {
	let (u, v, _, _, y, z) = $3
	in
	  (u, v, $2, true, y, z)
      }
  | TokGeometry objt rotation_body
      {
	let (u, v, w, x, _, _) = $3
	in
	  (u, v, w, x, Some($2), true)
      }



group_body:
    TokFinAccol
      {
	[], new couleur 1.0 1.0 1.0 1.
      }
  | TokColor color group_body
      {
	let (x, _) = $3
	in
	  ( x, $2)
      }
  | TokGridList TokDebCrochet objt_list group_body
      {
	let (_, y) = $4
	in
	  ($3, y)
      }



grid0_body:
    TokFinAccol
      {
	new point 0. 0. 0., Left, Left, Left, new couleur 1.0 1.0 1.0 1., []
      }
  | TokStartPoint vect grid0_body
      {
	let (_, v, w, x, y, z) = $3
	in
	  ($2, v, w, x, y, z)
      }
  | TokLineUp1 position grid0_body
      {
	let (u, _, w, x, y, z) = $3
	in
	  (u, $2, w, x, y, z)
      }
  | TokLineUp2 position grid0_body
      {
	let (u, v, _, x, y, z) = $3
	in
	  (u, v, $2, x, y, z)
      }
  | TokLineUp3 position grid0_body
      {
	let (u, v, w, _, y, z) = $3
	in
	  (u, v, w, $2, y, z)
      }
  | TokColor color grid0_body
      {
	let (u, v, w, x, _, z) = $3
	in
	  (u, v, w, x, $2, z)
      }
  | TokGridList TokDebCrochet grid_list grid0_body
      {
	let (u, v, w, x, y, _) = $4
	in
	  (u, v, w, x, y, $3)
      }



grid1_body:
    TokFinAccol
      {
	new point 0. 0. 0., AxeX, Left, Left, 0., new couleur 1.0 1.0 1.0 1., []
      }
  | TokStartPoint vect grid1_body
      {
	let (_, u, v, w, x, y, z) = $3
	in
	  ($2, u, v, w, x, y, z)
      }
  | TokAxis1 axis grid1_body
      {
	let (t, _, v, w, x, y, z) = $3
	in
	  (t, $2, v, w, x, y, z)
      }
  | TokLineUp2 position grid1_body
      {
	let (t, u, _, w, x, y, z) = $3
	in
	  (t, u, $2, w, x, y, z)
      }
  | TokLineUp3 position grid1_body
      {
	let (t, u, v, _, x, y, z) = $3
	in
	  (t, u, v, $2, x, y, z)
      }
  | TokSpace1 TokNOMBRE grid1_body
      {
	let (t, u, v, w, _, y, z) = $3
	in
	  (t, u, v, w, $2, y, z)
      }
  | TokColor color grid1_body
      {
	let (t, u, v, w, x, _, z) = $3
	in
	  (t, u, v, w, x, $2, z)
      }
  | TokGridList TokDebCrochet grid_list grid1_body
      {
	let (t, u, v, w, x, y, _) = $4
	in
	  (t, u, v, w, x, y, $3)
      }



grid2_body:
    TokFinAccol
      {
	new point 0. 0. 0., AxeNone, AxeNone, Middle, Middle, Left, 0., 0., (1., 1., 1.), new couleur 1.0 1.0 1.0 1., []
      }
  | TokStartPoint vect grid2_body
      {
	let (_,q , r, s, t, u, v, w, x, y, z) = $3
	in
	  ($2, q, r, s, t, u, v, w, x, y, z)
      }
  | TokAxis1 axis grid2_body
      {
	let (p, _, r, s, t, u, v, w, x, y, z) = $3
	in
	  (p, $2, r, s, t, u, v, w, x, y, z)
      }
  | TokAxis2 axis grid2_body
      {
	let (p, q, _, s, t, u, v, w, x, y, z) = $3
	in
	  (p, q, $2, s, t, u, v, w, x, y, z)
      }
  | TokLineUp1 position grid2_body
      {
	let (p, q, r, _, t, u, v, w, x, y, z) = $3
	in
	  (p, q, r, $2, t, u, v, w, x, y, z)
      }
  | TokLineUp2 position grid2_body
      {
	let (p, q, r, s, _, u, v, w, x, y, z) = $3
	in
	  (p, q, r, s, $2, u, v, w, x, y, z)
      }
  | TokLineUp3 position grid2_body
      {
	let (p, q, r, s, t, _, v, w, x, y, z) = $3
	in
	  (p, q, r, s, t, $2, v, w, x, y, z)
      }
  | TokSpace1 TokNOMBRE grid2_body
      {
	let (p, q, r, s, t, u, _, w, x, y, z) = $3
	in
	  (p, q, r, s, t, u, $2, w, x, y, z)
      }
  | TokSpace2 TokNOMBRE grid2_body
      {
	let (p, q, r, s, t, u, v, _, x, y, z) = $3
	in
	  (p, q, r, s, t, u, v, $2, x, y, z)
      }
  | TokRatio TokNOMBRE Tok2points TokNOMBRE Tok2points TokNOMBRE grid2_body
      {
	let (p, q, r, s, t, u, v, w, _, y, z) = $7
	in
	  (p, q, r, s, t, u, v, w, ($2, $4, $6), y, z)
      }
  | TokColor color grid2_body
      {
	let (p, q, r, s, t, u, v, w, x, _, z) = $3
	in
	  (p, q, r, s, t, u, v, w, x, $2, z)
      }
  | TokGridList TokDebCrochet grid_list grid2_body
      {
	let (p, q, r, s, t, u, v, w, x, y, _) = $4
	in
	  (p, q, r, s, t, u, v, w, x, y, $3)
      }



grid3_body:
    TokFinAccol
      {
	new point 0. 0. 0., AxeNone, AxeNone, Middle, Middle, Middle, 0., 0., 0., 1., 1., 1., new couleur 1.0 1.0 1.0 1., []
      }
  | TokStartPoint vect grid3_body
      {
	let (_, n, o, p, q, r, s, t, u, v, w, x, y, z) = $3
	in
	  ($2, n, o, p, q, r, s, t, u, v, w, x, y, z)
      }
  | TokAxis1 axis grid3_body
      {
	let (m, _, o, p, q, r, s, t, u, v, w, x, y, z) = $3
	in
	  (m, $2, o, p, q, r, s, t, u, v, w, x, y, z)
      }
  | TokAxis2 axis grid3_body
      {
	let (m, n, _, p, q, r, s, t, u, v, w, x, y, z) = $3
	in
	  (m, n, $2, p, q, r, s, t, u, v, w, x, y, z)
      }
  | TokLineUp1 position grid3_body
      {
	let (m, n, o, _, q, r, s, t, u, v, w, x, y, z) = $3
	in
	  (m, n, o, $2, q, r, s, t, u, v, w, x, y, z)
      }
  | TokLineUp2 position grid3_body
      {
	let (m, n, o, p, _, r, s, t, u, v, w, x, y, z) = $3
	in
	  (m, n, o, p, $2, r, s, t, u, v, w, x, y, z)
      }
  | TokLineUp3 position grid3_body
      {
	let (m, n, o, p, q, _, s, t, u, v, w, x, y, z) = $3
	in
	  (m, n, o, p, q, $2, s, t, u, v, w, x, y, z)
      }
  | TokSpace1 TokNOMBRE grid3_body
      {
	let (m, n, o, p, q, r, _, t, u, v, w, x, y, z) = $3
	in
	  (m, n, o, p, q, r, $2, t, u, v, w, x, y, z)
      }
  | TokSpace2 TokNOMBRE grid3_body
      {
	let (m, n, o, p, q, r, s, _, u, v, w, x, y, z) = $3
	in
	  (m, n, o, p, q, r, s, $2, u, v, w, x, y, z)
      }
  | TokSpace3 TokNOMBRE grid3_body
      {
	let (m, n, o, p, q, r, s, t, _, v, w, x, y, z) = $3
	in
	  (m, n, o, p, q, r, s, t, $2, v, w, x, y, z)
      }
  | TokRatio TokNOMBRE Tok2points TokNOMBRE Tok2points TokNOMBRE grid3_body
      {
	let (m, n, o, p, q, r, s, t, u, _, _, _, y, z) = $7
	in
	  (m, n, o, p, q, r, s, t, u, $2, $4, $6, y, z)
      }
  | TokColor color grid3_body
      {
	let (m, n, o, p, q, r, s, t, u, v, w, x, _, z) = $3
	in
	  (m, n, o, p, q, r, s, t, u, v, w, x, $2, z)
      }
  | TokGridList TokDebCrochet grid_list grid3_body
      {
	let (m, n, o, p, q, r, s, t, u, v, w, x, y, _) = $4
	in
	  (m, n, o, p, q, r, s, t, u, v, w, x, y, $3)
      }



hexa2_body:
    TokFinAccol
      {
	new point 0. 0. 0., AxeNone, AxeNone, Left, 0., (1., 1., 1.), new couleur 1.0 1.0 1.0 1., []
      }
  | TokStartPoint vect hexa2_body
      {
	let (_, t, u, v, w, x, y, z) = $3
	in
	  ($2, t, u, v, w, x, y, z)
      }
  | TokAxis1 axis hexa2_body
      {
	let (s, _, u, v, w, x, y, z) = $3
	in
	  (s, $2, u, v, w, x, y, z)
      }
  | TokAxis2 axis hexa2_body
      {
	let (s, t, _, v, w, x, y, z) = $3
	in
	  (s, t, $2, v, w, x, y, z)
      }
  | TokLineUp3 position hexa2_body
      {
	let (s, t, u, _, w, x, y, z) = $3
	in
	  (s, t, u, $2, w, x, y, z)
      }
  | TokSpace TokNOMBRE hexa2_body
      {
	let (s, t, u, v, _, x, y, z) = $3
	in
	  (s, t, u, v, $2, x, y, z)
      }
  | TokRatio TokNOMBRE Tok2points TokNOMBRE Tok2points TokNOMBRE hexa2_body
      {
	let (s, t, u, v, w, _, y, z) = $7
	in
	  (s, t, u, v, w, ($2, $4, $6), y, z)
      }
  | TokColor color hexa2_body
      {
	let (s, t, u, v, w, x, _, z) = $3
	in
	  (s, t, u, v, w, x, $2, z)
      }
  | TokGridList TokDebCrochet grid_list hexa2_body
      {
	let (s, t, u, v, w, x, y, _) = $4
	in
	  (s, t, u, v, w, x, y, $3)
      }



hexa3_body:
    TokFinAccol
      {
	new point 0. 0. 0., AxeNone, AxeNone, 0., 1., 1., 1., new couleur 1.0 1.0 1.0 1., []
      }
  | TokStartPoint vect hexa3_body
      {
	let (_, s, t, u, v, w, x, y, z) = $3
	in
	  ($2, s, t, u, v, w, x, y, z)
      }
  | TokAxis1 axis hexa3_body
      {
	let (r, _, t, u, v, w, x, y, z) = $3
	in
	  (r, $2, t, u, v, w, x, y, z)
      }
  | TokAxis2 axis hexa3_body
      {
	let (r, s, _, u, v, w, x, y, z) = $3
	in
	  (r, s, $2, u, v, w, x, y, z)
      }
  | TokSpace TokNOMBRE hexa3_body
      {
	let (r, s, t, _, v, w, x, y, z) = $3
	in
	  (r, s, t, $2, v, w, x, y, z)
      }
  | TokRatio TokNOMBRE Tok2points TokNOMBRE Tok2points TokNOMBRE hexa3_body
      {
	let (r, s, t, u, _, _, _, y, z) = $7
	in
	  (r, s, t, u, $2, $4, $6, y, z)
      }
  | TokColor color hexa3_body
      {
	let (r, s, t, u, v, w, x, _, z) = $3
	in
	  (r, s, t, u, v, w, x, $2, z)
      }
  | TokGridList TokDebCrochet grid_list hexa3_body
      {
	let (r, s, t, u, v, w, x, y, _) = $4
	in
	  (r, s, t, u, v, w, x, y, $3)
      }



view_body:
  /* rien */
      {
	AutoViewer, AutoCenter, AutoVertical
      }
  | TokViewer vect view_body
      {
	let (_, y, z) = $3
	in
	  (Viewer $2, y, z)
      }
  | TokCenter vect view_body
      {
	let (x, _, z) = $3
	in
	  (x, Center $2, z)
      }
  | TokVertical vect view_body
      {
	let (x, y, _) = $3
	in
	  (x, y, Vertical $2)
      }



mvt_body:
  /* rien */
      {
	((1., 1., 1.), (0., 0., 0.))
      }
  | TokDeltaX TokNOMBRE mvt_body
      {
	let ((vx, vy, vz), (_, y, z)) = $3
	in
	  ((vx, vy, vz), ($2, y, z))
      }
  | TokDeltaY TokNOMBRE mvt_body
      {
	let ((vx, vy, vz), (x, _, z)) = $3
	in
	  ((vx, vy, vz), (x, $2, z))
      }
  | TokDeltaZ TokNOMBRE mvt_body
      {
	let ((vx, vy, vz), (x, y, _)) = $3
	in
	  ((vx, vy, vz), (x, y, $2))
      }
  | TokSpeedX TokNOMBRE mvt_body
      {
	let ((_, vy, vz), (x, y, z)) = $3
	in
	  (($2, vy, vz), (x, y, z))
      }
  | TokSpeedY TokNOMBRE mvt_body
      {
	let ((vx, _, vz), (x, y, z)) = $3
	in
	  ((vx, $2, vz), (x, y, z))
      }
  | TokSpeedZ TokNOMBRE mvt_body
      {
	let ((vx, vy, _), (x, y, z)) = $3
	in
	  ((vx, vy, $2), (x, y, z))
      }




  /**********/
 /* Listes */
/**********/


scene_list:
    TokNOM TokVirgule scene_list
      { $1::$3 }
  | TokNOM
      { [$1] }



objt_list:
  objt TokVirgule objt_list
      { $1::$3 }
  | objt TokFinCrochet
      { $1::[] }      



grid_list:
  objt TokVirgule grid_list
      { 
	($1, new point 0. 0. 0.)::$3
      }
  | objt TokFinCrochet
      {
	($1, new point 0. 0. 0.)::[]
      }



polylinelist:
  vect TokVirgule polylinelist
      {
	$1::$3
      }
  | vect TokFinCrochet
      {
	$1::[]
      }

colorlist:
  color TokVirgule colorlist
      {
	$1::$3
      }
  | color TokFinCrochet
      {
	$1::[]
      }

edgeList:
  edge TokVirgule edgeList
      {
	$1::$3
      }
  | edge TokFinCrochet
      {
	$1::[]
      }

vertexList:
  def_vertex TokVirgule vertexList
      {
	$1::$3
      }
  | def_vertex TokFinCrochet
      {
	$1::[]
      }

simplexList:
  simplex TokVirgule simplexList
      {
	$1::$3
      }
  | simplex TokFinCrochet
      {
	$1::[]
      }

/* List of strings */
stringList:
  TokNOM TokVirgule stringList
      {
	$1::$3
      }
  | TokNOM TokFinCrochet
      {
	$1::[]
      }

  /*********************/
 /* types secondaires */
/*********************/


vect:
  TokInf TokNOMBRE TokVirgule TokNOMBRE TokVirgule TokNOMBRE TokSup
      {
	new point $2 $4 $6
      }



draw:
    TokBlend TokNOMBRE
      { Blend $2 }
  | TokClose
      { Close }
  | TokHalf
      { Mixed (Close, Shadow) }
  | TokMixed TokDebAccol mixed_body
      { let (d, u) = $3
	in
	  Mixed (d, u)
      }
  | TokNone
      { Shadow }
  | TokOpen
      { Open }
  | TokWire
      { Wire }



mixed_body:
    TokFinAccol
      {
	Shadow, Shadow
      }
  | TokDown draw mixed_body
      {
	let (_, u) = $3
	in
	  ($2, u)
      }
  | TokUp draw mixed_body
      {
	let (d, _) = $3
	in
	  (d, $2)
      }



axis:
    TokAxisX { AxeX }
  | TokAxisY { AxeY }
  | TokAxisZ { AxeZ }



position:
    TokLeft   { Left }
  | TokMiddle { Middle }
  | TokRight  { Right }


color:
  /* Couleur sans transparence */
    TokInf TokNOMBRE TokVirgule TokNOMBRE TokVirgule TokNOMBRE TokSup
      { new couleur $2 $4 $6 1. }
  /* Couleur avec transparence */
  | TokInf TokNOMBRE TokVirgule TokNOMBRE TokVirgule TokNOMBRE TokVirgule TokNOMBRE TokSup
      { new couleur $2 $4 $6 $8 }
