open Types
open Usefull
open Triplet
open Polyline

exception PolygonePlat

class polygone (col : couleur) (wire:draw) (points : point list ref) (colors:couleur list ref) (makeConvex : bool) =
  object (self)
    inherit polyline col wire points as super
    
    (** Members *)
    (** normale of the polygone, necessary for drawing *)
    val mutable normale = getPolygoneNormale !points
    
    (** list of colors, determines the color of each point of the polygone, if the number of colors is smaller
        than the number of points the points left will be colored in standard color. The assignment of the colors
        will be in the same order as the points were given (assignment might not be correct if the points are ordered by
        ordonnePointsPolygone() ) *)
    val mutable colorList = colors

    (*Methods*)
    method drawSelf () = (*(*Set color*)
                         super#setGLColor;
                         (* Front *)
                         GlDraw.begins `polygon;
			 GlDraw.normal3 normale#getAll;
			 List.iter (function pt -> GlDraw.vertex3 (pt#getX, pt#getY, pt#getZ)) !pointList;
			 GlDraw.ends ();
			 (* Back *)
			 GlDraw.begins `polygon;
			 GlDraw.normal3 (Triplet.oppose_triplet normale#getAll);
			 Usefull.revIter (function pt -> GlDraw.vertex3 (pt#getX, pt#getY, pt#getZ)) !pointList;
			 GlDraw.ends ()*)
                         let pList = ref (!pointList) and colList = ref (!colorList) 
                         in
                           (*Set color*)
			   super#setGLColor;
                           (* Front *)
                           GlDraw.begins `polygon;
			   GlDraw.normal3 normale#getAll;
                         
                           while (!pList<>[]) do
			     let col = (List.hd !colList) and pt = (List.hd !pList) 
			     in
			       (* Set color *)
			       GlDraw.color ~alpha:(col#getAlpha) (col#getRouge, col#getVert, col#getBleu);
                               GlLight.material `both (`diffuse (col#getRouge, col#getVert, col#getBleu, col#getAlpha));
			       GlLight.material `both (`ambient (col#getRouge, col#getVert, col#getBleu, col#getAlpha));
			       (* Set vertex *)
			       GlDraw.vertex3 (pt#getX, pt#getY, pt#getZ);
 			       colList := List.tl !colList;
			       pList := List.tl !pList;
                           done;
			    
			   (*List.iter (function pt -> GlDraw.vertex3 (pt#getX, pt#getY, pt#getZ)) !pointList;*)
			   GlDraw.ends ();

                           pList := List.rev !pointList;
                           colList := List.rev !colorList;

			   (* Back *)
			   GlDraw.begins `polygon;
			   GlDraw.normal3 (Triplet.oppose_triplet normale#getAll);

                           while (!pList<>[]) do
			     let col = (List.hd !colList) and pt = (List.hd !pList) 
			     in
			       (* Set color *)
			       GlDraw.color ~alpha:(col#getAlpha) (col#getRouge, col#getVert, col#getBleu);
                               GlLight.material `both (`diffuse (col#getRouge, col#getVert, col#getBleu, col#getAlpha));
			       GlLight.material `both (`ambient (col#getRouge, col#getVert, col#getBleu, col#getAlpha));
			       (* Set vertex *)
			       GlDraw.vertex3 (pt#getX, pt#getY, pt#getZ);
 			       colList := List.tl !colList;
			       pList := List.tl !pList;
                           done;

                           pList := List.rev !pointList;
			   (*Usefull.revIter (function pt -> GlDraw.vertex3 (pt#getX, pt#getY, pt#getZ)) !pointList;*)
			   GlDraw.ends ()
		       
    (* Ordonne la liste des points d'un polygone*)
    method private ordonnePointsPolygone lst = let lst = List.map (fun pt -> pt#getAll) lst
				in
				  try
				    let ptIntern = let rec sum pt size l = match l
									   with
									     | []   -> pt, size
									     | t::q -> sum (add_triplet pt t) (size +. 1.) q
						   in
						   let (pt, size) = sum (0., 0., 0.)  0.  lst
				 		   in
						     div_triplet pt size
				    in
				    let vect1 = normalise_triplet (diff_triplet (List.hd lst) ptIntern)
				    and norme = let rec f l = match l
							      with
								| []   -> raise PolygonePlat
								| t::q -> let n = prod_triplet (diff_triplet (List.hd lst) ptIntern)
											       (diff_triplet t             ptIntern)
								          in
									    match n
									    with
									      | (0., 0., 0.) -> f q
									      | _               -> n
						in
						  normalise_triplet (f (List.tl lst))
				    in
				    let fnAngle ptI = let vectI = normalise_triplet (diff_triplet ptI ptIntern)
						      in
						      let sinus = scalaire (prod_triplet vect1 vectI) norme
						      and cosPositif = (scalaire vect1 vectI) >= 0.
						      in
							if cosPositif
							then sinus
							else 2. -. sinus
				    in
				    let lstAngle = List.map (fun pt -> fnAngle pt, pt) lst
				    in
				    let lstOrdonne = List.sort (fun (theta1, _) -> fun (theta2, _) -> if theta1 < theta2
												      then -1
												      else (if theta1 = theta2
													    then 0
													    else 1
													   )
							       ) lstAngle
				    in

				    let lstConvexe = let testConvexe p1 p2 p3 = (scalaire (prod_triplet (diff_triplet p2 p1)
													(diff_triplet p3 p2))
											  norme
										) > 0.
						     and marque = ref (None:(float * float * float) option)
						     and cycle = let f = new File.file
								 in
								   List.iter (fun (_, pt) -> f#pushEnd pt) lstOrdonne;
								   f
						     in
						     let test cycl = let p1 = cycl#popBegin ()
								     and p2 = cycl#popBegin ()
								     and p3 = cycl#popBegin ()
								     in
								       if testConvexe p1 p2 p3
								       then
									 begin
									   cycl#pushEnd p1;
									   cycl#pushBegin p3;
									   cycl#pushBegin p2;
									   match !marque
									   with
									     | None   -> marque := Some p2;
											 true
									     | Some p -> p <> p3
									 end
								       else
									 begin
									   marque := None;
									   let pEnd = cycl#popEnd ()
									   in
									     cycl#pushBegin p3;
									     cycl#pushBegin p1;
									     cycl#pushBegin pEnd;
									     false
									 end
						     in
						     let rec toList cycl = try
									     let pt = cycl#popBegin ()
									     in
									       pt::(toList cycl)
									   with
									     | File.FileVide -> []
						     in
						       while test cycle
						       do
							 ()
						       done;
						       toList cycle
				    in
(*				      (List.map (fun (_, pt) -> triplet2point pt) lstOrdonne), triplet2point norme*)
				      ref (List.map triplet2point lstConvexe)
				  with
				    | PolygonePlat -> match lst
						      with
							| []    ->  ref []
							| p::[] -> ref ([triplet2point p])
							| t::q  -> let pExtrem = let rec f p n l = match l
												   with
												     | []   -> p
												     | t::q -> let nT = normale_carre_triplet t
													       in
														 if nT > n
														 then f t nT q
														 else f p n q
										 in
										 let n = normale_triplet t
										 in
										   f t n q
								   in
								   let pExtrem2 = let rec f p n l = match l
												    with
												      | []   -> p
												      | t::q -> let nT = normale_carre_triplet (diff_triplet t pExtrem)
														in
														  if nT > n
														  then f t nT q
														  else f p n q
										  in
										  let n = normale_triplet (diff_triplet t pExtrem)
										  in
										    f t n q
								   in
								   let norme = let (x, y, z) = diff_triplet pExtrem2 pExtrem
									       in
										 (-.y, x, 0.)
								   in
								     ref ([triplet2point pExtrem ; triplet2point pExtrem2])

    
    method setPointList newPointList makeConvex = if (makeConvex) then
                                                    pointList <- (self#ordonnePointsPolygone !newPointList)
                                                  else
	                                            pointList <- newPointList;
                                                  normale <- (getPolygoneNormale !newPointList)

    method getNormale = normale

    method getBoundingBox () = self#boundingPolyline

    (** Sets the color of each point according to a list of colors *)
    method setColors colList = (* If the number of colors is smaller than this of the points at standard colors at the end of the list *)
                                 if ((List.length !colList)<>(List.length !points)) then 
				   let tempList = ref [] in
				     for i=1 to ((List.length !points)-(List.length !colList)) do
				       tempList := color::(!tempList);
				     done;
				     colList := !colList @ !tempList;
				     colorList <- colList

    (** Sets the colors of all points equally *)
    method setColor color = super#setColor color;
                            self#setColors (ref (color::[]))

    initializer
      if (makeConvex) then
	pointList <- (self#ordonnePointsPolygone !points)
      else
	pointList <- points;
      normale <- (getPolygoneNormale !points);

      self#setColors colorList;

  end
