open Types
open Viewport

class virtual primitive (col:couleur) (wireVal:draw)= 
  object (self)
    inherit drawable
    (*Members*)
    (*-------*)

    val mutable color = col
    val mutable wire = wireVal
    
    (** draw  object blended *)
    val mutable drawBlend = false

    (** Methods *)
    
    (***********************************************)
    (**Fixe la couleur de l'objet  partir des     
       composantes rouge, vert, bleu et alpha      *)
    (***********************************************)
    (* Type : float * float *float * float -> unit *)
    (***********************************************)
    method setGLColor = let alpha = color#getAlpha and rouge = color#getRouge and vert = color#getVert and bleu = color#getBleu in
                          GlDraw.color ~alpha:alpha (rouge, vert, bleu);
                          GlLight.material `both (`diffuse (rouge, vert, bleu, alpha));
			  GlLight.material `both (`ambient (rouge, vert, bleu, alpha))

    method setColor col = color <- col
    method getColor = color

    method setWire newWire = wire <- newWire;
                             (match wire with
			        | Blend alpha -> drawBlend <- true;
				                 col#setAlpha (col#getAlpha *. alpha)
			        | _           -> () 
			     )
    method getWire = wire

    method getDrawBlend = drawBlend

    (** Draws the structure of the object, i.e. without content *)
    method virtual drawSelf: unit -> unit

    method draw () = if (drawBlend) then
                       begin
			 (* If object should be draw blend, insert it into the list of blended objects, since it has
			    to be drawn after all of the non-blended objects *)
			 let mat = GlMat.get_matrix `modelview_matrix
			 in 
			   let zPos = (Raw.get_float (GlMat.to_raw mat) ~pos:11)
			   in
			     (* Function that will be called for this object when blended objects will be drawn *)
			     let func () = GlMat.load mat;
			                   self#drawSelf();
			     in
			       view#addBlendFunction func zPos
		       end
		     else
		       self#drawSelf()


    initializer
      if (col#getAlpha <> 1.) then drawBlend <- true;
      
      (* Set object blended if drawMode blended is given *)
      (match wire with
	| Blend alpha -> drawBlend <- true;
	                 col#setAlpha (col#getAlpha *. alpha)
	| _           -> ());
  end
