(************************************************)
(*                                              *)
(* Lexer pour l'analyse synthaxique des entres *)
(*                                              *)
(************************************************)

{

open Parser
open Types

let keyword_table = Hashtbl.create 72

(** Keywords for the commands in imoview *)
let _ =  List.iter (fun (kwd, tok) -> Hashtbl.add keyword_table kwd tok)
              [ 
	        ("Translated", TokTranslated );
	        ("Scaled"    , TokScaled );
                ("AxisRotated",TokAxisRotated );

		("Box"     , TokBox );
	        ("Cone"    , TokCone );
	        ("Cylinder", TokCylinder );
	        ("Disc"    , TokDisc );
	        ("Frustum" , TokFrustum );
	        ("Sphere"  , TokSphere );
                ("Polygone", TokPolygone );
	        ("Spline"  , TokSpline );
                ("Polyline", TokPolyline );
	        ("Empty"   , TokEmpty );
                ("Line"    , TokLine );
		("Label"   , TokLabel );

		("Graph"   , TokGraph );
	        ("Edge"    , TokEdge );
	        ("EdgeList", TokEdgeList );
                ("Vertex"  , TokVertex );
	        ("VertexList"  , TokVertexList );
	        ("V1"          , TokV1 ); 
	        ("V2"          , TokV2 ); 
                ("Placer"      , TokPlacer);
                ("directed"    , TokDirected );
                ("PlaceAll"    , TokPlaceAll );
	        ("ShowCours"   , TokCours );
	        ("Dim"         , TokDim );
	        ("SaveTo"      , TokSave );

		("ComplexGraph", TokComplexGraph );
		("Simplex"    , TokSimplex );
		("SimplexList", TokSimplexList );

		( "Group"   , TokGroup );
		( "Grid0"   , TokGrid0 );
		( "Grid1"   , TokGrid1 );
		( "Grid2"   , TokGrid2 );
		( "Grid3"   , TokGrid3 );
		( "HexaGrid2", TokHexa2 );
		( "HexaGrid3", TokHexa3 );

		( "Position", TokPosition ); 
		( "Size"    , TokSize );
		( "Radius"  , TokRadius );
		( "Height"  , TokHeight );
		( "Solid"   , TokSolid );
		( "Slices"  , TokSlices );
		( "ApproxPoints", TokApproxPoints );          
		( "Taper"   , TokTaper );
		( "Stacks"  , TokStacks );
		( "Text"    , TokText );
		( "PointList", TokPointList );
		( "ColorList", TokColorList );


		( "Translation", TokTranslation );
		( "Scale"    , TokScale );
		( "Axis"     , TokAxis );
		( "Angle"    , TokAngle );

		( "Geometry" , TokGeometry );
		( "GridList" , TokGridList );

		( "Visible"  , TokVisible );

		( "StartPoint", TokStartPoint );
		( "Space"    , TokSpace );
		( "Space1"   , TokSpace1 );
		( "Space2"   , TokSpace2 );
		( "Space3"   , TokSpace3 );
		( "LineUp1"  , TokLineUp1 );
		( "LineUp2"  , TokLineUp2 );
		( "LineUp3"  , TokLineUp3 );
		( "Ratio"    , TokRatio );
		
		( "Contain"  , TokContain );
		( "Draw"     , TokDraw );

		( "Open"     , TokOpen );
		( "Half"     , TokHalf );
		( "Blend"    , TokBlend );
		( "Mixed"    , TokMixed );
		( "Wire"     , TokWire );
		( "Close"    , TokClose );
		( "None"     , TokNone );
		( "Up"       , TokUp );
		( "Down"     , TokDown );
		
		( "Color"    , TokColor );

		( "Left"     , TokLeft );
		
		( "Right"    , TokRight );
		
		( "Middle"   , TokMiddle );
		
		( "Axis1"    , TokAxis1 );
		( "Axis2"    , TokAxis2 );
		
		( "AxisX"    , TokAxisX );
		( "AxisY"    , TokAxisY );
		( "AxisZ"    , TokAxisZ );
		
		( "Definitions", TokDef );
		( "Scene"    , TokScene );
		( "LastScene", TokLast );
		( "Show"     , TokShow );
		( "Hide"     , TokHide );
		( "Replace"  , TokReplace );
		( "Play"     , TokPlay );
		( "Exec"     , TokExec );
		( "Pause"    , TokPause );
		
		( "View"     , TokView );
		( "Auto"     , TokAuto );
		( "Vertical" , TokVertical );
		( "Center"   , TokCenter );
		( "Viewer"   , TokViewer );
		
		( "Movement" , TokMove );
		( "Spin"     , TokSpin );
		( "Rotation" , TokRotation );
		( "DeltaX"   , TokDeltaX );
		( "DeltaY"   , TokDeltaY );
		( "DeltaZ"   , TokDeltaZ );
		( "SpeedX"   , TokSpeedX );
		( "SpeedY"   , TokSpeedY );
		( "SpeedZ"   , TokSpeedZ );

	      ]

let getKeyword = (Hashtbl.find keyword_table)

  (*

  | "Translated"             { TokTranslated }
  | "Scaled"                 { TokScaled }
  | "AxisRotated"            { TokAxisRotated }

  | "Box"                    { TokBox }
  | "Cone"                   { TokCone }
  | "Cylinder"               { TokCylinder }
  | "Disc"                   { TokDisc }
  | "Frustum"                { TokFrustum }
  | "Sphere"                 { TokSphere }
  | "Polygone"               { TokPolygone }
  | "Spline"                 { TokSpline }
  | "Polyline"               { TokPolyline }
  | "Empty"                  { TokEmpty }
  | "Line"                   { TokLine }

  | "Graph"                  { TokGraph }
  | "Edge"                   { TokEdge }
  | "EdgeList"               { TokEdgeList }
  | "Vertex"                 { TokVertex }
  | "VertexList"             { TokVertexList }
  | "V1"                     { TokV1 } 
  | "V2"                     { TokV2 } 
  | "Placer"                 { TokPlacer }
  | "directed"               { TokDirected }
  | "PlaceAll"               { TokPlaceAll }
  | "ShowCours"              { TokCours }

  | "Group"                  { TokGroup }
  | "Grid0"                  { TokGrid0 }
  | "Grid1"                  { TokGrid1 }
  | "Grid2"                  { TokGrid2 }
  | "Grid3"                  { TokGrid3 }
  | "HexaGrid2"              { TokHexa2 }
  | "HexaGrid3"              { TokHexa3 }

  | "Position"               { TokPosition } 
  | "Size"                   { TokSize }
  | "Radius"                 { TokRadius }
  | "Height"                 { TokHeight }
  | "Solid"                  { TokSolid }
  | "Slices"                 { TokSlices }
  | "ApproxPoints"           { TokApproxPoints }          
  | "Taper"                  { TokTaper }
  | "Stacks"                 { TokStacks }
  | "PointList"              { TokPointList }
  | "ColorList"              { TokColorList }

  | "Translation"            { TokTranslation }
  | "Scale"                  { TokScale }
  | "Axis"                   { TokAxis }
  | "Angle"                  { TokAngle }

  | "Geometry"               { TokGeometry }
  | "GridList"               { TokGridList }

  | "Visible"                { TokVisible }

  | "StartPoint"             { TokStartPoint }
  | "Space"                  { TokSpace }
  | "Space1"                 { TokSpace1 }
  | "Space2"                 { TokSpace2 }
  | "Space3"                 { TokSpace3 }
  | "LineUp1"                { TokLineUp1 }
  | "LineUp2"                { TokLineUp2 }
  | "LineUp3"                { TokLineUp3 }
  | "Ratio"                  { TokRatio }

  | "Contain"                { TokContain }
  | "Draw"                   { TokDraw }

  | "Open"                   { TokOpen }
  | "Half"                   { TokHalf }
  | "Blend"                  { TokBlend }
  | "Mixed"                  { TokMixed }
  | "Wire"                   { TokWire }
  | "Close"                  { TokClose }
  | "None"                   { TokNone }
  | "Up"                     { TokUp }
  | "Down"                   { TokDown }

  | "Color"                  { TokColor }

  | "Left"                   { TokLeft }

  | "Right"                  { TokRight }

  | "Middle"                 { TokMiddle }

  | "Axis1"                   { TokAxis1 }
  | "Axis2"                   { TokAxis2 }

  | "AxisX"                  { TokAxisX }
  | "AxisY"                  { TokAxisY }
  | "AxisZ"                  { TokAxisZ }

  | "Definitions"            { TokDef }
  | "Scene"                  { TokScene }
  | "LastScene"              { TokLast }
  | "Show"                   { TokShow }
  | "Hide"                   { TokHide }
  | "Replace"                { TokReplace }
  | "Play"                   { TokPlay }
  | "Exec"                   { TokExec }
  | "Pause"                  { TokPause }

  | "View"                   { TokView }
  | "Auto"                   { TokAuto }
  | "Vertical"               { TokVertical }
  | "Center"                 { TokCenter }
  | "Viewer"                 { TokViewer }

  | "Movement"               { TokMove }
  | "Spin"                   { TokSpin }
  | "Rotation"               { TokRotation }
  | "DeltaX"                 { TokDeltaX }
  | "DeltaY"                 { TokDeltaY }
  | "DeltaZ"                 { TokDeltaZ }
  | "SpeedX"                 { TokSpeedX }
  | "SpeedY"                 { TokSpeedY }
  | "SpeedZ"                 { TokSpeedZ }
*)

} 





(******************************)
(* Noms d'objets ou de scnes *)
(******************************)
let name   = ['a'-'z''_'](['0'-'9''a'-'z''A'-'Z''_''-''.'])*

(***********)
(* Nombres *)
(***********)
let number = ['-''+']?['0'-'9']+('.'['0'-'9']*)?

(**************)
(* Identifier *)
(**************)
(*let ident       = ['A'-'Z''a'-'z''\'''?']+['a'-'z''A'-'Z''0'-'9''_''\'''?''!']* 
  | '_'['a'-'z''A'-'Z''0'-'9''_''\'''?''!']+*)
let ident       = ['A'-'Z']+['a'-'z''A'-'Z''0'-'9''_''\'''?''!']* 
  | '_'['a'-'z''A'-'Z''0'-'9''_''\'''?''!']+


(********************)
(* Lexer principale *)
(********************)
rule token = parse

    [' ' '\t' '\r']+         { token lexbuf }
  | '\n'                     { incr Parametres.line ; token lexbuf }
  | ','	                     { TokVirgule }
  | '{'	                     { TokDebAccol }
  | '}'	                     { TokFinAccol }
  | '['	                     { TokDebCrochet }
  | ']'	                     { TokFinCrochet }
  | '<'                      { TokInf }
  | '>'	                     { TokSup }
  | ':'                      { Tok2points }

  

  | "True"                   { TokBOOL(true) }
  | "true"                   { TokBOOL(true) }
  | "False"                  { TokBOOL(false) }
  | "false"                  { TokBOOL(false) }

  (* Les commandes shell  excuter *)
  | '"'[^'"']*'"'            { TokSTRING(String.sub (Lexing.lexeme lexbuf) 1 ((Lexing.lexeme_end lexbuf) - ((Lexing.lexeme_start lexbuf) + 2))) }

  (* Les noms d'objets *)
  | name                     { TokNOM(Lexing.lexeme lexbuf) }

  (* Les nombres *)
  | number                   { TokNOMBRE(float_of_string (Lexing.lexeme lexbuf)) }

  (* Identifier *)
  | ident                    { let result = Lexing.lexeme lexbuf
                               in 
				 try
				   getKeyword result
				 with Not_found -> ID result
			     }

  (* Les commentaires : /* ... */ ou // ... \n ou (* ... *) ou (# ... #) *)
  | "//" | "#"               { commentaire_Ligne lexbuf }
  | "/*"                     { commentaire_C lexbuf }
  | "(*"                     { commentaire_Caml lexbuf }
  | "(#"                     { commentaire_Imoview lexbuf }

  | eof                      { TokFin }




(**************************)
(* Lexers de commentaires *)
(**************************)


(* Commentaires en /* ... */ *)
and commentaire_C = parse
    "*/"                     { token lexbuf }
  | [^'*' '\n']+             { commentaire_C lexbuf }
  | ['*']+[^'*' '/' '\n']+   { commentaire_C lexbuf }
  | ['*']+ '\n'              { incr Parametres.line; commentaire_C lexbuf }
  | '\n'                     { incr Parametres.line; commentaire_C lexbuf }
  | eof                      { TokFin }



(* Commentaires en (* ... *) *)
and commentaire_Caml = parse
    "*)"                     { token lexbuf }
  | [^'*' '\n']+             { commentaire_Caml lexbuf }
  | ['*']+[^'*' ')' '\n']+   { commentaire_Caml lexbuf }
  | ['*']+ '\n'              { incr Parametres.line; commentaire_Caml lexbuf }
  | '\n'                     { incr Parametres.line; commentaire_Caml lexbuf }
  | eof                      { TokFin }



(* Commentaires en (# ... #) *)
and commentaire_Imoview = parse
    "#)"                     { token lexbuf }
  | [^'#' '\n']+             { commentaire_Imoview lexbuf }
  | ['#']+[^'#' ')' '\n']+   { commentaire_Imoview lexbuf }
  | ['#']+ '\n'              { incr Parametres.line; commentaire_Imoview lexbuf }
  | '\n'                     { incr Parametres.line; commentaire_Imoview lexbuf }
  | eof                      { TokFin }



(* Commentaires en # ... \n ou // ... \n *)
and commentaire_Ligne = parse
    [^'\n']                  { commentaire_Ligne lexbuf }
  | '\n'                     { incr Parametres.line; token lexbuf }
  | eof                      { TokFin }
