open Parametres
open Types
open Content
open Viewport
open Group

class sphere (col:couleur) (position:point) (wire:draw) (cont: container option) (radiusVal : float) (slicesVal:int) (stacksVal: int)=
  object (self)
    inherit content col position wire cont as super
    
    (*members*)
    val mutable radius = radiusVal
    val mutable slices = slicesVal
    val mutable stacks = stacksVal
			   
    (*Methods*)
    (*-------*)

    (*Setter & Getter*)
    method setRadius newRadius = radius <-newRadius
    method getRadius = radius
    
    method setSlices newSlices = slices <-newSlices
    method getSlices = slices

    method setStacks newStacks = stacks <- newStacks
    method getStacks = stacks

    method drawSpecific sector drawInt =  let (min, max) = match sector with
                                                            | Inf    -> (1, (stacks+1)/2)
							    | Sup    -> (stacks/2, stacks-1)
							    | All    -> (1, stacks-1)
							    | Bottom -> (1, (stacks+4)/5)
					  in
					    let delta = max - min
					      in
						let cosStack = Array.make (delta + 1) 0.
						and sinStack = Array.make (delta + 1) 0.
						and cosSlice = Array.make slices 0.
						and sinSlice = Array.make slices 0.
						in
						  for i =  0  to delta
						  do
						    let theta = ((pi *. (float_of_int (i+min))) /. (float_of_int stacks)) -. Parametres.pi_2
						    in
						      cosStack.(i) <- cos theta;
						      sinStack.(i) <- sin theta
						  done;
						  (match sector
						   with 
						     | Inf    -> cosStack.(delta) <- 1.;
								 sinStack.(delta) <- 0.
						     | Sup    -> cosStack.(0) <- 1.;
								 sinStack.(0) <- 0.
						     | Bottom -> cosStack.(delta) <- 0.5877852522924731;
								 sinStack.(delta) <- -0.8090169943749474
						     | _      -> ()
						  );
						  for j =  0  to (slices - 1)
						  do
						    let phi = (pi2 *. (float_of_int j)) /. (float_of_int slices)
						    in
						      cosSlice.(j) <- cos phi;
						      sinSlice.(j) <- sin phi
						  done;
						  let drawSol drawInt = GlDraw.begins `triangle_fan;
									normeSens (0., -1., 0.) drawInt;
									GlDraw.vertex3 (0., -.radius, 0.);
									for j = 0 to (slices - 1)
									do
									  normeSens (cosSlice.(j) *. cosStack.(0), sinStack.(0), sinSlice.(j) *. cosStack.(0))
										    drawInt;
									  GlDraw.vertex3 (radius *. cosSlice.(j) *. cosStack.(0), radius *. sinStack.(0), radius *. sinSlice.(j) *. cosStack.(0))
									done;
									normeSens (cosStack.(0), sinStack.(0), 0.) drawInt;
									GlDraw.vertex3 (radius *. cosStack.(0), radius *. sinStack.(0), 0.);
									GlDraw.ends ()

						  and drawToit drawInt = GlDraw.begins `triangle_fan;
									 normeSens (0., 1., 0.) drawInt;
									 GlDraw.vertex3 (0., radius, 0.);
									 normeSens (cosStack.(delta), sinStack.(delta), 0.) drawInt;
									 GlDraw.vertex3 (radius *. cosStack.(delta), radius *. sinStack.(delta), 0.);
									 for j = (slices - 1) downto 0
									 do
									   normeSens (cosSlice.(j) *. cosStack.(delta), sinStack.(delta), sinSlice.(j) *. cosStack.(delta))
										     drawInt;
									   GlDraw.vertex3 (radius *. cosSlice.(j) *. cosStack.(delta), radius *. sinStack.(delta), radius *. cosStack.(delta) *. sinSlice.(j))
									 done;
									 GlDraw.ends ()

						  and drawCote drawInt = GlDraw.begins `quad_strip;
									 for i = 0 to (delta - 1)
									 do
									   for j = 0 to (slices - 1)
									   do
									     normeSens (cosSlice.(j) *. cosStack.(i), sinStack.(i), sinSlice.(j) *. cosStack.(i))
										       drawInt;
									     GlDraw.vertex3 (radius *. cosSlice.(j) *. cosStack.(i), radius *. sinStack.(i), radius *. cosStack.(i) *. sinSlice.(j));
									     normeSens (cosSlice.(j) *. cosStack.(i+1), sinStack.(i+1), sinSlice.(j) *. cosStack.(i+1))
										       drawInt;
									     GlDraw.vertex3 (radius *. cosSlice.(j) *. cosStack.(i+1), radius *. sinStack.(i+1), radius *. cosStack.(i+1) *. sinSlice.(j))
									   done;
									   normeSens (cosStack.(i), sinStack.(i), 0.0) drawInt;
									   GlDraw.vertex3 (radius *. cosStack.(i), radius *. sinStack.(i), 0.0);
									   normeSens (cosStack.(i+1), sinStack.(i+1), 0.0) drawInt;
									   GlDraw.vertex3 (radius *. cosStack.(i+1), radius *. sinStack.(i+1), 0.0)
									 done;
									 GlDraw.ends ()
						  in

						  let sbt = match sector
							    with
							      | Bottom -> (fun drawInt -> drawSol drawInt)

							      | Inf    -> (fun drawInt -> drawSol drawInt;
											  drawCote drawInt)

							      | Sup    -> (fun drawInt -> drawToit drawInt;
											  drawCote drawInt)

							      | All    -> (fun drawInt -> drawSol drawInt;
											  drawToit drawInt;
											  drawCote drawInt)
						  in
						    sbt false;
						    if drawInt
						    then
						      begin
							GlDraw.front_face `cw;
							sbt true;
							GlDraw.front_face `ccw
						      end

    method getBoundingBox () = let rad = ref radius
                               and (minX, minY, minZ, maxX, maxY, maxZ), zoom, trans = self#getContainerBounds
                               and coeffRayonCont cotes = cos (Parametres.pi /. (float_of_int cotes))
                               in
                                 if container = None
				 then
				   begin
				     if !rad == nan
				     then
				       rad := 1.0
				   end
				 else
				   begin
				     let coeff = (coeffRayonCont slices) *. (coeffRayonCont stacks)
				     and diag  = 0.5 *. (sqrt (let dX = maxX -. minX
							       and dY = maxY -. minY
							       and dZ = maxZ -. minZ
							       in
								 dX*.dX +. dY*.dY +. dZ*.dZ))
				     in
				     let defRayon = diag /. coeff
				     in
				       if !rad == nan
				       then
					 rad := defRayon
				       else
					 begin
					   let rR = (abs_float (!rad /. defRayon))
					   in
					     if rR < 1.0
					     then
					       zoom := rR
					 end
				   end;
      
                                   (-. !rad, -. !rad, -. !rad,
				    !rad,    !rad,    !rad)
      
  end

class ellipsoid (col:couleur) (position:point) (wire:draw) (cont: container option) (radiusVal : float) (slicesVal:int) (stacksVal: int) (measureVal:point)=
  object (self)
    inherit sphere col position wire cont radiusVal slicesVal stacksVal as super
    
    (*Members*)
    val mutable measure = measureVal

    (*Methods*)
    (*-------*)

    (*Setters and getters*)
    method setMeasure newMeasure = measure <- newMeasure
    method getMeasure = measure

    method drawSpecific sector drawInt = GlMat.push ();
                                         (*Rescale dimensions and afterwards draw ellipsoid*)
                                         GlMat.scale3 (measure#getX, measure#getY, measure#getZ);
					 super#drawSpecific sector drawInt;
					 GlMat.pop ()
  end
