open Types
open Parametres
open Triplet
open Usefull
open Polyline
open Polygon

class spline (col:couleur) (wire:draw)(points : point list ref) (approxPoints : int)=
  object(self)
    inherit polyline col wire (ref []) as super

    (*Computes a Catmull-Rom spline*)
    method private computeSpline pointList pointNum = let blend i t = match i with
                                                        | -2        -> ((-1.0*.t +. 2.0) *. t -. 1.0) *. t /. 2.0
							| -1        -> (((3.0 *. t -. 5.0) *. t) *. t +. 2.0) /. 2.0
							| 0         -> ((-3.0 *. t +. 4.0) *. t +. 1.0) *. t /. 2.0
							| x         -> ((t -. 1.0) *. t *. t) /. 2.0 in
							  (*Returns an interpolated point*)
                                                          let interpolate dif pointSet = let xp = ref 0.0 and yp = ref 0.0 and zp= ref 0.0 in
							    for j = -2 to 1 do
							      let b = (blend j dif) and head = List.hd !pointSet in
								pointSet := List.tl !pointSet;					   
								xp := !xp +. b *. head#getX;
								yp := !yp +. b *. head#getY;
								zp := !zp +. b *. head#getZ
							      done;
							    new point (!xp) (!yp) (!zp) and
               						  (*Appends the first element at front and the last element at end of the list -> necessary for splines*)
							  appendElements list = ((List.hd list)::list )@ ((getLast list)::[])
							  in
							    let coordList = ref (appendElements pointList) in 
							      let spline    = ref ((interpolate 0. (ref (!coordList)))::[] )
							        and listLength = List.length !coordList in
								  for i = 0 to (listLength - 4) do
								    for j= 1 to pointNum do
								      spline := (interpolate ((float_of_int j)/.(float_of_int pointNum)) (ref (!coordList))) :: (!spline);
								    done;
                              					    coordList:= List.tl !coordList;
								  done;
								    List.rev !spline

    initializer self#setPoints  (self#computeSpline !points approxPoints)
  end



class solidSpline (col:couleur) (wire:draw) (points : point list ref) (approxPoints:int) (slices:int) (radiusVal:float)=
  object(self)
    (*Compute spline*)
    inherit spline col wire points approxPoints as super
    
    (*Members*)
    val mutable polygoneList = ref []
    val mutable radius = radiusVal

    (*Methods*)
    method drawSelf () = List.iter (fun poly -> poly#drawSelf ()) !polygoneList

    (*Compute polygons representing solid spline*)
    method private createSolid spline slices radius = (*creates an initial vector orthogonal to the given with length l*)
                                                      let createInitial vec l = match vec with
						         | p when ((p#getY=0.0) & (p#getZ=0.0)) -> (scaleVec (new point 0.0 2.0 2.0) l)
  						         | p when (p#getZ=0.0) -> (scaleVec (new point 2.0 (-2.0*.p#getX/.p#getY) 2.0) l)
					                 | p -> (scaleVec (new point 2.0 2.0 ((-. 1.0 *. p#getX *. 2.0 -. p#getY *. 2.0)/.p#getZ )) l)
						       in
						         (*recursively generates a list of points that lie in the same plane in the shape of 
						         a circle; axis is a vector of length 1 orthogonal to this plane; vec is the previous 
						         solution, that is iteratively rotated; returns the base for a cylinder*)
					                 let rec createPointList vec axis angle left = 
							   let rotVec = (rotateVec vec axis angle) in
							     match left with
							       | 0 -> []
						               | x -> rotVec::(createPointList rotVec axis angle (x-1)) in
							     (*Generates a list of polygons from a list of points of the base*)
						             let rec generateSection plist1 plist2 = match (plist1,plist2) with
							       | ([],[])                        -> []
							       | (x1::[],x2::[])                -> []
							       | (x1::y1::tail1,x2::y2::tail2)  -> (x1::y1::y2::x2::[])::(generateSection (y1::tail1) (y2::tail2))
							       | (_,_)                          -> failwith "Lists have different sizes!" and
  							     createSolidBase points slices radius = mapPairs 
							                                             (fun p1 p2 -> let axis= unifyVec (difVec p2 p1) in
								                                     (addValToList (createPointList (createInitial axis radius) axis 
									                             (2.0*.pi/.(float_of_int slices)) (slices+1)) p1 )) points and
							     dist p1 p2 = vecLength (difVec p2 p1) and
							     getClosest point list = fold 
							                              (fun x -> x) 
									              (fun p1 p2 -> if ((dist p1 point)<(dist p2 point)) then p1 else p2)
										      (List.hd list)
										      list and
							     (*Reorders a list such that the element of list closest to list1 has position 0*)
						             align point list= let minEl = getClosest point list in
									         (fold 
										     (fun x -> x) 
										     (fun x l -> if (l=[]) then (x::[]) else if ((List.hd l)=minEl) then l else (x::l)) 
										     [] 
										     list ) @ (fold (fun x -> x) (fun x l -> if (x=minEl) then [] else (x::l)) [] list) in
							       let rec align2 list1 list2 = match list1 with
							         | []      -> []
							         | x::tail -> (getClosest x list2)::(align2 tail (deleteElem (getClosest x list2) list2)) and
						               (*Tries to remove the first level of a list by concatenating its elements*)
							        removeFirstLevel list = fold (fun x -> x) (fun x1 x2 -> x1@x2) [] list in
							                                  let baseList = (createSolidBase spline slices radius) in
								                            removeFirstLevel (mapPairs (fun base1 base2 -> generateSection base1 (align2 base1 base2)) baseList)

    method getBoundingBox () = let (minX,minY,minZ,maxX,maxY,maxZ)=super#getBoundingBox ()
                               in
                                 (minX-.radius, minY-.radius,minZ-.radius,maxX+.radius,maxY+.radius,maxZ+.radius)
											      
    initializer 
      (*Compute polygons*)
      polygoneList <- ref (List.map (fun poly -> new polygone col wire (ref poly) (ref []) false) (self#createSolid self#getPoints slices radius))
  end
