open Types
open Parametres
open Boundingbox

class transformer (trafoVal:transformation) (objVal:drawable option) =
  object (self)
    inherit drawable as super
    
    (*---------*)
    (* Members *)
    (*---------*)

    val mutable trafo = trafoVal
    val mutable obj = objVal

    (*---------*)
    (* Methods *)
    (*---------*)
    
    (***********************)
    (* SETTERS AND GETTERS *)
    (***********************)

    method setTrafo newTrafo = trafo <- newTrafo
    method getTrafo = trafo

    method setObject newObject = obj <- newObject
    method getObject = unSome(obj)

    method getBoundingBox () = match trafo
                               with
				   (* Rotation d'angle theta, d'axe axis et de centre O *)
				 | Rotation (axis, theta) -> let (min1, min2, min3, max1, max2, max3) = desordre (self#getObject#getBoundingBox () ) axis (axe_suiv axis)
							     and theta_radian = pi *. theta /. 180.0
				                             in
				                               let cos_theta = cos theta_radian
							       and sin_theta = sin theta_radian
							       in
							         let rot = function x -> function y -> (cos_theta *. x -. sin_theta *. y, sin_theta *. x +. cos_theta *. y)
								 in
								   let (y1, z1) = rot min2 min3
								   and (y2, z2) = rot max2 min3
								   and (y3, z3) = rot max2 max3
								   and (y4, z4) = rot min2 max3
								   in
								     reordre (min1, min_of_4 y1 y2 y3 y4, min_of_4 z1 z2 z3 z4, max1, max_of_4 y1 y2 y3 y4, max_of_4 z1 z2 z3 z4) 
								     axis
								     (axe_suiv axis)

				 (* Homothtie de centre O et de rapport pt#getX sur l'axe X etc... *)
				 | Scale pt -> let (minX, minY, minZ, maxX, maxY, maxZ) = self#getObject#getBoundingBox ()
				               in
				                 bonOrdre (minX *. pt#getX, minY *. pt#getY, minZ *. pt#getZ,
							   maxX *. pt#getX, maxY *. pt#getY, maxZ *. pt#getZ)

				 (* Translation de vecteur pt *)
				 | Translation pt -> let (minX, minY, minZ, maxX, maxY, maxZ) = self#getObject#getBoundingBox ()
						     in
				                       (minX +. pt#getX, minY +. pt#getY, minZ +. pt#getZ,
							maxX +. pt#getX, maxY +. pt#getY, maxZ +. pt#getZ)

    method draw () = GlMat.push ();
                     (match trafo
		       with
			 | Rotation (axis, angle) -> GlMat.rotate3 angle (match axis
			                             with
						       | AxeX    -> (1., 0., 0.)
						       | AxeY    -> (0., 1., 0.)
						       | AxeZ    -> (0., 0., 1.)
						       | AxeNone -> raise (Erreur "Draw : axe de rotation invalide")
									 )
			 | Scale pt               -> GlMat.scale3 pt#getAll
			 | Translation pt         -> GlMat.translate3 pt#getAll
		     );
		     if (obj <> None) then self#getObject#draw ();
		     GlMat.pop ()

  end
