let rec revIter f lst = match lst
			 with
			   | []   -> ()
			   | t::q -> revIter f q;
				     f t



(**********************************)
(* Retourne la valeur arrondie de *)
(* f sous la forme d'un entier.   *)
(**********************************)
(* Type : float -> int            *)
(**********************************)
let round f = let fl = floor f
	      in
	      let i = int_of_float fl
	      in
		if f -. fl < 0.5
		then
		  i
		else
		  i+1



(******************************************)
(* Renvoie le texte comprenant le taux de *)
(* compression et la taille en pixels.    *)
(******************************************)
(* Type : float -> string                 *)
(******************************************)
let rateText rateFloat = let ratePourcent = rateFloat /. 100.0
			 in
			   Printf.sprintf "%.0f %%\n(%dx%d pixels)" rateFloat
									(round (!Parametres.width  *. ratePourcent))
									(round (!Parametres.height *. ratePourcent))



(******************************************)
(* Retourne le nom du fichier destination *)
(* (sans l'extension)  et l'extiontion de *)
(* ce mme fichier.                       *)
(******************************************)
(* Type : string -> string * string       *)
(******************************************)
let splitName str = let n = String.length str
		    in
		    let rec f i str = if str.[i]='.'
				      then i
				      else if i=0
					   then -1
					   else f (i-1) str
		    in
		      if n=0
		      then
			("", "")
		      else
			let point = f (n-1) str
			in
			  if point < 0
			  then
			    (str, "")
			  else
			    (String.sub str 0 point, String.sub str (point+1) (n-(point+1)))

(*Delets a element from a list*)
let rec deleteElem elem list = match list with
  | []                     -> []
  | x::tail when (x=elem)  -> tail
  | x::tail -> x::(deleteElem elem tail);;

(*Higher order functions*)
let rec fold f g e l = match l with
  | []          -> e
  | head::tail  -> g (f head) (fold f g e tail);;

(*Performs a fold to each pair of subsequent elements of a list*)
let rec foldPairs f g e list = match list with
  | []     -> e
  | x::[]  -> e
  | x::y::tail -> g (f x y) (foldPairs f g e (y::tail));;

(*Applys a function to each subsequent pair of elements of a list*)
let rec mapPairs f list = foldPairs f (fun x1 x2 -> x1::x2) [] list;;

(*Returns the last element of a list*)
let rec getLast list = match list with
                                       | x::[] -> x
				       | x::tail -> getLast tail
				       | _ -> failwith "List empty!";;
